
C *********************************************************************
C ***    SUBROUTINES ASSOCIATED WITH COMPOSITE GRAIN MODEL          ***
C        (in sequential calling order)
C
C     *  SUBROUTINE COMPOSITE_GRAIN
C     *  SUBROUTINE ROTATE_2nd
C     *  SUBROUTINE ROTATE_4th
C     *  SUBROUTINE GRAIN_STRESS_CG
C     *  SUBROUTINE NEWTON_RAPHSON_CG
C     *  SUBROUTINE GRAIN_RATE_AND_MODULI_CG
C     *  SUBROUTINE COMPLIANCE_CG
C     *  SUBROUTINE UPDATE_TWINNING_CG
C     *  SUBROUTINE UPDATE_CRSS_CG
C     *  SUBROUTINE TWIN_SHAPE
C *********************************************************************

C ***************************************************************************
C     SUBROUTINE COMPOSITE_GRAIN      -->      VERSION 19/FEB/04
C
C     CALCULATES THE STRESS & STRAIN RATE FOR PARENT AND CHILD. PLUS THE
C     STRESS, STRAIN RATE AND VISCO-PLASTIC MODULUS OF COMPOSITE GRAIN KKK.
C
C     IF ICOMPGRAIN=1 : MIXED STRESS/STRAIN RATE CONTINUITY ACROSS INTERFACE.
C     IF ICOMPGRAIN=2 : STRESS CONTINUITY ACROSS INTERFACE.
C
C     IF INTERX=0 ENFORCES A FC CONDITION ON THE COMPOSITE GRAIN.
C     IF INTERX=1 ENFORCES A SC CONDITION ON THE COMPOSITE GRAIN.
C ***************************************************************************

      SUBROUTINE COMPOSITE_GRAIN (INTERX,KGX,KKK,IPHEL,IPH)

      INCLUDE 'VPSC7.DIM'

      DIMENSION AUX5(5),AUX33(3,3),AUX55(5,5),TRANS(3,3)
      DIMENSION INITIALIZE(NGRMX),SGX(5,NGRMX)

C --> ARRAYS EXPRESSED IN 'PTS' AXES
      COMMON/PTS  /SGPTS(5),DGPTS(5),XMGPTS(5,5),
     #             SG2(5,2),DG2(5,2),XMCSEC2(5,5,2),
     #             SBARPTS(5),DBARPTS(5),XMASTPTS(5,5)
C --> TEMPORARY ARRAYS ASSOCIATED WITH GRAIN SYSTEMS
      COMMON/NLNR2/SCX(5,NSYSMX,2),TAUX(NSYSMX,2),GAMD0X(NSYSMX,2),
     #             NSYSTX(2),NRSX(NSYSMX,2),ISENSEX(NSYSMX,2),W(2),
     #             DB(5),XMASTX(5,5)

C *** IN THE FIRST CALL SETS INITIAL STRESS IN PARENT & CHILD
      IF(INITIALIZE(KKK).EQ.0) THEN
        KG2=KKK+NGR(1)
        DO I=1,5
          SGX(I,KKK)=STRY(I,KGX)
          SGX(I,KG2)=STRY(I,KGX)
        ENDDO
        INITIALIZE(KKK)=1
      ENDIF

C *** BUILDS AXES OF SYSTEM ASSOCIATED WITH TWIN INTERFACE OF 'PTS'.
C *** 'AUX33' TRANSFORMS FROM 'PTS' AXES TO CRYSTAL AXES.
C *** 'AG   ' TRANSFORMS FROM CRYSTAL AXES TO SAMPLE AXES.
C *** 'TRANS' TRANSFORMS FROM SAMPLE AXES TO 'PTS' AXES.

      IF(KTWSMX(KKK).EQ.0) THEN
        DO I=1,3
        DO J=1,3
          TRANS(I,J)=XID3(I,J)
        ENDDO
        ENDDO
      ELSE
        IPTS=ABS(KTWSMX(KKK))
        DO I=1,3
          AUX33(I,1)=DBCA(I,IPTS,IPH)
          AUX33(I,3)=DNCA(I,IPTS,IPH)
        ENDDO
        DO I=1,3
          AUX33(1,2)=AUX33(2,3)*AUX33(3,1)-AUX33(3,3)*AUX33(2,1)
          AUX33(2,2)=AUX33(3,3)*AUX33(1,1)-AUX33(1,3)*AUX33(3,1)
          AUX33(3,2)=AUX33(1,3)*AUX33(2,1)-AUX33(2,3)*AUX33(1,1)
        ENDDO
        DO I=1,3
        DO J=1,3
          TRANS(J,I)=0.
          DO K=1,3
            TRANS(J,I)=TRANS(J,I)+AG(I,K,KKK)*AUX33(K,J)
          ENDDO
        ENDDO
        ENDDO
      ENDIF

C     ROTATE DBAR, SAV, SG, SCH AND XMAST FROM SAMPLE TO PTS AXES

      CALL ROTATE_2nd (DBAR,DBARPTS,TRANS,5)
      CALL ROTATE_2nd (SAV, SBARPTS,TRANS,5)

      DO I=1,5
      DO J=1,5
        XMASTX(I,J)=XMASTPH(I,J,IPH)    ! consistent w interact eq in VPSC7
      ENDDO
      ENDDO
      CALL ROTATE_4th (XMASTX,XMASTPTS,TRANS,5)

      DO IC=1,2
        KG2=KKK+(IC/2)*NGR(1)

        DO I=1,5
          AUX5(I)=SGX(I,KG2)
        ENDDO
        CALL ROTATE_2nd (AUX5,AUX5,TRANS,5)
        DO I=1,5
          SG2(I,IC)=AUX5(I)
        ENDDO

        DO IS=1,NSYST(IPHEL)
          DO I=1,5
            AUX5(I)=SCH(I,IS,KG2)
          ENDDO
          CALL ROTATE_2nd (AUX5,AUX5,TRANS,5)
          DO I=1,5
            SCX(I,IS,IC)=AUX5(I)
          ENDDO
        ENDDO
      ENDDO

      W(1)=WGT(KKK)
      W(2)=WGT(KKK+NGR(1))
      WTOT=W(1)+W(2)
      W(1)=W(1)/WTOT
      W(2)=W(2)/WTOT

c     if(kkk.eq.1) then
c       write(10,*) ' grain #1 before entering GRAIN_STRESS_CG'
c       write(10,'('' w1 & w2 '',2f10.4)') w
c       write(10,'('' sg2     '',5f10.3)') sg2
c       write(10,'('' sav    ='',5f10.3)') sav
c       write(10,'('' dbar   ='',5f10.3)') dbar
c       write(10,'('' xmastx  '',/,(5f10.3))') xmastx
c     endif

C *** SOLVE INTERACTION EQUATION TO OBTAIN STRESS OF COMPOSITE GRAIN
C     'SGPTS', AND ALSO PARENT & CHILD STRESSES 'SG2'

      CALL GRAIN_STRESS_CG (INTERX,KGX,KKK,IPHEL,IPH)

C *** WITH 'SG2' CALCULATE STRAIN RATE OF COMPOSITE GRAIN 'DGPTS', AND
C     PARENT & CHILD RATES 'DG2', SHEARS 'GAMDOT' AND MODULI 'XMCSEC2'

      CALL GRAIN_RATE_AND_MODULI_CG (KGX,KKK,IPHEL,IPH)

C *** WITH PARENT-CHILD MODULI 'XMCSEC2' CALCULATES SECANT MODULI 'XMGPTS'
C     OF COMPOSITE GRAIN

c     if(kkk.eq.1) then
c       write(*,*) 'write for grain #1 entering COMPLIANCE_CG'
c       write(*,'('' xmcsec1 '',/,(5f10.3))')
c    #             ((xmcsec2(i,j,1),j=1,5),i=1,5)
c       write(*,'('' xmcsec2 '',/,(5f10.3))')
c    #             ((xmcsec2(i,j,2),j=1,5),i=1,5)
c     endif
      
      IF(ICOMPGRAIN.EQ.1) THEN
        CALL COMPLIANCE_CG (NRS(1,1),W,XMCSEC2,XMGPTS)
      ELSE IF(ICOMPGRAIN.EQ.2) THEN
        DO I=1,5
        DO J=1,5
          XMGPTS(I,J)=W(1)*XMCSEC2(I,J,1)+W(2)*XMCSEC2(I,J,2)    ! eq (90-1)
        ENDDO
        ENDDO
      ENDIF

c     if(kkk.eq.1) then
c       write(*,*) 'write for grain #1 leaving COMPLIANCE_CG'
c       write(*,'('' xmgpts  '',/,(5f10.3))')
c    #             ((xmgpts(i,j),j=1,5),i=1,5)
c       pause
c     endif

C *** ROTATE DG, SG, AND XMGSEC OF THE COMPOSITE GRAIN TO SAMPLE AXES

      DO I=1,3
      DO J=1,3
        AUX33(J,I)=TRANS(I,J)
      ENDDO
      ENDDO

      CALL ROTATE_2nd (DGPTS,AUX5,AUX33,5)
        DO I=1,5
         DG(I,KGX)=AUX5(I)
        ENDDO

      CALL ROTATE_2nd (SGPTS,AUX5,AUX33,5)
        DO I=1,5
         SG(I,KKK)=AUX5(I)
        ENDDO

      CALL ROTATE_4th (XMGPTS,AUX55,AUX33,5)
        DO I=1,5
        DO J=1,5
          XMCTG (I,J,KGX)=AUX55(I,J)    ! this Msec for secant, neff, tangent
        ENDDO
        ENDDO

      DO IC=1,2
        KG2=KKK+(IC/2)*NGR(1)
        DO I=1,5
          AUX5(I)=SG2(I,IC)
        ENDDO
        CALL ROTATE_2nd (AUX5,AUX5,TRANS,5)
        DO I=1,5
          SGX(I,KG2)=AUX5(I)
        ENDDO
      ENDDO

      RETURN
      END
C
C ***************************************************************************
C     SUBROUTINE ROTATE_2nd     --->    VERSION 20/DEC/02
C
C     TRANSFORMS 2nd ORDER TENSOR FROM VECTOR TO CARTESIAN, ROTATES USING
C     MATRIX 'A', AND TRANSFORMS BACK TO VECTOR NOTATION
C ***************************************************************************

      SUBROUTINE ROTATE_2nd (VEC,VECROT,A,KDIM)

      DIMENSION VEC(5),VECROT(5),A(3,3)
      DIMENSION AUX33(3,3),AUX55(5,5),AUX3333(3,3,3,3),AUXr(3,3)

      CALL CHG_BASIS(VEC,AUX33,AUX55,AUX3333,1,KDIM)

      DO I=1,3
      DO J=1,3
        AUXr(I,J)=0.
        DO I1=1,3
        DO J1=1,3
          AUXr(I,J)=AUXr(I,J)+A(I,I1)*A(J,J1)*AUX33(I1,J1)
        ENDDO
        ENDDO
      ENDDO
      ENDDO

      CALL CHG_BASIS(VECROT,AUXr,AUX55,AUX3333,2,KDIM)

      RETURN
      END
C
C ***************************************************************************
C     SUBROUTINE ROTATE_4th     --->    VERSION 20/DEC/02
C
C     TRANSFORMS 4th ORDER TENSOR FROM MATRIX TO CARTESIAN, ROTATES USING
C     MATRIX 'A', AND TRANSFORMS BACK TO MATRIX NOTATION
C ***************************************************************************

      SUBROUTINE ROTATE_4th (MAT,MATROT,A,KDIM)

      DIMENSION MAT(5,5),MATROT(5,5),A(3,3)
      DIMENSION AUX5(5),AUX33(3,3),AUX3333(3,3,3,3),AUXr(3,3,3,3)
      REAL      MAT,MATROT

      CALL CHG_BASIS(AUX5,AUX33,MAT,AUX3333,3,KDIM)

      DO I=1,3
      DO J=1,3
      DO K=1,3
      DO L=1,3
        AUXr(I,J,K,L)=0.
        DO I1=1,3
        DO J1=1,3
        DO K1=1,3
        DO L1=1,3
          AUXr(I,J,K,L)=AUXr(I,J,K,L)+
     #      A(I,I1)*A(J,J1)*A(K,K1)*A(L,L1)*AUX3333(I1,J1,K1,L1)
        ENDDO
        ENDDO
        ENDDO
        ENDDO
      ENDDO
      ENDDO
      ENDDO
      ENDDO

      CALL CHG_BASIS(AUX5,AUX33,MATROT,AUXr,4,KDIM)

      RETURN
      END
C
C *****************************************************************************
C     SUBROUTINE GRAIN_STRESS_CG   --->   VERSION OF feb/19/2004
C
C     GIVEN A GUESS STRESS 'X(I)' AND GRAIN INDEX 'KKK', SOLVES INTERACTION
C     EQUATION TO FIND THE STRESS 'X' COMPATIBLE WITH MACROSCOPIC COMPLIANCE.
C *****************************************************************************

      SUBROUTINE GRAIN_STRESS_CG (INTERX,KGX,KKK,IPHEL,IPH)

      INCLUDE 'VPSC7.DIM'

      DIMENSION X(8),XORI(8)

C --> ARRAYS EXPRESSED IN 'PTS' AXES
      COMMON/PTS  /SGPTS(5),DGPTS(5),XMGPTS(5,5),
     #             SG2(5,2),DG2(5,2),XMCSEC2(5,5,2),
     #             SBARPTS(5),DBARPTS(5),XMASTPTS(5,5)
C --> TEMPORARY ARRAYS ASSOCIATED WITH GRAIN SYSTEMS
      COMMON/NLNR2/SCX(5,NSYSMX,2),TAUX(NSYSMX,2),GAMD0X(NSYSMX,2),
     #             NSYSTX(2),NRSX(NSYSMX,2),ISENSEX(NSYSMX,2),W(2),
     #             DB(5),XMASTX(5,5)


C *** EMPIRIC ALGORITHM TO GET TAULIM FOR NR SUBROUTINE (RL: 01/FEB/00)
C *** APPARENTLY UNNECESSARY IF NORMALIZE_2 IS USED (CT: APRIL/00)
      taulim=2.*(tnorm(dbar,5,1)/gamd0g(1))**(1./nrsmin)

      iprint=0   ! controls diagnostic print-out
      ngg=699
      iprx=iprint*(ngg/kkk)*(kkk/ngg)+
     #     iprint*((ngg+ngr(1))/kkk)*(kkk/(ngg+ngr(1)))
      if(iprx.eq.1) then
        write(10,'('' INSIDE GRAIN_STRESS_CG: GRAIN'',i5 )') KKK
        write(10,'('' ***  TAULIM='',f10.3)') taulim
      endif

      IF(ICOMPGRAIN.EQ.1) NDIME=8
      IF(ICOMPGRAIN.EQ.2) NDIME=5

C *** COPY MAIN ARRAYS INTO AUXILIAR ARRAYS FOR COMPUTATIONAL EFFICIENCY
C     AND TO MAKE 'NEWTON_RAPHSON' A 'STAND-ALONE' SUBROUTINE.
C *** ALGORITHM BELOW SPECIFIC FOR PHASE1 (PARENT) + PHASE2 (CHILD), BUT
C     EACH PHASE MAY HAVE DIFFERENT SYSTEMS.

      DO IC=1,2
        KG2=KKK+IC/2*NGR(1)
        NSYSTX(IC)=NSYST(IC)
        DO IS=1,NSYSTX(IC)
          ISENSEX(IS,IC)=ISENSE(IS,IC)
          NRSX(IS,IC)   =NRS(IS,IC)
          TAUX(IS,IC)   =CRSS(IS,KG2)
          GAMD0X(IS,IC)=GAMD0G(KG2)
        ENDDO
      ENDDO

C *** INDEPENDENT STRESS COMPONENTS ARE A FUNCTION OF BASIS USED AND THE
C     DEFINITION OF THE INTERFACE AXES.
C     THEY ALSO DEPEND ON CONTINUITY CONDITION ENFORCED ACROSS INTERFACE.

      DO I=1,5
        X(I)=SG2(I,1)
      ENDDO
      IF(ICOMPGRAIN.EQ.1) THEN
        X(6)=SG2(1,2)
        X(7)=SG2(2,2)
        X(8)=SG2(5,2)
      ENDIF

      if(iprx.eq.1) then
        write(10,'('' SG2 1 ='',5F10.3)') (SG2(I,1),I=1,5)
        write(10,'('' SG2 2 ='',5F10.3)') (SG2(I,2),I=1,5)
c       write(10,'('' CRSS 1='',6F10.3)') (TAUX(IS,1),IS=1,NSYSTX(1))
c       write(10,'('' CRSS 2='',6F10.3)') (TAUX(IS,2),IS=1,NSYSTX(2))
      endif

C     MAKE SURE STRESS 'X' IS NOT OUTSIDE THE YIELD SURFACE.
C     CORRECTS THE NORM IF IT EXCEEDS THE YIELD SURFACE.

      TAUMAX=0.
      DO IC=1,2
        DO IS=1,NSYSTX(IC)
          RSSX=SCX(1,IS,IC)*SG2(1,IC)+SCX(2,IS,IC)*SG2(2,IC)+
     #         SCX(3,IS,IC)*SG2(3,IC)+SCX(4,IS,IC)*SG2(4,IC)+
     #         SCX(5,IS,IC)*SG2(5,IC)
          IF(.NOT.(RSSX.GT.0 .OR. ISENSEX(IS,IC).EQ.1)) RSSX=0.
          RSSX=RSSX/TAUX(IS,IC)
          IF(ABS(RSSX).GT.TAUMAX) THEN
            TAUMAX=ABS(RSSX)
            ICX=IC
          ENDIF
        ENDDO
      ENDDO

      IF(TAUMAX. LT. 1.E-6) THEN
        WRITE(*,'('' TAUMAX<1e-6 inside GRAIN_STRESS_CG'')')
        WRITE(*,'('' GRAIN #'',i5)')     KKK+(ICX/2)*NGR(1)
        WRITE(*,'('' STRY  ='',5E12.4)') (SG2(I,ICX),I=1,5)
        WRITE(*,'('' CRSS  ='',6E12.4)') (TAUX(IS,ICX),IS=1,NSYSTX(ICX))
        STOP
      ENDIF

C      IF(TAUMAX.GT.TAULIM) THEN   ! TEMPORARY: START FROM SCYS ALWAYS
        DO I=1,NDIME
          X(I)=X(I)/TAUMAX
        ENDDO
C      ENDIF

C ******************************************************************

      IF(INTERX.EQ.0) THEN
        DO I=1,5
          DB(I)=DBARPTS(I)
          DO J=1,5
            XMASTX(I,J)=0.
          ENDDO
        ENDDO
      ELSE IF(INTERX.GT.0) THEN
        DO I=1,5
        DO J=1,5
          XMASTX(I,J)=XMASTPTS(I,J)
        ENDDO
        ENDDO
        DO I=1,5
          DB(I)=DBARPTS(I)       ! should I use dast and sast ??
          DO J=1,5
            DB(I)=DB(I)+XMASTX(I,J)*SBARPTS(J)
          ENDDO
        ENDDO
      ENDIF

C *** CALLS NEWTON-RAPHSON TO CALCULATE STRESS IN COMPOSITE GRAIN

      DO I=1,NDIME
        XORI(I)=X(I)
      ENDDO
      ITMX=100
      EPS=5.e-04

c     if(kkk.eq.1) then
c       write(*,*) 'state of grain #1 before entering N-R'
c       write(*,'('' dbarpts='',5f10.3)') dbarpts
c       write(*,'('' sbarpts='',5f10.3)') sbarpts
c       write(*,'('' xmastpts'',/,(5f10.3))') xmastpts
c       pause
c     endif

      CALL NEWTON_RAPHSON_CG (X,ITMX,EPS,TAULIM,IERROR, kkk)

      IF(IERROR.GT.0) THEN
        IF(IERROR.EQ.1) WRITE(*,'('' SINGULAR SYST IN NEWT_RAPH_CG ->'',
     #       '' CANNOT SOLVE GRAIN'',I6,'' IN PHASE'',I6)') KKK,IPH
        IF(IERROR.EQ.2) THEN
          WRITE(*,'('' ITMAX REACHED IN NEWT_RAPH_CG ->'',
     #       '' CANNOT SOLVE GRAIN'',I6,'' IN PHASE'',I6)') KKK,IPH
          WRITE(*,'('' INPUT STRESS IS RETAINED & CG IS UNCOUPLED'',/)')
          NUNCOUPLED=NUNCOUPLED+1
          WRITE(10,'('' UNCOUPLED CG='',I5,I10,'' UNCOUPLED SO FAR'')')
     #                                  KKK,NUNCOUPLED
          ICOMPGR(KKK)=0
          ICOMPGR(KKK+NGR(1))=0
        ENDIF

        DO I=1,NDIME
          X(I)=XORI(I)
        ENDDO
      ENDIF

C *** STRESS OF PARENT, CHILD AND COMPOSITE GRAIN

      SG2(1,1)=X(1)
      SG2(2,1)=X(2)
      SG2(3,1)=X(3)
      SG2(4,1)=X(4)
      SG2(5,1)=X(5)
      IF(ICOMPGRAIN.EQ.1) THEN
        SG2(1,2)=X(6)
        SG2(2,2)=X(7)
        SG2(3,2)=X(3)
        SG2(4,2)=X(4)
        SG2(5,2)=X(8)
      ELSE IF(ICOMPGRAIN.EQ.2) THEN
        SG2(1,2)=X(1)
        SG2(2,2)=X(2)
        SG2(3,2)=X(3)
        SG2(4,2)=X(4)
        SG2(5,2)=X(5)
      ENDIF

      DO I=1,5
        SGPTS(I)=W(1)*SG2(I,1)+W(2)*SG2(I,2)
      ENDDO

      if(iprx.eq.1) then
        write(10,'(''stresses in grain'',i5,'' leaving NR'')') kkk
        write(10,'('' sg1 ='',5f10.3)') (sg2(i,1),i=1,5)
        write(10,'('' sg2 ='',5f10.3)') (sg2(i,2),i=1,5)
c       write(10,'('' spts='',5f10.3)') (sgpts(i),i=1,5)
      endif

      RETURN
      END
C
C ***************************************************************************
C
C     SUBROUTINE NEWTON_RAPHSON_CG   --->   VERSION 19/FEB/2004
C
C     GIVEN AN INPUT GRAIN STRESS 'X' SOLVES THE VISCOPLASTIC EQUATION
C     USING NEWTON-RAPHSON LINEARIZATION AND ITERATING UNTIL CONVERGENCE.
C     - Fi(X)   : ARE THE FUNCTIONS TO MINIMIZE.
C     - FGRADij : ARE THE DERIVATIVES OF Fi WITH RESPECT TO Xj.
C     CORRECTIONS TO 'X' AND RELAXATION OF TOLERANCE ELIMINATED (FEB/2000).
C     PASSING ALL RELEVANT CRYSTAL ARRAYS THROUGH COMMON (13/DEC/02).
C
C ***************************************************************************

      SUBROUTINE NEWTON_RAPHSON_CG (X,KMAX,EPS,TAULIM,IERROR, kgx)

      INCLUDE 'VPSC7.DIM'

      DIMENSION F1(5),F2(5),FGRAD1(5,5),FGRAD2(5,5)
      DIMENSION F(8),FGRAD(8,8),FGRADX(5,5),X(8),XOLD(8)

      DIMENSION RSS(NSYSMX,2),GD(NSYSMX,2)

C --> TEMPORARY ARRAYS ASSOCIATED WITH GRAIN SYSTEMS
      COMMON/NLNR2/SCX(5,NSYSMX,2),TAUX(NSYSMX,2),GAMD0X(NSYSMX,2),
     #             NSYSTX(2),NRSX(NSYSMX,2),ISENSEX(NSYSMX,2),W(2),
     #             DB(5),XMASTX(5,5)

      iprint=0   ! controls diagnostic print-out
      ngg=699
      iprx=iprint*(ngg/kgx)*(kgx/ngg)+
     #     iprint*((ngg+ngr(1))/kgx)*(kgx/(ngg+ngr(1)))

C     COEF=0.5      ! temporary, to control convergence
      IERROR=0

      IF(ICOMPGRAIN.EQ.1) NDIME=8
      IF(ICOMPGRAIN.EQ.2) NDIME=5

      if(iprx.eq.1) write(10,'(''   INSIDE NEWTON_RAPHSON_CG'')')

      DO 1000 K=1,KMAX

C *** RESOLVED SHEARS CALCULATION & OUTSIDE Y.S. ERROR MANAGING BLOCK.
C *** NRS MAY BE EVEN OR ODD:
C     GD ALWAYS > 0 --> GD IS USED TO GET DERIVATIVES TO BUILD THE
C     COEFFICIENT MATRIX FOR N-R METHOD IN F(I) CALCULATION (INDEPENDENT
C     TERM FOR N-R METHOD)   RSS*GD=GAMDOT -> SIGN(RSS*GD)=SIGN(RSS)

      DO IC=1,2
        IC5=IC/2*5
        IC3=IC/2*3
        DO IS=1,NSYSTX(IC)
          IF(ICOMPGRAIN.EQ.1) THEN
            RSSX=X(1+IC5)*SCX(1,IS,IC)+X(2+IC5)*SCX(2,IS,IC)+
     #           X(3    )*SCX(3,IS,IC)+X(4    )*SCX(4,IS,IC)+
     #           X(5+IC3)*SCX(5,IS,IC)
          ELSE IF(ICOMPGRAIN.EQ.2) THEN
            RSSX=X(1)*SCX(1,IS,IC)+X(2)*SCX(2,IS,IC)+
     #           X(3)*SCX(3,IS,IC)+X(4)*SCX(4,IS,IC)+
     #           X(5)*SCX(5,IS,IC)
          ENDIF

          IF(.NOT.(RSSX.GT.0 .OR. ISENSEX(IS,IC).EQ.1)) RSSX=0.
          IF(ABS(RSSX).LT.1.E-10) RSSX=0.
          RSS(IS,IC)=RSSX/TAUX(IS,IC)

          GD(IS,IC)=GAMD0X(IS,IC)
          IF(NRSX(IS,IC).NE.1)
     #         GD(IS,IC)=GD(IS,IC)*ABS(RSS(IS,IC)**(NRSX(IS,IC)-1))

          IF(ABS(RSS(IS,IC)).GT.TAULIM) THEN
            DO I=1,NDIME
              X(I)=XOLD(I)+0.2*(X(I)-XOLD(I))   ! replaced COEF by 0.2
            ENDDO
            GO TO 1000
          ENDIF
        ENDDO
      ENDDO

c     if(iprx.eq.1) then
c       write(10,'('' X(I) ='',8F10.3)') (X(I),I=1,NDIME)
c       write(10,'('' SCX 1='',5F10.3)')
c    #              ((SCX(I,IS,1),I=1,5),IS=1,NSYSTX(1))
c       write(10,'('' RRSS 1='',6F10.3)') (RSS(IS,1),IS=1,NSYSTX(1))
c       write(10,'('' NRSX 1='',12I5  )') (NRSX(IS,1),IS=1,NSYSTX(1))
c       write(10,'('' GAMD01='',6F10.3)') (GAMD0X(IS,1),IS=1,NSYSTX(1))
c       write(10,'('' GD   1='',6F10.4)') ( GD(IS,1),IS=1,NSYSTX(1))
c       write(10,'('' SCX 2='',5F10.3)')
c    #              ((SCX(I,IS,2),I=1,5),IS=1,NSYSTX(2))
c       write(10,'('' RRSS 2='',6F10.3)') (RSS(IS,2),IS=1,NSYSTX(2))
c       write(10,'('' NRSX 2='',12I5  )') (NRSX(IS,2),IS=1,NSYSTX(2))
c       write(10,'('' GAMD02='',6F10.3)') (GAMD0X(IS,2),IS=1,NSYSTX(2))
c       write(10,'('' GD   2='',6F10.4)') ( GD(IS,2),IS=1,NSYSTX(2))
c     endif

C *** DEFINE PARENT & CHILD INDIVIDUAL RATES AND THEIR DERIVATIVES

        DO I=1,5
          F1(I)=0.
          DO IS=1,NSYSTX(1)
            F1(I)=F1(I)+SCX(I,IS,1)*RSS(IS,1)*GD(IS,1)
          ENDDO
        ENDDO
        DO I=1,5
          F2(I)=0.
          DO IS=1,NSYSTX(2)
            F2(I)=F2(I)+SCX(I,IS,2)*RSS(IS,2)*GD(IS,2)
          ENDDO
        ENDDO
        DO I=1,5
        DO J=1,5
          FGRAD1(I,J)=0.
          DO IS=1,NSYSTX(1)
            SFACTOR=NRSX(IS,1)*GD(IS,1)/TAUX(IS,1)
            FGRAD1(I,J)=FGRAD1(I,J)+SCX(I,IS,1)*SCX(J,IS,1)*SFACTOR
          ENDDO
        ENDDO
        ENDDO
        DO I=1,5
        DO J=1,5
          FGRAD2(I,J)=0.
          DO IS=1,NSYSTX(2)
            SFACTOR=NRSX(IS,2)*GD(IS,2)/TAUX(IS,2)
            FGRAD2(I,J)=FGRAD2(I,J)+SCX(I,IS,2)*SCX(J,IS,2)*SFACTOR
          ENDDO
        ENDDO
        ENDDO

C *** CALCULATES FUNCTIONS AND DERIVATIVES

        IF(ICOMPGRAIN.EQ.1) THEN

          DO I=1,5
            F(I)=-DB(I)+(W(1)*F1(I)+W(2)*F2(I))
            F(I)= F(I) +XMASTX(I,1)*(W(1)*X(1)+W(2)*X(6))
     #                 +XMASTX(I,2)*(W(1)*X(2)+W(2)*X(7))
     #                 +XMASTX(I,3)*(W(1)*X(3)+W(2)*X(3))
     #                 +XMASTX(I,4)*(W(1)*X(4)+W(2)*X(4))
     #                 +XMASTX(I,5)*(W(1)*X(5)+W(2)*X(8))
          ENDDO
            F(6)= F1(1)-F2(1)
            F(7)= F1(2)-F2(2)
            F(8)= F1(5)-F2(5)

          DO I=1,5
            FGRAD(I,1)=W(1)*(FGRAD1(I,1)+XMASTX(I,1))
            FGRAD(I,2)=W(1)*(FGRAD1(I,2)+XMASTX(I,2))
            FGRAD(I,5)=W(1)*(FGRAD1(I,5)+XMASTX(I,5))
            FGRAD(I,3)=W(1)*(FGRAD1(I,3)+XMASTX(I,3))+
     #                 W(2)*(FGRAD2(I,3)+XMASTX(I,3))
            FGRAD(I,4)=W(1)*(FGRAD1(I,4)+XMASTX(I,4))+
     #                 W(2)*(FGRAD2(I,4)+XMASTX(I,4))
            FGRAD(I,6)=W(2)*(FGRAD2(I,1)+XMASTX(I,1))
            FGRAD(I,7)=W(2)*(FGRAD2(I,2)+XMASTX(I,2))
            FGRAD(I,8)=W(2)*(FGRAD2(I,5)+XMASTX(I,5))
          ENDDO

          DO I=6,8
            IF(I.EQ.6) IX=1
            IF(I.EQ.7) IX=2
            IF(I.EQ.8) IX=5
            FGRAD(I,1)=FGRAD1(IX,1)
            FGRAD(I,2)=FGRAD1(IX,2)
            FGRAD(I,5)=FGRAD1(IX,5)
            FGRAD(I,3)=FGRAD1(IX,3)-FGRAD2(IX,3)
            FGRAD(I,4)=FGRAD1(IX,4)-FGRAD2(IX,4)
            FGRAD(I,6)=            -FGRAD2(IX,1)
            FGRAD(I,7)=            -FGRAD2(IX,2)
            FGRAD(I,8)=            -FGRAD2(IX,5)
          ENDDO

        ELSE IF(ICOMPGRAIN.EQ.2) THEN

          DO I=1,5
            F(I)=-DB(I)+(W(1)*F1(I)+W(2)*F2(I))
            DO J=1,5
              F(I)= F(I) +XMASTX(I,J)*X(J)
            ENDDO
          ENDDO

          DO I=1,5
          DO J=1,5
            FGRADX(I,J)=W(1)*(FGRAD1(I,J)+XMASTX(I,J))+
     #                  W(2)*(FGRAD2(I,J)+XMASTX(I,J))
          ENDDO
          ENDDO

        ENDIF

        DO I=1,NDIME
          XOLD(I)=X(I)
        ENDDO

        if(iprx.eq.1) then
c       write(10,'('' x(i) '',8f9.3)') (x(i),i=1,ndime)
        write(10,'('' w1 d1(i) '',f5.3,2x,5f10.4)') w(1),f1
c       write(10,'('' fgrad1'',/,(5f10.4))') ((fgrad1(i,j),j=1,5),i=1,5)
        write(10,'('' w2 d2(i) '',f5.3,2x,5f10.4)') w(2),f2
c       write(10,'('' fgrad2'',/,(5f10.4))') ((fgrad2(i,j),j=1,5),i=1,5)
        endif

C *** SOLVES LINEAR SYSTEM
        IF(ICOMPGRAIN.EQ.1) CALL LU_EQSYSTEM (FGRAD ,F,NDIME,ISINGULAR)
        IF(ICOMPGRAIN.EQ.2) CALL LU_EQSYSTEM (FGRADX,F,NDIME,ISINGULAR)

        IF(ISINGULAR.EQ.1) THEN
          IERROR=1
          RETURN
        ELSE

c         if(iprx.eq.1) then
c           write(10,*) '***** solution f(i) *****'
c           write(10,'(8f10.3)') (f(i),i=1,ndime)
c         endif

          DO I=1,NDIME
            X(I)=XOLD(I)-F(I)
          ENDDO
        ENDIF

        RERROR=TMISMATCH(X,XOLD,NDIME,1)
        IF(RERROR.LT.EPS) RETURN

1000  CONTINUE      ! END OF MASTER DO
C *******************************************************************

      IERROR=2

      RETURN
      END
C
C ****************************************************************************
C     SUBROUTINE GRAIN_RATE_AND_MODULI_CG    -->  VERSION 27/DEC/02
C
C     GIVEN STRESSES 'SG2' IN PARENT & CHILD OF GRAIN 'KKK', CALCULATES
C     STRAIN-RATE AND VISCO-PLASTIC MODULI USING THE RATE SENSITIVITY
C     KINEMATIC LAW.
C
C     GRAIN ARRAYS HAVE BEEN COPIED INTO AUXILIAR ARRAYS IN 'COMPOSITE_GRAIN_
C     STRESS' AND ARE PASSED THROUGH COMMON/NLNR/
C ****************************************************************************

      SUBROUTINE GRAIN_RATE_AND_MODULI_CG (KGX,KKK,IPHEL,IPH)

      INCLUDE 'VPSC7.DIM'

      DIMENSION RSS(NSYSMX,2)

C --> ARRAYS EXPRESSED IN 'PTS' AXES
      COMMON/PTS  /SGPTS(5),DGPTS(5),XMGPTS(5,5),
     #             SG2(5,2),DG2(5,2),XMCSEC2(5,5,2),
     #             SBARPTS(5),DBARPTS(5),XMASTPTS(5,5)
C --> TEMPORARY ARRAYS ASSOCIATED WITH GRAIN SYSTEMS
      COMMON/NLNR2/SCX(5,NSYSMX,2),TAUX(NSYSMX,2),GAMD0X(NSYSMX,2),
     #             NSYSTX(2),NRSX(NSYSMX,2),ISENSEX(NSYSMX,2),W(2),
     #             DB(5),XMASTX(5,5)


C     GETS RESOLVED SHEAR STRESSES 'rssx' AND SHEAR RATES 'gamdot'.
C       SIGN(GAMDOT)=SIGN(RSSX).
C       NRS CAN BE EVEN OR ODD.
C       RSS IS ALWAYS > 0 AND IS USED TO CALCULATE VISCOUS COMPLIANCE.

      DO IC=1,2
        KG2=KGX+(IC/2)*NGR(1)
        DO IS=1,NSYSTX(IC)
          RSSX=SCX(1,IS,IC)*SG2(1,IC)+SCX(2,IS,IC)*SG2(2,IC)+
     #         SCX(3,IS,IC)*SG2(3,IC)+SCX(4,IS,IC)*SG2(4,IC)+
     #         SCX(5,IS,IC)*SG2(5,IC)
          IF(.NOT.(RSSX.GT.0 .OR. ISENSEX(IS,IC).EQ.1)) RSSX=0.
          RSSX=RSSX/TAUX(IS,IC)
          RSS(IS,IC)=GAMD0X(IS,IC)*ABS(RSSX**(NRSX(IS,IC)-1))/
     #                   TAUX(IS,IC)
          GAMDOT(IS,KG2)=GAMD0X(IS,IC)*ABS(RSSX**NRSX(IS,IC))*
     #                   SIGN(1.,RSSX)
        ENDDO
      ENDDO

c     if(kkk.eq.1) then
        write(10,*) ' grain #1 inside GRAIN_RATE_CG'
c       write(*,'('' rss(is,1)'',/,(6f10.3))')
c    #              (rss(is,1),is=1,nsystx(1))
c       write(*,'('' rss(is,2)'',/,(6f10.3))')
c    #              (rss(is,2),is=1,nsystx(2))
c       pause
c     endif

C     CALCULATE STRAIN-RATE IN PARENT, CHILD AND COMPOSITE GRAIN.

      DO IC=1,2
        KG2=KGX+(IC/2)*NGR(1)
        DO I=1,5
          DG2(I,IC)=0.
          DO IS=1,NSYSTX(IC)
            DG2(I,IC)=DG2(I,IC)+SCX(I,IS,IC)*GAMDOT(IS,KG2)
          ENDDO
        ENDDO
      ENDDO
      DO I=1,5
        DGPTS(I)=W(1)*DG2(I,1)+W(2)*DG2(I,2)
      ENDDO

C     CALCULATE SECANT COMPLIANCE IN PARENT AND CHILD.

      DO IC=1,2
        DO I=1,5
        DO J=1,5
          XMCSEC2(I,J,IC)=0.
          DO IS=1,NSYSTX(IC)
            XMCSEC2(I,J,IC)=XMCSEC2(I,J,IC)+SCX(I,IS,IC)*SCX(J,IS,IC)*
     #                                      RSS(IS,IC)
          ENDDO
        ENDDO
        ENDDO
      ENDDO

      RETURN
      END
C
C ***********************************************************************
C     SUBROUTINE COMPLIANCE_CG     --->    VERSION 03/jan/03
C
C     CALCULATES SECANT COMPLIANCE ASSOCIATED WITH A GRAIN COMPOSED OF
C     TWO PHASES WITH INDIVIDUAL SECANT COMPLIANCE 'XMSEC(I,J)' AND
C     INDIVIDUAL VOLUME FRACTIONS 'W(N)'.
C     BASED ON EXPRESSION IN [Lebensohn, MSMSE, vol 7, 1999]
C ***********************************************************************

      SUBROUTINE COMPLIANCE_CG (NRSX,W,XMCSEC2,XMCOMP)

      dimension xk(5,5),xa(5,5),xident(5,5)
      dimension aux1(5,5),aux2(5,5),aux3(5,5),aux4(5,5),aux5(5,5)
      DIMENSION W(2),XMCSEC2(5,5,2),XMCOMP(5,5)


      do i=1,5
      do j=1,5
        XK(I,J)    =(I/J)*(J/I)*1.E06          ! matrix K
        XIDENT(I,J)=(I/J)*(J/I)                ! 5x5 identity
      enddo
      enddo
      XK(3,3)=1.e-6                            ! axes dependent
      XK(4,4)=1.e-6

      do i=1,5
      do j=1,5
        aux1(i,j)=xident(i,j)
        aux2(i,j)=xident(i,j)
        do k=1,5
          aux1(i,j)=aux1(i,j)+xk(i,k)*NRSX*xmcsec2(k,j,1)   ! (I+K:Mtg,m)
          aux2(i,j)=aux2(i,j)+xk(i,k)*NRSX*xmcsec2(k,j,2)   ! (I+K:Mtg,t)
        enddo
      enddo
      enddo

      CALL LU_INVERSE (AUX1,5)

      do i=1,5
      do j=1,5
        xa(i,j)=0.
        do k=1,5
          xa(i,j)=xa(i,j)+aux1(i,k)*aux2(k,j)          ! matrix A
        enddo
      enddo
      enddo

      do i=1,5
      do j=1,5
        aux3(i,j)=0.
        do k=1,5
          aux3(i,j)=aux3(i,j)+NRSX*xmcsec2(i,k,1)*xa(k,j)    ! Mtg,m:A
        enddo
      enddo
      enddo

      do i=1,5
      do j=1,5
        aux4(i,j)=w(1)*aux3(i,j)+w(2)*NRSX*xmcsec2(i,j,2)
        aux5(i,j)=w(1)*xa(i,j)+w(2)*xident(i,j)
      enddo
      enddo

      CALL LU_INVERSE (AUX5,5)

      do i=1,5
      do j=1,5
        XMCOMP(I,J)=0.
        do k=1,5
          XMCOMP(I,J)=XMCOMP(I,J)+1./NRSX*aux4(i,k)*aux5(k,j)
        enddo
      enddo
      enddo

      return
      end
C **********************************************************************
C    SUBROUTINE UPDATE_TWINNING_CG  ---> VERSION OF 03/AUG/2007
C
C    CALLS SPECIFIC ROUTINES FOR Mg AND Zr WHEN IHARDLAW=2
C **********************************************************************

      SUBROUTINE UPDATE_TWINNING_CG

      INCLUDE 'VPSC7.DIM'

        IF(CG_ID.EQ.1) CALL UPDATE_TWINNING_CG_ZR
        IF(CG_ID.EQ.2) CALL UPDATE_TWINNING_CG_MG
        IF(CG_ID.EQ.3) CALL UPDATE_TWINNING_CG_ZR2

      RETURN
      END

C **********************************************************************
C    SUBROUTINE UPDATE_TWINNING_CG_ZR  ---> VERSION OF 13/may/2005
C
C    SCANS TWIN SYSTEMS, CALCULATES ACCUMULATED TWIN FRACTIONS.
C    TAGS THE PREDOMINANT TWIN SYSTEM 'PTS' IN EACH GRAIN.
C    CREATES CHILD AND DEFINES ITS ORIENTATION AS TWIN RELATED TO PARENT
C    IF TWFRPTS EXCEEDS THRES1.
C    TRANSFERS VOL FRACTION FROM PARENT TO CHILD IF THRE1<TWFPTS<THRE2.
C    DEACTIVATES TRANSFER IF TWFPTS>THRE2.
C **********************************************************************
C
C    IPTSGR(KGX)     :INDEX OF PREDOMINANT TWIN SYSTEM IN THE GRAIN.
C    IPTMGR(KGX)     :INDEX OF PREDOMINANT TWIN MODE IN THE GRAIN.
C    TWFRSY(TWS,KKK): ACCUMMULATED TWIN VOLUME FRACTION IN SYSTEM TWS
C                     IN GRAIN KKK (RELATIVE TO CG VOLUME).
C    TWFRGR(TWM):     ACCUMULATED TWIN FRACT IN EACH MODE IN GRAIN KKK
C                     IN THE STEP (RELATIVE TO CG VOLUME).
C    TWFRPH(TWM,IPH): ACCUMULATED TWIN FRACT IN EACH MODE OF EACH 'PHASE'
C                     (PARENT OR CHILD) (ABSOLUTE VOLUME FRACTION)
C    KTWSMX(KKK)=0 --> NO 'PTS' TAGGED IN THIS GRAIN.
C    KTWSMX(KKK)>0 --> INDEX OF THE TWINNING SYSTEM WITH MAXIMUM
C                      TWIN VOLUME FRACTION IN THE GRAIN
C    KTWSMX(KKK)<0 --> GRAIN REACHED MAX TWIN FRACTION AND IS UNTAGGED
C    KTWMMX(KKK) :    INDEX OF TWINNING MODE TO WHOM KTWSMX(KKK) BELONGS.

C **********************************************************************

      SUBROUTINE UPDATE_TWINNING_CG_ZR

      INCLUDE 'VPSC7.DIM'

      integer::number_grains

      DIMENSION BTWIN(3),ATWIN(3,3)
c      DIMENSION TWFRGR(NTWMMX),IPTSGR(NGRPEL),IPTMGR(NGRPEL)
      DIMENSION TWFRGR(NTWMMX),IPTMGR(NGRPEL)
      DIMENSION WGPTS(NGRMX)
C      COMMON/TWINCG/TWIN_ORIENT(3,3,NGRMX),AXIS3(NGRMX),
C     #              CREATE_PARENT(NGRMX),CREATE_TWIN(NGRMX)

      write(10,'(/,'' ENTERING UPDATE_TWINNING_CG'')')
      iprint=1   ! controls diagnostic print-out
      number_grains=0

C *** RESET ACCUMULATED TWIN SHEARS TO ZERO & ELIMINATES CHILDREN (temporary)
c     IF(IRESET.EQ.1) THEN
c       NTNEWBORN=0
c       WTNEWBORN=0.
c       DO IPH=IPHBOT,IPHTOP
c         IPHEL=IPH-IPHBOT+1
c         DO KKK=NGR(IPH-1)+1,NGR(IPH)
c           KTWSMX(KKK)=0
c           KTWMMX(KKK)=0
c           ICOMPGR(KKK)=0
c           IF(IPHEL.EQ.1) THEN
c             KKP=KKK+NGR(1)
c             WGT(KKK)=WGT(KKK)+WGT(KKP)
c             WPH(1)=WPH(1)+WGT(KKP)
c             WPH(2)=WPH(2)-WGT(KKP)
c             WGT(KKP)=0.
c           ENDIF
c           DO KTS=1,NTWSYS(IPHEL)
c             TWFRSY(KTS,KKK)=0.
c           ENDDO
c         ENDDO
c       ENDDO
c       RETURN
c     ENDIF

C **************************************************************************
C    THIS DO LOOP SCANS ALL GRAINS IN EACH PHASE/ELEMENT. IN EACH GRAIN
C    TWINNING STATISTICS ARE PERFORMED AND TAGGING FOR REORIENTATION IS DONE.
C **************************************************************************

      nnewborn=0
      wnewborn=0.

      KGX=1
      DO IPH=IPHBOT,IPHTOP
        IPHEL=IPH-IPHBOT+1
        NSLSYX=NSLSYS(IPHEL)
      DO KKK=NGR(IPH-1)+1,NGR(IPH)

        ngg=1
        iprx=iprint*(ngg/kkk)*(kkk/ngg)+
     #       iprint*((ngg+ngr(1))/kkk)*(kkk/(ngg+ngr(1)))

        IF(IPHEL.EQ.1) WGTCG=WGT(KKK)+WGT(KKK+NGR(IPHBOT))
        IF(IPHEL.EQ.2) WGTCG=WGT(KKK)+WGT(KKK-NGR(IPHBOT))
        IPTSGR(KGX)=0
        IPTMGR(KGX)=0
        IF(WGT(KKK).LT.1.E-6) GO TO 1000      ! SKIPS UNBORN CHILDREN

        TWFMAX=0.
        ITS=NSLSYX
        KTS=0
        DO ITM=1,NTWMOD(IPHEL)
          KK=ITM+NMODES(IPHEL)-NTWMOD(IPHEL)
          GMODE=0.
          TWFRGR(ITM)=0.
          TWSHX=TWSH(ITM,IPHEL)
          DO NSMX=1,NSM(KK,IPHEL)
            KTS=KTS+1             ! shifted counter for twin systems
            ITS=ITS+1             ! absolute counter over all systems
            IF(GAMDOT(ITS,KGX).GT.0.) then
              GABS=ABS(GAMDOT(ITS,KGX))*TINCR
              GMODE=GMODE+GABS
              TWFRSY(KTS,KKK)=TWFRSY(KTS,KKK)+GABS/TWSHX*WGT(KKK)/WGTCG
            ENDIF
            TWFRGR(ITM)=TWFRGR(ITM)+TWFRSY(KTS,KKK)   ! relative to CG

            IF (TWFRSY(KTS,KKK).GT.TWFMAX) THEN
              IPTSGR(KGX)=ITS
              IPTMGR(KGX)=ITM
              TWFMAX=TWFRSY(KTS,KKK)
            ENDIF
          ENDDO      ! END OF LOOP OVER TWIN SYSTEMS IN EACH MODE

          TWFRPH(ITM,IPH)=TWFRPH(ITM,IPH)+(GMODE/TWSHX)*WGT(KKK)

        ENDDO      ! END OF LOOP OVER TWIN MODES IN THE PHASE
c        write(30,*) 'twin system', iptsgr(1)
c        do i=1,ntwmod(iphel)
c          twin_grain(i,kkk)=twfrgr(i)
c        enddo
        if(iprx.eq.1) write(10,'(''  kkk,iptm,ipts,twfmax,wgtcg'',
     #         3i5,2e12.4)') kkk,iptmgr(kgx),iptsgr(kgx),twfmax,wgtcg

 1000 KGX=KGX+1
      ENDDO
      ENDDO

C *************************************************************************
C *** THE FOLLOWING RUNS ONLY FOR PARENT BUT ALSO SETS PARAMETERS FOR CHILD
C *************************************************************************

C *** IF TWIN FRACT EXCEEDS MIN THRESHOLD BUT GRAIN HAS NOT BEEN TAGGED YET:
C     *  TAGS THE 'Predominant Twin System'(PTS) IN PARENT AND CHILD BY
C        STORING ITS INDEX IN 'KTWSMX(KKK)'. (TAG REMAINS EVEN IF ANOTHER
C        TWIN SYSTEM BECOMES PREDOMINANT LATER).
C     *  CREATES CHILD AND DEFINES ITS ORIENTATION AS TWIN RELATED TO PARENT.
C        (CO-ROTATION SCHEME WILL TAKE CARE OF THE PARENT-CHILD REORIENTATION)

      TWINTHRES=0.05 

      KGX=1
      IPH=IPHBOT
      IPHEL=IPH-IPHBOT+1
      NSLSYX=NSLSYS(IPHEL)
      DO KKK=NGR(IPH-1)+1,NGR(IPH)

        ngp=1
        ngc=ngp+ngr(1)
        iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

        KKP=KKK+NGR(1)
        IPTS=IPTSGR(KGX)
        IPTM=IPTMGR(KGX)
        TWFRPTS=0.
        IF(IPTS.NE.0) THEN
          IF(KTWSMX(KKK).EQ.0) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
          IF(KTWSMX(KKK).GT.0) TWFRPTS=TWFRSY(KTWSMX(KKK)-NSLSYX,KKK)

C --> KILLS THE 'CHILD' WHEN ANOTHER TWIN SYSTEM PREVALES AND THE CHILD
C --> HAS NOT YET ACCUMULATED THE MAXIMUM TWIN FRACTION (KTWSMX<0)
          inewborn=0
          if(inewborn.eq.1) then
          IF(KTWSMX(KKK).GT.0 .AND. KTWSMX(KKK).NE.IPTS) THEN
            TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
            KTWSMX(KKK)=0
            KTWMMX(KKK)=0
            ICOMPGR(KKK)=0
            WGT(KKK)=WGT(KKK)+WGT(KKP)
            WPH(1)=WPH(1)+WGT(KKP)
            WPH(2)=1.-WPH(1)
            WGT(KKP)=0.

            nnewborn=nnewborn+1
            wnewborn=wnewborn+wgt(kkk)
            ntnewborn=ntnewborn+1
            wtnewborn=wtnewborn+wgt(kkk)

            DO IS=1,NSYST(1)
              TAUE(IS,KKP)=TAUE(IS,KKK)
            ENDDO
          ENDIF
          endif     ! end of if(inewborn.eq.1)

        ENDIF      ! end of if(ipts.ne.0)

        IF(KTWSMX(KKK).EQ.0 .AND. TWFRPTS.GE.0.05) THEN

          if(iprx.eq.1) write(10,'('' TAGGING: kkk,iptm,ipts,twfmax'',
     #                  3i5,e12.4)')         kkk,iptm,ipts,twfmax

          KTWSMX(KKK)=IPTS
          KTWMMX(KKK)=IPTM
          KTWSMX(KKP)=IPTS
          KTWMMX(KKP)=IPTM

          DO I=1,3
            BTWIN(I)=DBCA(I,IPTS,1)
          ENDDO
          CALL TWIN_ORIENTATION (BTWIN,ATWIN)
          DO I=1,3
          DO J=1,3
            AG(I,J,KKP)=0.
            DO K=1,3
              AG(I,J,KKP)=AG(I,J,KKP)+AG(I,K,KKK)*ATWIN(K,J)
            ENDDO
c grain consider as a composite grain as soon as PTS determined
c            ICOMPGR(KKK)= ICOMPGRAIN
          ENDDO
          ENDDO

C *** SETS THE CRSS (FOREST) IN THE NEWBORN CHILD EQUAL TO THE CRSS IN PARENT
C     TIMES A FACTOR ASSOCIATED WITH BASINSKY MECHANISM.
          HBASINSKI=1.0
          NSLSYX=NSLSYS(2)
          DO ISS=1,NSLSYX
ccc            TAUE(ISS,KKP) =HBASINSKI*TAUE(ISS,KKK)
          TAUE(ISS,KKP) =TAU(ISS,0,2)       ! reset crss to tau0 (may 04 05)
          ENDDO
          DO ITS=NSLSYX+1,NSYST(2)
ccc            TAUE(ITS,KKP) =HBASINSKI*TAUE(ITS,KKK)
         TAUE(ITS,KKP) =  TAU(ITS,0,2)     ! reset crss to tau0 (may 04 05)
          ENDDO
C *** INCR CRSS OF 'PTS' IN CHILD TO PREVENT 'DETWINNING' (moved to upd_crss)
ccc       TAUE(IPTS,KKP)=50.*TAUE(IPTS,KKK)       ! new GND+HP hardening

        ENDIF      ! END OF IF KTWSMX.EQ.0 .AND...

C *** IF KTWSMX(KKK).GT.0 & TWFRPTS<THRES2 --> GRAIN IS STILL TWIN REORIENTING.
C *** TRANSFER FRACTION FROM PARENT TO CHILD

        IF(KTWSMX(KKK).GT.0) THEN
          IPTS=KTWSMX(KKK)
          IPTM=KTWMMX(KKK)
          KKP=KKK+NGR(IPHBOT)
          WGTCG=WGT(KKK)+WGT(KKP)
          TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
C  *** remove the IF loop to make sure that the total volume fraction of
C  *** the PTS is transfered to the child phase
C          IF(TWFRPTS.LE.TWTHRES(2,IPTM,IPHEL)) THEN

            if(iprx.eq.1) then
              write(10,'('' updates weight & ipts of twinned grain'')')
              write(10,'('' kkk,ipts,TWFRPTS'',2i5,e12.4)')
     #                      kkk,ipts,TWFRPTS
              write(10,'('' wgt(kkk) & wgt(kkp)'',2e12.4)')
     #                      wgt(kkk),wgt(kkp)
            endif

            WPH(1)  =WPH(1)-WGT(KKK)
            WGT(KKK)=WGTCG*(1.-TWFRPTS)
            WGT(KKP)=WGTCG-WGT(KKK)
            WPH(1)  =WPH(1)+WGT(KKK)
            WPH(2)  =1.-WPH(1)

            if(iprx.eq.1) write(10,'('' updated weights  '',2e12.4)')
     #                               wgt(kkk),wgt(kkp)
c          ENDIF

C *** IF THE TWIN FRACTION EXCEEDS MAXIMUM THRESHOLD THEN:
C       * 'UNTAGS' GRAIN AFTER HAVING UPDATED VOLUME FRACT OF PARENT & CHILD.
C       * INCREASES CRSS OF TWINS IN PARENT TO SUPPRESS TWINNING (optional).
C       * INCREASES CRSS OF PTS IN PARENT TO SUPPRESS FURTHER CHILD GROWTH.
C       * RESETS TO ZERO THE TWIN FRACTION TRANSFERED TO CHILD.
C       * STARTS TREATING PARENT+CHILD AS A COMPOSITE GRAIN IF ICOMPGRAIN=1.

          IF(TWFRPTS.GE.TWTHRES(2,IPTM,IPHEL)) THEN
            if(iprx.eq.1)
     #      write(10,'('' untagging kkk,iptm,ipts,twfrpts'',3i5,e12.4)')
     #                                kkk,iptm,ipts,twfrpts
c            write(30,*)  kkk,iptm,ipts,twfrpts
            number_grains=number_grains+1

            NSLSYX=NSLSYS(1)
C           DO ITS=NSLSYX+1,NSYST(1)
C             TAUE(ITS,KKK)= 5.*TAUE(ITS,KKK)
C           ENDDO

ccc         TAUE(IPTS,KKK)=50.*TAUE(IPTS,KKK)      ! hardening less HP term

            TWFRSY(IPTS-NSLSYX,KKK)= 0.
            KTWSMX(KKK) =-KTWSMX(KKK)
            KTWSMX(KKP) =-KTWSMX(KKP)
C grain considered as a CG if twinned volume fraction> thres2
            ICOMPGR(KKK)= ICOMPGRAIN
          ENDIF

        ENDIF      ! END OF IF(KTWSMX.GT.0)

      KGX=KGX+1
      ENDDO      ! END OF DO KKK

      if(inewborn.eq.1) write(10,'(''-> newborn (n nt w wt)'',
     #     2i5,2f10.6 )') nnewborn,ntnewborn,wnewborn,wtnewborn


C*****************************************************************
C*****  CALCULATE THE 2NDARY TWIN VOLUME FRACTION IN PRIMARY TWINS
C*****  USING ONLY THE PTS
C*****************************************************************
        WTW2M1=0
        WTW2M2=0
        DO KKC=1+NGR(1),NGR(IPHTOP)
           IPTS=IPTSGR(KKC)
           IPTM=IPTMGR(KKC)
           TWFRPTS=0
           IF (IPTS.NE.0) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKC)
           WGPTS(KKC)=WGT(KKC)*TWFRPTS
           IF (IPTM.EQ.1) WTW2M1=WTW2M1+WGPTS(KKC)
           IF (IPTM.EQ.2) WTW2M2=WTW2M2+WGPTS(KKC)
        ENDDO

c      write(30,*) number_grains
      RETURN
      END
C

C **********************************************************************
C    SUBROUTINE UPDATE_TWINNING_CG_MG  ---> VERSION OF August 2006
C
C    SCANS TWIN SYSTEMS, CALCULATES ACCUMULATED TWIN FRACTIONS.
C    TAGS THE PREDOMINANT TWIN SYSTEM 'PTS' IN EACH GRAIN and child.
C    CREATES CHILD and grand-child AND DEFINES ITS ORIENTATION
C    AS TWIN RELATED TO PARENT IF TWFRPTS EXCEEDS THRES1.
C    TRANSFERS VOL FRACTION FROM PARENT TO CHILD IF THRE1<TWFPTS<THRE2.
C    DEACTIVATES TRANSFER IF TWFPTS>THRE2.
C **********************************************************************
C
C    IPTSGR(KGX)     :INDEX OF PREDOMINANT TWIN SYSTEM IN THE GRAIN.
C    IPTMGR(KGX)     :INDEX OF PREDOMINANT TWIN MODE IN THE GRAIN.
C    TWFRSY(TWS,KKK): ACCUMMULATED TWIN VOLUME FRACTION IN SYSTEM TWS
C                     IN GRAIN KKK (RELATIVE TO CG VOLUME).
C    TWFRGR(TWM):     ACCUMULATED TWIN FRACT IN EACH MODE IN GRAIN KKK
C                     IN THE STEP (RELATIVE TO CG VOLUME).
C    TWFRPH(TWM,IPH): ACCUMULATED TWIN FRACT IN EACH MODE OF EACH 'PHASE'
C                     (PARENT OR CHILD) (ABSOLUTE VOLUME FRACTION)
C    KTWSMX(KKK)=0 --> NO 'PTS' TAGGED IN THIS GRAIN.
C    KTWSMX(KKK)>0 --> INDEX OF THE TWINNING SYSTEM WITH MAXIMUM
C                      TWIN VOLUME FRACTION IN THE GRAIN
C    KTWSMX(KKK)<0 --> GRAIN REACHED MAX TWIN FRACTION AND IS UNTAGGED
C    KTWMMX(KKK) :    INDEX OF TWINNING MODE TO WHOM KTWSMX(KKK) BELONGS.

C **********************************************************************

      SUBROUTINE UPDATE_TWINNING_CG_MG

      INCLUDE 'VPSC7.DIM'

      DIMENSION BTWIN(3),ATWIN(3,3)
c      DIMENSION TWFRGR(NTWMMX),IPTSGR(NGRPEL),IPTMGR(NGRPEL)
      DIMENSION TWFRGR(NTWMMX),IPTMGR(NGRPEL)
c      COMMON/TWINCG/TWIN_ORIENT(3,3,NGRMX),AXIS3(NGRMX),
c     #              CREATE_PARENT(NGRMX),CREATE_TWIN(NGRMX),
c     #              JUST_CREATED(NGRMX)

      write(10,'(/,'' ENTERING UPDATE_TWINNING_CG'')')
      iprint=1   ! controls diagnostic print-out

C **************************************************************************
C    THIS DO LOOP SCANS ALL GRAINS IN EACH PHASE/ELEMENT. IN EACH GRAIN
C    TWINNING STATISTICS ARE PERFORMED AND TAGGING FOR REORIENTATION IS DONE.
C **************************************************************************
      KGX=1
      DO IPH=IPHBOT,IPHTOP
        IPHEL=IPH-IPHBOT+1
        NSLSYX=NSLSYS(IPHEL)
      DO KKK=NGR(IPH-1)+1,NGR(IPH)

        ngp=1
        ngc=ngp+ngr(1)
        iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

c ** change the weight of the composite grain according to the phase
        IF(IPHEL.EQ.1) THEN
           WGTCG=WGT(KKK)+WGT(KKK+NGR(1))
           IF(IPHTOP.GT.2) WGTCG=WGTCG+WGT(KKK+2*NGR(1))
        ELSE IF(IPHEL.EQ.2) THEN
           WGTCG=WGT(KKK)
C           IF(IPHTOP.LE.2) WGTCG=WGT(KKK)+WGT(KKK-NGR(1))
           IF(IPHTOP.GT.2) WGTCG=WGTCG+WGT(KKK+NGR(1))
        ELSE IF(IPHEL.EQ.3) THEN
              WGTCG=WGT(KKK)
        ENDIF
        IPTSGR(KGX)=0
        IPTMGR(KGX)=0

        IF(WGT(KKK).LT.1.E-6) GO TO 1000      ! SKIPS UNBORN CHILDREN
        TWFMAX=0.
        ITS=NSLSYX
        KTS=0
        DO ITM=1,NTWMOD(IPHEL)
          KK=ITM+NMODES(IPHEL)-NTWMOD(IPHEL)
          GMODE=0.
          TWFRGR(ITM)=0.
          TWSHX=TWSH(ITM,IPHEL)
          DO NSMX=1,NSM(KK,IPHEL)
            KTS=KTS+1             ! shifted counter for twin systems
            ITS=ITS+1             ! absolute counter over all systems
c            write(*,*) ITS,KGX,GAMDOT(ITS,KGX)
            IF(GAMDOT(ITS,KGX).LT.0.0) PAUSE 'NEGATIVE TWIN SHEAR !!'
            GSYST=GAMDOT(ITS,KGX)*TINCR
            GMODE=GMODE+GSYST
            TWFRINC=GSYST/TWSHX*WGT(KKK)/WGTCG
            TWFRSY(KTS,KKK)=TWFRSY(KTS,KKK)+TWFRINC
            if(twfrsy(kts,kkk).gt.1.0) twfrsy(kts,kkk)=0.99  !avoid the problem of having twin fraction greater than 1
            TWFRGR(ITM)=TWFRGR(ITM)+TWFRSY(KTS,KKK)   ! relative to CG

C *** TRANSFER FRACTION BACK TO PARENT IF PTS IS ACTIVE IN CHILD (DETWINNING)
C *** DETWINNING NOT ALLOWED IN PARENT IF THERE IS A GRAND-CHILD
           M_CONDITION=0
           IF(IPHEL.EQ.3) M_CONDITION=1 !put 1 if you want to allow detwinning of primary twins, 0 otherwise
           IF((IPHEL.EQ.2).AND.(KTWSMX(KKK).EQ.0)) M_CONDITION=1
           IF(M_CONDITION.EQ.1) THEN
              IF(ITS.EQ.ABS(KTWSMX(KKK-NGR(1)))) THEN
                KKX=KGX-NGR(1)          ! child # relative to element
                TWFRSY(KTS,KKX)=TWFRSY(KTS,KKX)-GAMDOT(ITS,KKK)*TINCR/
     #                                        TWSHX*WGT(KKK)/WGTCG
                if (twfrsy(kts,kkx).lt.0) twfrsy(kts,kkx)=0
C *** If detwinning occurs, in parent all shear strain = 0 and the threshold stress is equal to tau0
c                if(detwin(kkk-ngr(1)).eq.0) then
c                   do is=1,NSYST(IPHEL)
c                      gamdot(is,kkk-ngr(1))=0
c                      taue(is,kkk-ngr(1))=tau(is,0,2)
c                      detwin(kkk-ngr(1))=1    !to prevent the reinitialization at each step
c                   enddo
c                endif
              ENDIF
           ENDIF

           IF (TWFRSY(KTS,KKK).GT.TWFMAX) THEN
              IPTSGR(KGX)=ITS
              IPTMGR(KGX)=ITM
              TWFMAX=TWFRSY(KTS,KKK)
           ENDIF
          ENDDO      ! END OF LOOP OVER TWIN SYSTEMS IN EACH MODE

          TWFRPH(ITM,IPH)=TWFRPH(ITM,IPH)+(GMODE/TWSHX)*WGT(KKK)

        ENDDO      ! END OF LOOP OVER TWIN MODES IN THE PHASE

        if(iprx.eq.1) write(10,'(''  kkk,iptm,ipts,twfmax,wgtcg'',
     #         3i5,2e12.4)') kkk,iptmgr(kgx),iptsgr(kgx),twfmax,wgtcg

 1000 KGX=KGX+1
      ENDDO
      ENDDO

C **************************************************************************************************
C *** THE FOLLOWING RUNS ONLY FOR PARENT BUT ALSO SETS PARAMETERS FOR CHILD AND GRAND-CHILD
C **************************************************************************************************

      TWINTHRES=0.05     ! HARDWIRES MIN FRACTION REQUIRED FOR DEFINING PTS

C *** IF TWIN FRACT EXCEEDS MIN THRESHOLD BUT GRAIN HAS NOT BEEN TAGGED YET:
C     *  TAGS THE 'Predominant Twin System'(PTS) IN PARENT BY
C        STORING ITS INDEX IN 'KTWSMX(KKK)'. (TAG REMAINS EVEN IF ANOTHER
C        TWIN SYSTEM BECOMES PREDOMINANT LATER).
C     *  CREATES CHILD AND DEFINES ITS ORIENTATION AS TWIN RELATED TO PARENT.
C        (CO-ROTATION SCHEME WILL TAKE CARE OF THE PARENT-CHILD REORIENTATION)
C     *  DO A SIMILAR TAGGING PROCESS FOR CHILD AND GRAND-CHILD
C        THE GRAND-CHILD WILL BE TAGGED AS THE CHILD


      KGX=1
C *** MODIFY THIS SECTION TO TAKE INTO ACCOUNT 2ND GENERATION TWINS (GP)

      IPH=IPHBOT
      IPHEL=IPH-IPHBOT+1
      NSLSYX=NSLSYS(IPHEL)
      DO KKK=NGR(IPH-1)+1,NGR(IPH)

        ngp=1
        ngc=ngp+ngr(1)
        iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

        KKP=KKK+NGR(1) !CHILD
        KKP2=KKP+NGR(1) !GRAND-CHILD
        IPTS=IPTSGR(KKK)
        IPTM=IPTMGR(KKK)
        TWFRPTS=0.
        IF(IPTS.NE.0) THEN
          IF(KTWSMX(KKK).EQ.0) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
          IF(KTWSMX(KKK).GT.0) TWFRPTS=TWFRSY(KTWSMX(KKK)-NSLSYX,KKK)

C *** CHECKS IF THE PTS IS DETWINNING AND SHRINKS BELOW THE THRESHOLD.
C *** UNTAGS THE GRAIN AND ELIMINATES THE CHILD
C *** ONLY CHILD WITHOUT GRAND-CHILD IS ALLOWED TO DETWIN
          IF((KTWSMX(KKK).GT.0).AND.(TWFRPTS.LT.TWINTHRES)) THEN
           IF(KTWSMX(KKP).EQ.0) THEN
             KTWSMX(KKK)=0
             KTWMMX(KKK)=0
             ICOMPGR(KKK)=0
             WGT(KKK)=WGT(KKK)+WGT(KKP)
             WPH(1)=WPH(1)+WGT(KKP)
             WPH(2)=WPH(2)-WGT(KKP)
             WGT(KKP)=0.
c ***      when the grain detwins, the twin volume fraction in that twin grain should be put back to 0
             do i=1,NTWSYS(2)
                 twfrsy(i,kkp)=0
                 twfrsy(i,kkk)=0 !remove all other twin systems volume fraction in parent to prevent creation of a new child
             enddo
             DO I=1,NSYST(2)
                GAMDOT(I,KKP)=0
             ENDDO
             create_twin(kkp)=0
           ENDIF
          ENDIF
        ENDIF      ! end of if(ipts.ne.0)

C *** IF GRAIN HASN'T BEEN TAGGED YET CREATE A CHILD
        IF(KTWSMX(KKK).EQ.0 .AND. TWFRPTS.GE.TWINTHRES) THEN

          if(iprx.eq.1) write(10,'('' TAGGING: kkk,iptm,ipts,twfmax'',
     #                  3i5,e12.4)')         kkk,iptm,ipts,twfmax

c  *** the child has just been created, so we tagg it to make sure that its CRSS won't be updated for the first step it exists

          JUST_CREATED(KKK+NGR(IPHBOT)) = 1
          write(10,*) 'viens de ce faire', KKK+NGR(Iphbot)
          KTWSMX(KKK)=IPTS
          KTWMMX(KKK)=IPTM

          DO I=1,3
            BTWIN(I)=DBCA(I,IPTS,IPH)
          ENDDO
          CALL TWIN_ORIENTATION (BTWIN,ATWIN)
          DO I=1,3
          DO J=1,3
            AG(I,J,KKP)=0.
            DO K=1,3
              AG(I,J,KKP)=AG(I,J,KKP)+AG(I,K,KKK)*ATWIN(K,J)
            ENDDO
          ENDDO
          ENDDO

C *** SETS THE CRSS (FOREST) IN THE NEWBORN CHILD EQUAL TO THE CRSS IN PARENT
C     TIMES A FACTOR ASSOCIATED WITH BASINSKY MECHANISM   ! OLD VERSION
C *** now when child created crss=tau0
CCC       HBASINSKI=1.0
          NSLSYX=NSLSYS(2)
          DO ISS=1,NSLSYX
CCC         TAUE(ISS,KKP) =HBASINSKI*TAUE(ISS,KKK)
            TAUE(ISS,KKP) =TAU(ISS,0,2)     ! reset crss to tau0 (may 04 05)
          ENDDO
          DO ITS=NSLSYX+1,NSYST(2)
CCC         TAUE(ITS,KKP) =HBASINSKI*TAUE(ITS,KKK)
            TAUE(ITS,KKP) =TAU(ITS,0,2)     ! reset crss to tau0 (may 04 05)
          ENDDO
C *** INCR CRSS OF 'PTS' IN CHILD TO PREVENT 'DETWINNING'
ccc       TAUE(IPTS,KKP)=50.*TAUE(IPTS,KKK)

        ENDIF      ! END OF IF KTWSMX.EQ.0 .AND...

C *** IF KTWSMX(KKK).GT.0  --> GRAIN IS STILL TWIN REORIENTING.
C *** TWFRPTS<THRES2: THIS CONDITION IS NOT NECESSARY ANYMORE B/C WE STOP THE TWIN SYSTEM TO REACHED FRACTION ABOVE THRES2 (GP)
C *** TRANSFER FRACTION FROM PARENT TO CHILD
C *** IF IPHTOP>2, LOOK IF THERE IS A GRAND-CHILD

        IF(KTWSMX(KKK).GT.0) THEN
          IPTS=KTWSMX(KKK)
          IPTM=KTWMMX(KKK)
          WGTCG=WGT(KKK)+WGT(KKP)
          IF(IPHTOP.GT.2) WGTCG=WGTCG+WGT(KKP2)
          TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
          WPH(1) =WPH(1)-WGT(KKK)
          WGT(KKK)=WGTCG*(1.-TWFRPTS)
          WPH(1)  =WPH(1)+WGT(KKK)
          IF((IPHTOP.LE.2).or.
     #     (TWFRPTS.LT.(0.98*TWTHRES(2,IPTM,IPHEL)))) THEN
            wph(2)=wph(2)-wgt(kkp)
            WGT(KKP)=WGTCG-WGT(KKK)
            wph(2)=wph(2)+wgt(kkP)
          ENDIF

C *** LOOK FOR PTS IN CHILD only if child fully formed

          IF((IPHTOP.GT.2).AND.
     #       (TWFRPTS.GE.(0.98*TWTHRES(2,IPTM,IPHEL)))) THEN
            IPTS_CHILD=IPTSGR(KKP)
            IPTM_CHILD=IPTMGR(KKP)
            TWFRPTS_CHILD=0.
            IF(IPTS_CHILD.NE.0) THEN
               IF(KTWSMX(KKP).EQ.0)
     #             TWFRPTS_CHILD=TWFRSY(IPTS_CHILD-NSLSYX,KKP)
               IF(KTWSMX(KKP).GT.0)
     #             TWFRPTS_CHILD=TWFRSY(KTWSMX(KKP)-NSLSYX,KKP)

C    IF YOU WANT TO ALLOWED DETWINNING FOR PRIMARY TWINS, UNCOMMENT THE FOLLOWING LINES ****
C *** CHECKS IF THE PTS IS DETWINNING AND SHRINKS BELOW THE THRESHOLD.
C *** UNTAGS THE CHILD GRAIN AND ELIMINATES THE GRAND-CHILD
               IF(KTWSMX(KKP).GT.0.AND.TWFRPTS_CHILD.LT.TWINTHRES) THEN
                 KTWSMX(KKP)=0
                 KTWMMX(KKP)=0
                 KTWSMX(KKP2)=0
                 KTWMMX(KKP2)=0
                 WGT(KKP)=WGT(KKP)+WGT(KKP2)
                 WPH(2)=WPH(2)+WGT(KKP2)
                 WPH(3)=WPH(3)-WGT(KKP2)
                 WGT(KKP2)=0.
c ***          when the grain detwins, the twin volume fraction in that grain should be put back to 0
                 do i=1,NTWSYS(3)
                   twfrsy(i,kkp2)=0
                 enddo
                 DO I=1,NSYST(3)
                   GAMDOT(I,KKP2)=0
                 ENDDO
                 create_twin(kkp2)=0
               ENDIF

            ENDIF !IF(IPTS_CHILD.NE.0)

            IF(KTWSMX(KKP).EQ.0.AND.TWFRPTS_CHILD.GE.TWINTHRES) THEN
              KTWSMX(KKP)=IPTS_CHILD
              KTWMMX(KKP)=IPTM_CHILD
              KTWSMX(KKP2)=IPTS_CHILD
              KTWMMX(KKP2)=IPTM_CHILD

              DO I=1,3
                BTWIN(I)=DBCA(I,IPTS_CHILD,IPH)
              ENDDO
              CALL TWIN_ORIENTATION (BTWIN,ATWIN)
              DO I=1,3
              DO J=1,3
                 AG(I,J,KKP2)=0.
                 DO K=1,3
                   AG(I,J,KKP2)=AG(I,J,KKP2)+AG(I,K,KKP)*ATWIN(K,J)
                 ENDDO
              ENDDO
              ENDDO

C *** SETS THE CRSS (FOREST) IN THE NEWBORN CHILD EQUAL TO THE CRSS IN PARENT
C     TIMES A FACTOR ASSOCIATED WITH BASINSKY MECHANISM.
CCC           HBASINSKI=1.0
              NSLSYX=NSLSYS(3)
              DO ISS=1,NSLSYX
               TAUE(ISS,KKP2) =TAU(ISS,0,2)     ! reset crss to tau0 (may 04 05)
              ENDDO
              DO ITS=NSLSYX+1,NSYST(2)
               TAUE(ITS,KKP2) =TAU(ITS,0,2)     ! reset crss to tau0 (may 04 05)
              ENDDO

            ENDIF      ! END OF IF KTWSMX.EQ.0 .AND...

            IF(KTWSMX(KKP).GT.0) THEN
                IPTS_CHILD=KTWSMX(KKP)
                IPTM_CHILD=KTWMMX(KKP)
                WGT_CHILD=WGTCG-WGT(KKK)
                TWFRPTS_CHILD=TWFRSY(IPTS_CHILD-NSLSYX,KKP)
                WPH(2) =WPH(2)-WGT(KKP)
                WGT(KKP)=WGT_CHILD*(1.-TWFRPTS_CHILD)
                WPH(2)  =WPH(2)+WGT(KKP)
                WGT(KKP2)=WGT_CHILD-WGT(KKP)
                WPH(3)  =1-(wph(1)+wph(2))
            ELSE
              WPH(2)=WPH(2)-WGT(KKP)
              WGT(KKP)=WGTCG-WGT(KKK)
              WPH(2)=WPH(2)+WGT(KKP)
            ENDIF

          ENDIF !IF((IPHTOP.GT.2).AND.....)

          if(iprx.eq.1) then
            write(10,'('' updates weight & ipts of twinned grain'')')
            write(10,'('' kkk,ipts,TWFRPTS'',2i5,e12.4)')
     #                    kkk,ipts,TWFRPTS
            write(10,'('' wgt(kkk) & wgt(kkp)'',2e12.4)')
     #                    wgt(kkk),wgt(kkp)
          endif


          if(iprx.eq.1) write(10,'('' updated weights  '',2e12.4)')
     #                             wgt(kkk),wgt(kkp)


            IF(TWFRPTS.GE..05.AND.TWFRPTS.LE..7) THEN
              ICOMPGR(KKK)= ICOMPGRAIN
            ELSE
              ICOMPGR(KKK)=0
            ENDIF


        ENDIF      ! END OF IF(KTWSMX(kkk).GT.0)

      KGX=KGX+1

      ENDDO      ! END OF DO KKK


      RETURN
      END

C **********************************************************************
C    SUBROUTINE UPDATE_CRSS_CG  ---> VERSION OF 03/AUG/2007
C
C    CALLS SPECIFIC ROUTINES FOR Mg AND Zr WHEN IHARDLAW=2
C **********************************************************************

      SUBROUTINE UPDATE_CRSS_CG (IOPTION)

      INCLUDE 'VPSC7.DIM'

c      IF (IOPTION.EQ.1) THEN
c        WRITE(*,*) ' ENTER COMPOSITE GRAIN MODEL TO BE USED:'
c        WRITE(*,*) '   0: FOR UNCOUPLED PARENT AND CHILD'
c        WRITE(*,*) '   1: FOR MIX STRESS/STRAIN CONTINUITY --> '
c        WRITE(*,*) '   2: FOR CONTINUOUS STRESS ACROSS INTERFACE --> '
c        READ(*,*) ICOMPGRAIN

c        WRITE(*,*) 'ENTER CG_ID: 1 for Zr, 2 for Mg, 3 for dislo -->  '
c        READ(*,*)   CG_ID

c        WRITE(*,*) 'ENTER INITIAL GRAIN SIZE (in micro m) -->  '
c        READ(*,*)   GRSZE
c      ENDIF

        IF(CG_ID.EQ.1) CALL UPDATE_CRSS_CG_ZR (IOPTION)
        IF(CG_ID.EQ.2) CALL UPDATE_CRSS_CG_MG (IOPTION)
        IF(CG_ID.EQ.3) CALL UPDATE_CRSS_CG_disl_sys (IOPTION) !ADDEDMZ
		IF(CG_ID.EQ.4) CALL UPDATE_CRSS_CG_disl (IOPTION)

      RETURN
      END
C******************************************************************************
c     Subroutine starter
C******************************************************************************

      SUBROUTINE starter(Ipo)

      INCLUDE 'VPSC7.DIM'

      IF (Ipo.EQ.2) THEN
        WRITE(*,*) 'ENTER COMPOSITE GRAIN MODEL TO BE USED:'
        WRITE(*,*) '   0: FOR UNCOUPLED PARENT AND CHILD'
        WRITE(*,*) '   1: FOR MIX STRESS/STRAIN CONTINUITY --> '
        WRITE(*,*) '   2: FOR CONTINUOUS STRESS ACROSS INTERFACE --> '
cmk        READ(*,*) ICOMPGRAIN
           ICOMPGRAIN = 0

        WRITE(*,*) 'ENTER CG_ID:' 
        WRITE(*,*) '1 Voce Zr -->' 
        WRITE(*,*) '2 Voce Mg -->' 
        WRITE(*,*) '3 HCP with dislocation model --> '

c        READ(*,*)   CG_ID
	   CG_ID = 3

        WRITE(*,*) 'ENTER INITIAL GRAIN SIZE (in micro m) -->  '
        READ(*,*)   GRSZE
c           GRSZE = 13.0d0
      ENDIF

      RETURN
      END


C******************************************************************************
c
C******************************************************************************
C *****************************************************************************
C     SUBROUTINE UPDATE_CRSS_CG_ZR      --->      VERSION OF 15/may/05
C
C     UPDATES HARDENING IN PARENT AND CHILD DOMAINS OF A COMPOSITE GRAIN.
C     WHILE MODEL IS GENERAL, THE PRESENT VERSION FOCUSES ON TWINNING.
C
C     A VOCE LAW, FUNCTION OF THE ACCUMULATED SHEAR IN EACH GRAIN, CONTROLS
C     THE FOREST HARDENING INSIDE 'SUB-GRAINS' (PARENT OR CHILD).
C     THE SELF & LATENT COUPLING COEFFICIENTS 'HARD' ARE ASSUMED TO BE 1
C     (BUT MAY BE GIVEN OTHER VALUES).
C     THE INCREMENT OF CRSS IS GIVEN BY AN ANALYTIC INTEGRAL EXPRESSION OF
C     THE VOCE MODULATING FUNCTION, RATHER THAN USING FORWARD EXTRAPOLATION
C     AS WAS DONE BEFORE 28/SET/00.
C
C     HALL-PETCH HARDENING OPTION:
C        A DIRECTIONAL HALL-PETCH CONTRIBUTION TO HARDENING IS CONSIDERED,
C        ASSOCIATED WITH THE REDUCTION IN THE DISLOCATION MEAN FREE PATH
C        DUE TO TWINNING OF THE 'PTS'
C        * CRSS(IS,KKK): CURRENT CRSS IN SYSTEM 'IS'
C        * TAUE(IS,KKK): CRSS IN SYSTEM 'IS' LESS THE HALL-PETCH CONTRIB.
C
C     GEOMETRICALLY NECESSARY DISLOCATION HARDENING OPTION:
C        A TERM IS ADDED TO THE RATE OF CRSS DUE TO BUILD-UP OF GND's.
C        THIS TERM PROVIDES SATGE IV AND THE VOCE LAW IS NOW THE CLASICAL
C        VOCE WITH SATURATION. WE KEEP THE H-P CONTRIBUTION IN THIS APPROACH.
*******************************************************************************

      SUBROUTINE UPDATE_CRSS_CG_ZR (IOPTION)

      INCLUDE 'VPSC7.DIM'

      DIMENSION DMFP(NSYSMX)
c      COMMON/TWINCG/TWIN_ORIENT(3,3,NGRMX),AXIS3(NGRMX),
c     #              CREATE_PARENT(NGRMX),CREATE_TWIN(NGRMX)

      iprint=0   ! controls diagnostic print-out

      IF (IOPTION.EQ.1) THEN

C *** INITIALIZE THE CRSS IN ALL SYSTEMS OF EACH PHASE IN ALL GRAINS
C *** THE 'CHILD' PHASE HAS INITIAL ZERO WEIGHT, AND CRSS's WILL BE
C *** REDEFINED WHEN THE LAMELLAE ARE CREATED.
        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1
          DO KKK=NGR(IPH-1)+1,NGR(IPH)
            DO IS=1,NSYST(IPHEL)
              CRSS(IS,KKK)=TAU(IS,0,IPHEL)+HPFAC(IS,IPHEL)/SQRT(GRSZE)
              TAUE(IS,KKK)=TAU(IS,0,IPHEL)     ! Voce + GND contribution

              create_twin(kkk)  =0
              create_parent(kkk)=0

            ENDDO

            ngg=699
            iprx=iprint*(ngg/kkk)*(kkk/ngg)+
     #           iprint*((ngg+ngr(1))/kkk)*(kkk/(ngg+ngr(1)))
            if(iprx.ne.0) then
              write(10,'('' initial CRSS for grain # '', i4)') kkk
              write(10,'(12f6.0)') (crss(is,kkk),is=1,nsyst(iphel))
            endif

          ENDDO
        ENDDO

C *** CALCULATES THE MEAN FREE PATH 'SYSMFP(is,it)' THAT TWIN SYSTEM 'it'
C     DEFINES FOR OTHER SLIP OR TWIN SYSTEMS 'is'.
C     -->   Dmfp = Dsep * SYSMFP      and Dmfp < Dg
C           Dsep: SEPARATION OF TWIN LAMELLAE / Dg: GRAIN SIZE.
C *** CALCULATES TRANSMISSIBILITY FACTORS 'TRANSFER(is,it)' ACROSS TWINS.

        CALL TWIN_BARRIERS

      ENDIF      ! END OF IOPTION=1

C **************************************************************************
C *** UPDATES CRSS's IN PARENT AND CHILD
C **************************************************************************

      IF (IOPTION.EQ.2) THEN

        write(10,'(/,'' ENTERING UPDATE_CRSS_CG'')')

c****************************************
c       calculate edot macro but can do better
c****************************************


        KGX=1
        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1
          NSLSYX=NSLSYS(IPHEL)
c*******************************************************************************
c       dislocation ratio factor rhosat maybe
c*********************************************************
        DO KKK=NGR(IPH-1)+1,NGR(IPH)
          ngp=699
          ngc=ngp+ngr(1)
          iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))


        IF(WGT(KKK).GT.1.E-6) THEN   ! AVOID 'GHOST' GRAINS IN TWINNED PHASE

C *** DEFINES A MINIMUM MEAN FREE PATH 'DMFPX' IN GRAIN/PARENT/CHILD
C *** DEFINES A BARRIER FACTOR 'BARF' TO DELAY BARRIER EFFECT.
C *** NEW APPROACH WITH INCREASING TWIN THICKNESS:
C     * TWTHRES1: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg]
C     * TWTHRES2: MAXIMUM TWIN FRACTION IN THE GRAIN

          IPTS=ABS(KTWSMX(KKK))
          IPTM=KTWMMX(KKK)
          BARF=0.
          TWFRPTS=0.
          DMFPX=GRSZE

          IF(IPTS.NE.0) THEN
            TH0X=0.05
            TH1X=TWTHRES(1,IPTM,1)
            TH2X=TWTHRES(2,IPTM,1)
            KKX=KKK
            IF(IPHEL.EQ.2) KKX=KKK-NGR(1)
            IF(KTWSMX(KKX).GT.0) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKX)
            IF(KTWSMX(KKX).LT.0) TWFRPTS=(1.-TH2X)
            IF(IPHEL.EQ.1) THEN

C *** POWER LAW INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
              POWER=1.0
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X)
     #                       BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER
              IF(TWFRPTS.GE.TH2X) BARF=1.

C *** SIGMOIDAL INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
C             SLOPE=2.
C             TFX=(2*TWFRPTS-TH0X-TH2X)/(TH2X-TH0X)
C             BARF=1./(1.+EXP(-SLOPE*TFX))

              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE
            ELSE IF(IPHEL.EQ.2) THEN
              BARF=1.                      ! assumes full barrier in twin
C             DMFPX=TWFRPTS*TH1X*GRSZE     ! for increasing twin thickness
              DMFPX=TH2X*TH1X*GRSZE        ! assumes always maximum thickness
            ENDIF
          ENDIF

C --> OLD APPROACH WITH FIX TWIN THICKNESS
C         IF(KTWSMX(KKK).LT.0 .AND. IPHEL.EQ.1) THEN   ! delayed Hall-Petch
C         IF(IPTS.GT.0        .AND. IPHEL.EQ.1) THEN   ! gradual Hall-Petch
C           DMFPX=(1.-TWFRPTS)/TWFRPTS*TWTHRES(1,IPTM,IPHEL)*GRSZE
C         ELSE IF(IPTS.GT.0 .AND. IPHEL.EQ.2) THEN
C           DMFPX=        TWTHRES(1,IPTM,IPHEL)*GRSZE
C         ENDIF

          IF(DMFPX.LT.0.) THEN
            WRITE(*,'('' DMFPX<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx
            STOP
          ENDIF

C *** ALLOW FOR LATENT HARDENING IN VOCE TERM (although usually Hss=Hss'=1).
C *** THE 'SELF' HARDENING OF 'PTS' IS AUTOMATICALLY ZERO.
C *** 'PTS' HARDENS ALL SYSTEMS VIA DIRECTIONAL HALL-PETCH & GND EFFECTS.
          GAMTOTX=GTOTGR(KKK)
          DELTGAM=0.0
          DO IS=1,NSYST(IPHEL)
            IF(IS.NE.IPTS) DELTGAM=DELTGAM+ABS(GAMDOT(IS,KGX))*TINCR
          ENDDO
c*******************************************************************************
c
c       palstic slip rate
c***************************************
          DO IS=1,NSYST(IPHEL)
            DTAU=0.
            DO JS=1,NSYST(IPHEL)
              IF(JS.NE.IPTS) DTAU=DTAU+HARD(IS,JS,IPHEL)*
     #                                 ABS(GAMDOT(JS,KGX))*TINCR
            ENDDO
            TAU0 =TAU (IS,0,IPHEL)
            TAU1 =TAU (IS,1,IPHEL)
            THET0=THET(IS,0,IPHEL)
            THET1=THET(IS,1,IPHEL)
            TINY=1.E-4*TAU0

            VOCE=0.0
            IF(ABS(THET0).GT.TINY) THEN
              VOCE=THET1*DELTGAM
              IF(ABS(TAU1).GT.TINY) THEN
                FACT=ABS(THET0/TAU1)
                EXPINI=EXP(-GAMTOTX*FACT)
                EXPDEL=EXP(-DELTGAM*FACT)
                VOCE  =VOCE-(FACT*TAU1-THET1)/FACT*EXPINI*
     #            (EXPDEL-1.)-THET1/FACT*EXPINI*
     #            (EXPDEL*((GAMTOTX+DELTGAM)*FACT+1.)-(GAMTOTX*FACT+1.))
              ENDIF
            ENDIF

            IF(IPTS.EQ.0) DMFP(IS)=GRSZE
            IF(IPTS.GT.0) DMFP(IS)=DMFPX*SYSMFP(IS,IPTS-NSLSYX)
            IF(DMFP(IS).GT.GRSZE) DMFP(IS)=GRSZE

C *******************************************************************
C *** VERSION WITH DIRECTIONAL HALL-PETCH PLUS CORRECTION
C *** BECAUSE OF TRANSMISSIBILITY & DELAYED BARRIER EFFECT
C
C            HPHARD=HPFAC(IS,IPHEL)/SQRT(DMFP(IS))
C     #                            *(1.-TRANSFER(IS,IPTS-NSLSYX))
C            IF(IPTS.NE.0 .AND. IPHEL.EQ.1) THEN
C              HPHARD=HPHARD*BARF
C            ENDIF
C            HPHARD=MAX(HPHARD,HPFAC(IS,IPHEL)/SQRT(GRSZE))
C
C            CRSS(IS,KKK)=TAUE(IS,KKK)+VOCE*DTAU/DELTGAM+HPHARD
C            TAUE(IS,KKK)=CRSS(IS,KKK)-HPHARD
C *******************************************************************

C *******************************************************************
C *** VERSION WITH VOCE + GND + HALL-PETCH EFFECT

C ---> ! new GND+HP hardening
            TAUVOCE=VOCE*DTAU/DELTGAM
            TAUHP  =BARF*HPFAC(IS,IPHEL)/SQRT(DMFP(IS))
            TAUHP0 =HPFAC(IS,IPHEL)/SQRT(GRSZE)

c *** different options of permeability across TTW or CTW or both
cc          if(ipts.ge.16 .and. ipts.le.21)    ! transfer through TTW
cc          if(ipts.ge.22)                     ! transfer through CTW
cc          if(ipts.ne.0)                      ! transfer through TTW & CTW
cc   #              TAUHP=TAUHP*(1.-TRANSFER(IS,IPTS-NSLSYX))
cc          if(is.le.3) TAUHP=0.               ! no barriers for PRISM

            TAUHP=MAX(TAUHP,TAUHP0)

            TAUGND =BARF*GNDFAC(IS,IPHEL)/DMFP(IS)/
     #                          TAUE(IS,KKK)*ABS(GAMDOT(IS,KGX))*TINCR
            PTSDAMP=1.
            IF(IS.EQ.IPTS) THEN
              TAUGND=0.
              PTSDAMP=1.+(TWFRPTS/TWTHRES(2,IPTM,1))**20.
            ENDIF

cc          IF(IS.GT.NSLSYX) TAUGND=GNDFAC(IS,IPHEL)/GRSZE/
cc   #                          TAUE(IS,KKK)*DELTGAM

            CRSS(IS,KKK)=PTSDAMP*(TAUE(IS,KKK)+TAUVOCE+TAUGND)+TAUHP
            TAUE(IS,KKK)=TAUE(IS,KKK)+TAUVOCE+TAUGND

C ---> ! old HP hardening
C           TAUHP  =HPFAC(IS,IPHEL)
C           GNDHARD=BARF*GNDFAC(IS,IPHEL)/DMFP(IS)/(CRSS(IS,KKK)-TAUHP)
C           IF(IS.EQ.IPTS) GNDHARD=0.
C           CRSS(IS,KKK)=CRSS(IS,KKK)+VOCE*DTAU/DELTGAM
C    #                               +GNDHARD*ABS(GAMDOT(IS,KGX))*TINCR
C           TAUE(IS,KKK)=TAUE(IS,KKK)+VOCE*DTAU/DELTGAM

            if(iprx.eq.1) then
              write(*,'('' kkk is iptm ipts rspce'',10x,4i5,2f9.1)')
     #                   kkk,is,iptm,ipts,SYSMFP(is,ipts-nslsyx)
              write(*,'(''    twthr1 twthr2 twfrpts barf'',5x,4f9.1)')
     #                        th1x,th2x,twfrpts,barf
              write(*,'(''    crss  hpfac dmfp tauhp'',5x,4f9.1)')
     #              crss(is,kkk),hpfac(is,iphel),dmfp(is),tauhp
              pause
            endif
C *********************************************************************

          ENDDO      ! END OF DO LOOP OVER IS=1,NSYST
          GTOTGR(KKK)=GAMTOTX+DELTGAM
        ENDIF      ! END OF IF WGT(KKK)>0

      if(iprx.eq.1) then
c         write(10,'('' kkk dmfp(is)'',i5,6x,3f6.1,/,(12f6.1))')
c    #                  kkk,(dmfp(is),is=1,nsyst(iphel))
c         write(10,'('' TAUE(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
c    #          kkk,(taue(is,kkk),is=1,nsyst(iphel))
          write(10,'('' CRSS(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
     #          kkk,(crss(is,kkk),is=1,nsyst(iphel))
          gamdnorm=tnorm5(dbar,5,1)
          write(10,'('' GAMD(is,kkk)'',i5,6x,3f7.2,/,(12f7.2))')
     #          kkk,(gamdot(is,kkk)/gamdnorm,is=1,nsyst(iphel))
      endif

        KGX=KGX+1
        ENDDO      ! END OF LOOP OVER GRAINS IN PHASE 'IPH'
        ENDDO      ! END OF LOOP OVER PHASES
      ENDIF      ! END OF IF(IOPTION.EQ.2)

      RETURN
      END

C *****************************************************************************
C     SUBROUTINE UPDATE_CRSS_CG_MG      --->      VERSION 16/NOV/2007
C
C     UPDATES HARDENING IN PARENT AND CHILD DOMAINS OF A COMPOSITE GRAIN.
C     WHILE MODEL IS GENERAL, THE PRESENT VERSION FOCUSES ON TWINNING.
C
C     A VOCE LAW, FUNCTION OF THE ACCUMULATED SHEAR IN EACH GRAIN, CONTROLS
C     THE FOREST HARDENING INSIDE 'SUB-GRAINS' (PARENT OR CHILD).
C     THE INCREMENT OF CRSS IS GIVEN BY AN ANALYTIC INTEGRAL EXPRESSION OF
C     THE VOCE MODULATING FUNCTION, RATHER THAN USING FORWARD EXTRAPOLATION
C     AS WAS DONE BEFORE 28/SET/00.
C
C     HALL-PETCH HARDENING OPTION:
C        A DIRECTIONAL HALL-PETCH CONTRIBUTION TO HARDENING IS CONSIDERED,
C        ASSOCIATED WITH THE REDUCTION IN THE DISLOCATION MEAN FREE PATH
C        DUE TO TWINNING OF THE 'PTS'
C        * CRSS(IS,KKK): CURRENT CRSS IN SYSTEM 'IS'
C        * TAUE(IS,KKK): CRSS IN SYSTEM 'IS' LESS THE HALL-PETCH CONTRIB.
C
C     GEOMETRICALLY NECESSARY DISLOCATION HARDENING OPTION:
C        A TERM IS ADDED TO THE RATE OF CRSS DUE TO BUILD-UP OF GND's.
C        THIS TERM PROVIDES SATGE IV AND THE VOCE LAW IS NOW THE CLASICAL
C        VOCE WITH SATURATION. WE KEEP THE H-P CONTRIBUTION IN THIS APPROACH.
*******************************************************************************

      SUBROUTINE UPDATE_CRSS_CG_MG (IOPTION)

      INCLUDE 'VPSC7.DIM'

      DIMENSION DMFP(NSYSMX),DMFP_HP(NSYSMX)
      DIMENSION THETX(NMODMX),TAUX(NMODMX)
      DIMENSION gamma_accu(nmodmx,ngrmx),detwin(ngrmx)
      DIMENSION  TAU_TWIN(NSYSMX,NMODMX,NPHPEL)
      DIMENSION THET_TWIN(NSYSMX,NMODMX,NPHPEL)
      INTEGER CONDITION,PREDOM

c      COMMON/TWINCG/TWIN_ORIENT(3,3,NGRMX),AXIS3(NGRMX),
c     #              CREATE_PARENT(NGRMX),CREATE_TWIN(NGRMX),
c     #              JUST_CREATED(NGRMX)
      iprint=1   ! controls diagnostic print-out

      IF (IOPTION.EQ.1) THEN

C *** INITIALIZE THE CRSS IN ALL SYSTEMS OF EACH PHASE IN ALL GRAINS
C *** THE 'CHILD'AND 'GRAND-CHILD' PHASE HAVE INITIAL ZERO WEIGHT, AND CRSS's WILL BE
C *** REDEFINED WHEN THE LAMELLAE ARE CREATED.
C *** READ THE TWINNING HARDENING PARAMETERS THAT ARE DIFFERENT IN THE CASE OF MAGNESIUM
C *** READ AT THE END OF THE CRYSTAL FILE

ccc     open(22,file='twhard.in',status='unknown')
        read(ur1,'(a)') prosa
        DO IPH=IPHBOT,IPHTOP
           read(ur1,'(a)') prosa
           DO ITWM=1,NTWMOD(IPHBOT)
              IMOD=NSLMOD(iphbot)+ITWM
              read(ur1,'(a)') prosa
              READ(ur1,*)     MODEX,NSMX,NRSX,ISENSEX
              READ(ur1,*)     TAU0X,(TAUX(I),I=1,NSLMOD(IPHBOT)),
     #                    (THETX(I),I=1,NSLMOD(IPHBOT)),HPFACX,GNDFACX
C              WRITE(*,*)   TAU0X,(TAUX(I),I=1,NSLMOD(IPHBOT)),
C     #                    (THETX(I),I=1,NSLMOD(IPHBOT)),HPFACX,GNDFACX
c           ENDDO
C --> need to generalize for 3rd generation twins, etc
c       IF(IPHTOP.LE.2)
c    #    READ(UR1,*) TWSHX,ISECTWX,THRES1X,THRES2X
c       IF(IPHTOP.GT.2)
c    #    READ(UR1,*) TWSHX,ISECTWX,THRES1X,THRES2X,THRES3X,THRES4X
c       IF(IPHTOP.GT.2) THEN
c         TWTHRES(3,NTWMOD(IPH),IPH)=THRES3X
c         TWTHRES(4,NTWMOD(IPH),IPH)=THRES4X
c       ENDIF

c          DO IPH=IPHBOT,IPHTOP
              DO ITW=1,NSM(IMOD,iph)
                ISY=NSLSYS(IPH)+ITW
                TAU(ISY,0,iph) =TAU0X
                DO J=1,NSLMOD(IPHBOT)
                   TAU_TWIN(ISY,J,iph) =TAUX(J)
                   THET_TWIN(ISY,J,iph)=THETX(J)
                ENDDO
                HPFAC(ISY,iph) =HPFACX
                GNDFAC(ISY,iph)=GNDFACX
              ENDDO
           ENDDO
        ENDDO

        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1
          DO KKK=NGR(IPH-1)+1,NGR(IPH)
            DO IS=1,NSYST(IPHEL)
              CRSS(IS,KKK)=TAU(IS,0,IPHEL)+HPFAC(IS,IPHEL)/SQRT(GRSZE)
              TAUE(IS,KKK)=TAU(IS,0,IPHEL)     ! Voce + GND contribution

              DO IM=1,NMODES(IPHEL)
                gamma_accu(im,kkk)=0.
              ENDDO
              detwin(kkk)=0
              create_twin(kkk)  =0
              create_parent(kkk)=0
              JUST_CREATED(KKK)=0
            ENDDO

            ngp=1
            ngc=ngp+ngr(1)
            iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))
            if(iprx.ne.0) then
              write(10,'('' initial CRSS for grain # '', i4)') kkk
              write(10,'(12f6.0)') (crss(is,kkk),is=1,nsyst(iphel))
            endif

          ENDDO
        ENDDO

C *** CALCULATES THE MEAN FREE PATH 'SYSMFP(is,it)' THAT TWIN SYSTEM 'it'
C     DEFINES FOR OTHER SLIP OR TWIN SYSTEMS 'is'.

C     -->   Dmfp = Dsep * SYSMFP      and Dmfp < Dg
C           Dsep: SEPARATION OF TWIN LAMELLAE / Dg: GRAIN SIZE.
C *** CALCULATES TRANSMISSIBILITY FACTORS 'TRANSFER(is,it)' ACROSS TWINS.

        CALL TWIN_BARRIERS

      ENDIF      ! END OF IOPTION=1

C **************************************************************************
C *** UPDATES CRSS's IN PARENT, CHILD AND GRAND-CHILD
C **************************************************************************

      IF (IOPTION.EQ.2) THEN

        write(10,'(/,'' ENTERING UPDATE_CRSS_CG'')')

        KGX=1
        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1
          NSLSYX=NSLSYS(IPHEL)
        DO KKK=NGR(IPH-1)+1,NGR(IPH)
c        write(30,*) kkk,kgx
         ngp=1
         ngc=ngp+ngr(1)
         iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

c *** avoid updating the crss of a child that has just been created
         IF(JUST_CREATED(KKK).EQ.0) THEN

         IF(WGT(KKK).GT.1.E-6) THEN   ! AVOID 'GHOST' GRAINS IN TWINNED PHASES

C *** DEFINES A MINIMUM MEAN FREE PATH 'DMFPX' IN GRAIN/PARENT/CHILD
C *** DEFINES A BARRIER FACTOR 'BARF' TO DELAY BARRIER EFFECT.
C *** NEW APPROACH WITH INCREASING TWIN THICKNESS:
C     * TWTHRES1: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg] 1ST GENERATION
C     * TWTHRES2: MAXIMUM TWIN FRACTION IN THE GRAIN 1ST GENERATION
C     * TWTHRES3: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg] 2ND GENERATION
C     * TWTHRES4: MAXIMUM TWIN FRACTION IN THE GRAIN 2ND GENERATION

          IPTS=ABS(KTWSMX(KKK))
          IPTM=KTWMMX(KKK)
          BARF=0.
          TWFRPTS=0.
          DMFPX=GRSZE
          dmfpx_hp=grsze
          CONDITION=0
          TH0X=0.05

C ** DIFFERENT CONDITIONS TO CALCULATE DMFPX DEPENDING ON PHASE (GP)
C ** CODE WRITTEN AS IF THERE WERE 2 OR 3 PHASES
          IF(IPHEL.EQ.1) THEN
            IF(IPTS.NE.0) THEN
               TH1X_MIN=TWTHRES(1,IPTM,1)
               TH2X=TWTHRES(2,IPTM,1)
               CONDITION=1
             ENDIF
          ELSE IF(IPHEL.EQ.2) THEN
            IF(IPHTOP.LE.2) THEN
               IPTM_PARENT=KTWMMX(KKK-NGR(1))
               TH1X_MIN=TWTHRES(1,IPTM_PARENT,1)
               TH2X=TWTHRES(2,IPTM_PARENT,1)
               CONDITION=2
            ELSE
               IF(IPTS.EQ.0) THEN    !VERIFY THAT grand CHILD HAS NOT BEEN CREATED YET
                 IPTM_PARENT=KTWMMX(KKK-NGR(1))
                 TH1X_MIN=TWTHRES(1,IPTM_PARENT,1)
                 TH2X=TWTHRES(2,IPTM_PARENT,1)
                 CONDITION=2
               ELSE
                 IPTM_PARENT=KTWMMX(KKK-NGR(1))
                 TH1X_MIN=TWTHRES(3,IPTM,1)*TWTHRES(1,IPTM_PARENT,1) !DISTRIBUTES THE 2NDARY TWINS OVER ALL THE FIRST TWIN LAMELLAE
                 TH2X=TWTHRES(4,IPTM,1)
                 CONDITION=3
                ENDIF
             ENDIF
          ELSE IF(IPHEL.EQ.3) THEN
              IPTM_PARENT=KTWMMX(KKK-2*NGR(1))
              TH1X_MIN=TWTHRES(3,IPTM,1)*TWTHRES(1,IPTM_PARENT,1) !DISTRIBUTES THE 2NDARY TWINS OVER ALL THE FIRST TWIN LAMELLAE
              TH2X=TWTHRES(4,IPTM,1) !THIS REQUIRES THAT THE GRAND-CHILD IS TAGGED AS THE CHILD
              CONDITION=4
          ENDIF

          IF(CONDITION.NE.0) THEN !THIS DOESN'T RUN IF GRAIN WITHOUT TWIN

C *** VERSION JULY 5 2006 (GP)
C *** TWFRPTS WILL BE USE IN PARENT PHASE TO PREVENT MORE PTS DEFORMATION
C *** TWFRPTS IN PARENT WILL BE USED IN CHILD TO CALCULATED DMFPX FOR HP EFFECT
C *** SAME REASONING BETWEEN CHILD AND GRAND-CHILD

C ** NEW CONDITIONS TO CALCULATE TWIN FRACTION (GP)
            IF(IPHEL.EQ.1) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
            IF(IPHEL.EQ.2) THEN
               IF(IPHTOP.LE.2) THEN
                  IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                  TWFRPTS=TWFRSY(IPTS_PARENT-NSLSYX,KKK-NGR(1))
               ELSE
                  IF(IPTS.EQ.0) THEN
                    IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                    TWFRPTS=TWFRSY(IPTS_PARENT-NSLSYX,KKK-NGR(1))
                  ELSE
                    TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
                  ENDIF
               ENDIF
            ENDIF
            IF(IPHEL.EQ.3) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK-NGR(1))

            IF(CONDITION.EQ.1) THEN
c             CALCUL OF THRES1 IF WE ASSUME IT CONSTANT
c              TH1X=TH1X_MIN
C             CALCUL OF THRES1 IF WE ASSUME IT NON CONSTANT, VARYING WITH TWFRPTS
C             THIS FUNCTION ASSUME TH1X CONSTANT UNTIL TWFRPTS=0.25 AND THEN IT INCREASES TILL ~ 1 for twfrpts=0.7
              IF(twfrpts.le..7) THEN
                  TH1X=TH1X_MIN-0.4*TWFRPTS+TWFRPTS**2+1.5*TWFRPTS**3
c                   TH1X=1
              ELSE
                  TH1X=1
              ENDIF
              PREDOM=IPTS
C *** POWER LAW INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
              POWER=1.0
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X)
     #                       BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER
              IF(TWFRPTS.GE.TH2X) BARF=1.

C *** SIGMOIDAL INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
C             SLOPE=2.
C             TFX=(2*TWFRPTS-TH0X-TH2X)/(TH2X-TH0X)
C             BARF=1./(1.+EXP(-SLOPE*TFX))

              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE
              DMFPX_HP=(1.-TWFRPTS)*TH1X*GRSZE

            ELSE IF(CONDITION.EQ.2) THEN
c                 TH1X=TH1X_MIN
C   CALCUL OF THRES1 IF WE ASSUME IT NON CONSTANT, VARYING WITH TWFRPTS
C   THIS FUNCTION ASSUME TH1X CONSTANT UNTIL TWFRPTS=0.25 AND THEN IT INCREASES TILL ~ 1 for twfrpts=0.7
                 if(twfrpts.le..7) then
                     TH1X=TH1X_MIN-0.4*TWFRPTS+TWFRPTS**2+1.5*TWFRPTS**3
c                     TH1X=1
                 ELSE
                     TH1X=1
                 ENDIF
                 IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                 PREDOM=IPTS_PARENT
                 BARF=1.                      ! assumes full barrier in twin
                 DMFPX_HP=TWFRPTS*TH1X*GRSZE     ! for increasing twin thickness FOR HP
                 DMFPX=TH2X*TH1X*GRSZE        ! assumes always maximum thickness FOR GNDs
c              endif
c            if(kkk.eq.600) write(30,*) kkk, ipts_parent, twfrpts,
c     #        dmfpx_hp
            ELSE IF(CONDITION.EQ.3) THEN   ! for conditions 3 and 4, we need to change the value of grsze
C             CALCUL OF THRES1 IF WE ASSUME IT CONSTANT
              TH1X=TH1X_MIN
              PREDOM=IPTS
              IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
              IPTM_PARENT=KTWMMX(KKK-NGR(1))
              TWFRPTS_PARENT=TWFRSY(IPTS_PARENT-NSLSYX,KKK-NGR(1))
              GRSZE_NEW=2*TWFRPTS_PARENT*TWTHRES(1,IPTM_PARENT,1)*GRSZE
              POWER=1.0
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X)
     #                   BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER
              IF(TWFRPTS.GE.TH2X) BARF=1.
              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE_NEW
              DMFPX_HP=(1.-TWFRPTS)*TH1X*GRSZE_NEW

            ELSE IF(CONDITION.EQ.4) THEN
C             CALCUL OF THRES1 IF WE ASSUME IT CONSTANT
              TH1X=TH1X_MIN
              PREDOM=IPTS
              IPTS_PARENT=ABS(KTWSMX(KKK-2*NGR(1)))
              IPTM_PARENT=KTWMMX(KKK-2*NGR(1))
              TWFRPTS_PARENT=TWFRSY(IPTS_PARENT-NSLSYX,KKK-2*NGR(1))
              GRSZE_NEW=2*TWFRPTS_PARENT*TWTHRES(1,IPTM_PARENT,1)*GRSZE
              BARF=1.                      ! assumes full barrier in twin
              DMFPX_HP=TWFRPTS*TH1X*GRSZE_NEW
              DMFPX=TH2X*TH1X*GRSZE_NEW
            ENDIF
c          if(kkk.eq.10.or.kkk.eq.564) write(30,*) kkk, th1x
          ENDIF

          IF(DMFPX.LT.0.) THEN
            WRITE(*,'('' DMFPX<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx
            STOP
          ENDIF
          IF(DMFPX_HP.LT.0.) THEN
            WRITE(*,'('' DMFPX_HP<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx_hp'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx_HP
            STOP

          ENDIF


CCC *** CALCULATE THE ACCUMULATED SHEAR STRAIN FOR EACH DEFORMATION MODE (GP)
        it=1
        DO IM=1,NSLMOD(IPHEL)
           DO IS=1,NSM(IM,IPHEL)
              GAMMA_ACCU(IM,KKK)=GAMMA_ACCU(IM,KKK)
     #                 +ABS(GAMDOT(it,KKK))*TINCR
           it=it+1
           ENDDO
        ENDDO

c       igrain=1
c       if(kkk.eq.igrain) write(30,'(I3,19F15.5)') just_created(kkk),
c     #         wgt(kkk), (gamdot(im,kkk), im=1,18)
c       if(kkk.eq.igrain+ngr(1)) write(40,'(I3,19F15.5)')
c     #   just_created(kkk),
c     #         wgt(kkk), (gamdot(im,kkk), im=1,18)

c        do im=13,18
c        if(kkk.gt.ngr(1).and.gamdot(im,kkk).gt.1e-15)
c     #  write(30,'(2I4,f18.15,I4)') kkk,im,gamdot(im,kkk),
c     #  ktwsmx(kkk-ngr(1))
c        enddo

C *** ALLOW FOR LATENT HARDENING IN VOCE TERM (although usually Hss=Hss'=1).
C *** THE 'SELF' HARDENING OF 'PTS' IS AUTOMATICALLY ZERO.
C *** 'PTS' HARDENS ALL SYSTEMS VIA DIRECTIONAL HALL-PETCH & GND EFFECTS.
          GAMTOTX=GTOTGR(KKK)
          DELTGAM=0.0
c          if(kgx.eq.10) write(30,*) 'deltgam: before beginning loop',
c     #      deltgam
          DO IS=1,NSYST(IPHEL)
            IF(IS.NE.IPTS) DELTGAM=DELTGAM+ABS(GAMDOT(IS,KGX))*TINCR
c            if(kgx.eq.10) write(30,*) is,gamdot(is,kgx)
c     #       ,tincr,deltgam
          ENDDO
c          if(kgx.eq.10) write(30,*) 'at the end of the loop', deltgam
          TWFRGRAIN=0.
          DO IT=1,NTWSYS(IPHEL)
            TWFRGRAIN=TWFRGRAIN+TWFRSY(IT,KKK)
          ENDDO

C *** CALCULATE HARDENING USING VOCE LAW FOR SLIP SYSTEMS
          DO IS=1,NSYST(IPHEL)
            IF(IS.LE.NSYST(IPHEL)-NTWSYS(IPHEL)) THEN
              DTAU=0.
c              if(kkk.eq.1840) write(30,*) 'dtau:before beginning loop',
c     #         dtau
              DO JS=1,NSYST(IPHEL)
                 IF(JS.NE.IPTS) DTAU=DTAU+HARD(IS,JS,IPHEL)*
     #                                 ABS(GAMDOT(JS,KGX))*TINCR
c              if(kkk.eq.1840) write(30,'(2i5,f5.1,3f18.15)') is,js,
c     #         hard(is,js,iphel),gamdot(js,kkk),tincr,dtau
              ENDDO
c              if(kkk.eq.1840) write(30,*) 'at the end of the loop', dtau
              TAU0 =TAU (IS,0,IPHEL)
              TAU1 =TAU (IS,1,IPHEL)
              THET0=THET(IS,0,IPHEL)
              THET1=THET(IS,1,IPHEL)
              TINY=1.E-4*TAU0

              VOCE=0.0
              IF(ABS(THET0).GT.TINY) THEN
                VOCE=THET1*DELTGAM
                IF(ABS(TAU1).GT.TINY) THEN
                  FACT=ABS(THET0/TAU1)
                  EXPINI=EXP(-GAMTOTX*FACT)
                  EXPDEL=EXP(-DELTGAM*FACT)
                  VOCE  =VOCE-(FACT*TAU1-THET1)/FACT*EXPINI*
     #            (EXPDEL-1.)-THET1/FACT*EXPINI*
     #            (EXPDEL*((GAMTOTX+DELTGAM)*FACT+1.)-(GAMTOTX*FACT+1.))
                ENDIF
              ENDIF

            ELSE  ! new law for twinning hardening (GP)
              TAU0 =TAU (IS,0,IPHEL)
              TAU_SLIP_TWIN=0
                DO I=1,NSLMOD(IPHEL)
                   TAU_SLIP_TWIN=TAU_SLIP_TWIN + TAU_TWIN(IS,I,IPHEL)*
     #             (1-EXP(-(GAMMA_ACCU(I,KKK)*THET_TWIN(IS,I,IPHEL))/
     #             TAU_TWIN(IS,I,IPHEL)))
c                   if(kkk.eq.igrain)
c     #             out(i)= TAU_TWIN(IS,I,IPHEL)*
c     #             (1-EXP(-(GAMMA_ACCU(I,KKK)*THET_TWIN(IS,I,IPHEL))/
c     #             TAU_TWIN(IS,I,IPHEL)))
                ENDDO
c                if(kkk.eq.igrain) write(40,'(3F15.5)') (out(i),i=1,3)
             ENDIF


C ** CALCULATION OF THE MEAN FREE PATH ACCORDING TO THE TYPE OF GRAIN DEALT WITH
            IF(CONDITION.EQ.0) THEN
              DMFP(IS)=GRSZE
              DMFP_HP(IS)=GRSZE
            ENDIF

            IF(CONDITION.NE.0) THEN
              DMFP(IS)=DMFPX*SYSMFP(IS,PREDOM-NSLSYX)
              DMFP_HP(IS)=DMFPX_HP*SYSMFP(IS,PREDOM-NSLSYX)
            ENDIF

            IF((CONDITION.EQ.1).OR.(CONDITION.EQ.2)) THEN
               IF(DMFP(IS).GT.GRSZE) DMFP(IS)=GRSZE
               IF(DMFP_HP(IS).GT.GRSZE) DMFP_HP(IS)=GRSZE
            ENDIF

            IF((CONDITION.EQ.3).OR.(CONDITION.EQ.4)) THEN
               IF(DMFP(IS).GT.GRSZE_NEW) DMFP(IS)=GRSZE_NEW
               IF(DMFP_HP(IS).GT.GRSZE_NEW) DMFP_HP(IS)=GRSZE_NEW
            ENDIF

C *******************************************************************
C *** VERSION WITH VOCE + GND + HALL-PETCH EFFECT

C ---> ! new GND+HP hardening
            IF(IS.LE.NSYST(IPHEL)-NTWSYS(IPHEL))
     #                   TAUVOCE=VOCE*DTAU/DELTGAM
            TAUHP  =BARF*HPFAC(IS,IPHEL)/SQRT(DMFP_HP(IS))
            TAUHP0 =HPFAC(IS,IPHEL)/SQRT(GRSZE)

c *** different options of permeability across TTW or CTW or both
cc          if(ipts.ge.16 .and. ipts.le.21)    ! transfer through TTW
cc          if(ipts.ge.22)                     ! transfer through CTW
cc          if(ipts.ne.0)                      ! transfer through TTW & CTW
cc   #              TAUHP=TAUHP*(1.-TRANSFER(IS,IPTS-NSLSYX))
cc          if(is.le.3) TAUHP=0.               ! no barriers for PRISM

            TAUHP=MAX(TAUHP,TAUHP0)

c            if(gamdot(is,kgx).eq.0) gamdot(is,kgx)=0.00001
            TAUGND =BARF*GNDFAC(IS,IPHEL)/DMFP(IS)/
     #                          TAUE(IS,KKK)*ABS(GAMDOT(IS,KGX))*TINCR

C *** NEW VERSION JULY 5 2006
C *** PTS IS BLOCKED IN PARENT WHEN THRES2 IS REACHED
C *** ALL THE TWIN SYSTEMS ARE BLOCKED WHEN THE SUM OF THEIR VOL FRACTION REACHES 1
C *** IN CHILD EVERY TWIN SYSTEMS KEEPS ITS CRSS (NO PTSDAMP FACTOR) IF NO GRAND-CHILD CONSIDERED
C *** -->NO SYSTEM BLOCKED IN CHILD IF NO GRAND-CHILD CONSIDERED
C *** SAME THING BETWEEN CHILD AND GRAND-CHILD
C *** IN THE CHILD WE BLOCK PTS PRODUCING GRAND-CHILD WHEN ITS VOLUME FRACTION REACHED THRES4
c *** AND ALL THE OTHER TWIN SYSTEMS ARE STOPPED WHEN THE SUM OF THEIR TWIN VOLUME FRACTIONS REACHED 1
            PTSDAMP=1.
            IF(IPHEL.EQ.1) THEN
             if(ktwsmx(kkk).gt.0) then
              IF(IS.GT.NSLSYX) THEN
                 TAUGND=0.
                 if(is.eq.ipts) then
                   PTSDAMP=1+(TWFRPTS/TWTHRES(2,IPTM,1))**20
                 else
                   PTSDAMP=1.+ twfrgrain**20
                 ENDIF
              ENDIF
             endif
            ENDIF


C ***    BLOCK ALL THE twinning SYSTEMS BUT THE PTS, OTHERWISE DETWINNING PREVENTED.
            IF(IPHEL.EQ.2) THEN
             kkpp=kkk-ngr(1)
               IF(IS.GT.NSLSYX) THEN
                 TAUGND=0.
                 if(is.ne.ktwsmx(kkpp)) then
                     PTSDAMP=50
                 endif
               ENDIF
             ENDIF

c            IF(IPHTOP.GT.2) THEN
c              IF(IPHEL.EQ.2) THEN
c               if(iptsgr(kkk).gt.0) then
c                if(ktwsmx(kkk).eq.0) then
c                   ipts=iptsgr(kkk)
c                   iptm=iptmgr(kkk)
c                endif
c                IF(IS.GT.NSLSYX) THEN
c                  TAUGND=0.
c                  IF(IS.EQ.IPTS) THEN
c                    PTSDAMP=1+
c     #               (twfrsy(ipts-nslsyx,kkk)/TWTHRES(4,IPTM,1))**20
c                  ELSE
c                    PTSDAMP=1.+ twfrgrain**20
c                  ENDIF
c                ENDIF
c               endif
c              ENDIF
c            ENDIF

cc          IF(IS.GT.NSLSYX) TAUGND=GNDFAC(IS,IPHEL)/GRSZE/
cc   #                          TAUE(IS,KKK)*DELTGAM

              IF(IS.LE.NSYST(IPHEL)-NTWSYS(IPHEL)) THEN
                CRSS(IS,KKK)=PTSDAMP*(TAUE(IS,KKK)+TAUVOCE+TAUGND)+TAUHP
                TAUE(IS,KKK)=TAUE(IS,KKK)+TAUVOCE+TAUGND
              ELSE
                CRSS(IS,KKK)=PTSDAMP*(TAU0+TAU_SLIP_TWIN+TAUGND)+TAUHP
              ENDIF
C *********************************************************************

          ENDDO      ! END OF DO LOOP OVER IS=1,NSYST
          GTOTGR(KKK)=GAMTOTX+DELTGAM
        ENDIF      ! END OF IF WGT(KKK)>0

        ELSE
           JUST_CREATED(KKK)=0
        ENDIF

      if(iprx.eq.1) then
c         write(10,'('' kkk dmfp(is)'',i5,6x,3f6.1,/,(12f6.1))')
c    #                  kkk,(dmfp(is),is=1,nsyst(iphel))
c         write(10,'('' TAUE(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
c    #          kkk,(taue(is,kkk),is=1,nsyst(iphel))
          write(10,'('' CRSS(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
     #          kkk,(crss(is,kkk),is=1,nsyst(iphel))
          gamdnorm=tnorm5(dbar,5,1)
          write(10,'('' GAMD(is,kkk)'',i5,6x,3f7.2,/,(12f7.2))')
     #          kkk,(gamdot(is,kkk)/gamdnorm,is=1,nsyst(iphel))

      endif

        KGX=KGX+1
        ENDDO      ! END OF LOOP OVER GRAINS IN PHASE 'IPH'
        ENDDO      ! END OF LOOP OVER PHASES
      ENDIF      ! END OF IF(IOPTION.EQ.2)

      RETURN

      END

C ***********************************************************************************
C ****  SUBROUTINE TWIN_SHAPE                ---> JULY 06  (GP)  updated in August 2006
C ****  IN THAT SUBROUTINE, AN ELLIPSOID SHAPE IS GIVEN TO THE TWINS
C ****  IOPT=0: INITIALIZE ALL THE ELLIPSOID TO a=b=c=1 AND PHI1=PHI=PHI2=0 for 1st and 2nd generation twins
c  -->  iopt=0  eliminated by CNT --> done inside upd_crss
C ****  IOPT=1: CREATION OF THE TWIN, SHAPE IS GIVEN AS WELL AS ORIENTATION OF ELLIPSOID
C ****          THICKNESS OF ELLISPOID UPDATED ACCORDING TO c=TWFRSY(PTS)*THRES1*DG
C ****          UPDATE SHAPE AT EACH STEP
C ****  THIS SUBROUTINE WILL ALSO GIVEN A SHAPE TO THE PARENT GRAIN DEPENDING ON TWIN volume fraction
C ***********************************************************************************

      SUBROUTINE TWIN_SHAPE(IOPT,KKK)

      INCLUDE 'VPSC7.DIM'

      DIMENSION ANG(2),DUMMY(3,3),AA(3,3)
      REAL MAT_PROJ(3,3), PROJECTION(3), NORM_PROJ
      REAL RATIO
c      COMMON/TWINCG/TWIN_ORIENT(3,3,NGRMX),AXIS3(NGRMX),
c     #              CREATE_PARENT(NGRMX),CREATE_TWIN(NGRMX)

      NSLSYX=NSLSYS(IPHBOT)

      IF(IOPT.EQ.0) THEN   ! INITIALIZE TWIN SHAPE TO A SPHERE
      create_twin(kkk)=0
      create_parent(kkk-ngr(iphbot))=0
      create_parent(kkk)=0 ! give a value for the grand-child

       ELSEIF(IOPT.EQ.1) THEN  ! CREATES THE TWIN ELLIPSOID AND UPDATE SHAPE
         KGX=KKK-NGR(IPHBOT) !identify parent grain
         IPTS=KTWSMX(KGX)
         IPTM=KTWMMX(KGX)

C *** CREATION OF THE ELLIPSOID REPRESENTING THE TWIN
         IF(CREATE_TWIN(KKK).EQ.0) THEN

c         write(*,*) 'creation of twins for grain', kgx,
c     #     twfrsy(ipts-12,kgx)
c         pause

C       ** DEFINE ELLIPSOID ORIENTATION IN GRAIN FRAME (lattice orientation of the grain)
           DO I=1,3
              TWIN_ORIENT(I,1,KKK)=DBCA(I,IPTS,IPHBOT)
              TWIN_ORIENT(I,3,KKK)=DNCA(I,IPTS,IPHBOT)
           ENDDO
           TWIN_ORIENT(1,2,KKK)=TWIN_ORIENT(2,3,KKK)
     #                          *TWIN_ORIENT(3,1,KKK)
     #                    -TWIN_ORIENT(3,3,KKK)*TWIN_ORIENT(2,1,KKK)
           TWIN_ORIENT(2,2,KKK)=TWIN_ORIENT(3,3,KKK)
     #                          *TWIN_ORIENT(1,1,KKK)
     #                    -TWIN_ORIENT(1,3,KKK)*TWIN_ORIENT(3,1,KKK)
           TWIN_ORIENT(3,2,KKK)=TWIN_ORIENT(1,3,KKK)
     #                          *TWIN_ORIENT(2,1,KKK)
     #                    -TWIN_ORIENT(2,3,KKK)*TWIN_ORIENT(1,1,KKK)


C    ** DEFINE TWIN ELLIPSOID ORIENTATION IN SAMPLE FRAME

           DO I=1,3
           DO J=1,3
              DUMMY(I,J)=0
              DO K=1,3
                 DUMMY(I,J)=DUMMY(I,J)+AG(I,K,KGX)*
     #                         TWIN_ORIENT(K,J,KKK)
              ENDDO
           ENDDO
           ENDDO

           DO I=1,3
           DO J=1,3
              TWIN_ORIENT(I,J,KKK)=DUMMY(I,J)
              AXISGR(I,J,KKK)=DUMMY(I,J)
           ENDDO
           ENDDO

C    **   DEFINE AXES LENGTHS FOR ELLIPSOID REPRESENTING THE TWIN
C    **   WE NEED TO CALCULATE THE LENGTHS OF EACH AXIS IN THE ELLIPSOID OF THE GRAIN
          DO I=1,3
C          ** WE FIRST NEED TO CALCULATE THE POLAR (ANG(2)) AND AZIMUTHAL (ANG(1)) ANGLES
             DOT_PRODUCT=0
             DO J=1,3
                DOT_PRODUCT=DOT_PRODUCT+AXISGR(I,J,KKK)*AXISGR(3,J,KGX)
             ENDDO
             if(dot_product.gt.1.0) dot_product=1.0
             if(dot_product.lt.-1.0) dot_product=-1.0
             ANG(2)=ACOS(DOT_PRODUCT)
             IF(ANG(2).LT.0) ANG(2)=-ANG(2)
             IF(ANG(2).GT.3.1415926535898)
     #              ANG(2)=2*3.1415926535898-ANG(2)
             DO L=1,3
             DO M=1,3
                IF(L.EQ.M) THEN
                   MAT_PROJ(L,M)=1
                ELSE
                   MAT_PROJ(L,M)=0
                ENDIF
                MAT_PROJ(L,M)=MAT_PROJ(L,M)-AXISGR(3,L,KGX)
     #                                     *AXISGR(3,M,KGX)
             ENDDO
             ENDDO
             DO L=1,3
                PROJECTION(L)=0
                DO M=1,3
                   PROJECTION(L)=PROJECTION(L)
     #                +MAT_PROJ(L,M)*AXISGR(I,M,KKK)
                ENDDO
             ENDDO
             NORM_PROJ=0
             DO L=1,3
                NORM_PROJ=NORM_PROJ+PROJECTION(L)*PROJECTION(L)
             ENDDO
             NORM_PROJ=SQRT(NORM_PROJ)
             DO L=1,3
                PROJECTION(L)=PROJECTION(L)/NORM_PROJ
             ENDDO
             DOT_PRODUCT=0
             DO J=1,3
                DOT_PRODUCT=DOT_PRODUCT+PROJECTION(J)*AXISGR(1,J,KGX)
             ENDDO
             if(dot_product.gt.1.0) dot_product=1.0
             if(dot_product.lt.-1.0) dot_product=-1.0
             ANG(1)=ACOS(DOT_PRODUCT)
             X1=AXISGR(0,2,KGX)*SIN(ANG(2))*COS(ANG(1))
             X2=AXISGR(0,1,KGX)*SIN(ANG(2))*SIN(ANG(1))
             X3=AXISGR(0,3,KGX)*COS(ANG(2))
             AXISGR(0,I,KKK)=SQRT(X1*X1+X2*X2+X3*X3)
          ENDDO
          AXIS3(KKK)=AXISGR(0,3,KKK) ! NEEDS TO BE KEPT IN MEMORY FOR UPDATING TWIN THICKNESS
          twthres1=twthres(1,IPTM,IPHBOT+1)
          if(kkk.gt.(2*ngr(1))) twthres1=twthres(3,iptm,iphtop)
          AXISGR(0,3,KKK)=TWFRSY(IPTS-NSLSYX,KGX)*TWTHRES1
     #                   *AXISGR(0,3,KKK)
          RATIO=AXISGR(0,2,KKK)/AXISGR(0,3,KKK)
          IF(RATIO.GT.20) AXISGR(0,3,KKK)=AXISGR(0,2,KKK)/20    !LIMIT THE ELLIPSOID RATIO TO BE GREATER THAN 50
                                                                !AVOID PROBLEM FOR ESHELBY TENSOR CALCULATION

C   **    THE LONGEST AXIS SHOULD BE THE 2ND ONE
          IF(AXISGR(0,1,KKK).GT.AXISGR(0,2,KKK)) THEN
            EXCHANGE=AXISGR(0,1,KKK)
            AXISGR(0,1,KKK)=AXISGR(0,2,KKK)
            AXISGR(0,2,KKK)=EXCHANGE
            DO I=1,3
               EXCHANGE=AXISGR(1,I,KKK)
               AXISGR(1,I,KKK)=AXISGR(2,I,KKK)
               AXISGR(2,I,KKK)=EXCHANGE
            ENDDO
          ENDIF

C   **    MAKE THE SYSTEMS RIGHT HANDED
          DO I=1,3
             DO J=1,3
                AA(I,J)=AXISGR(I,J,KKK)
             ENDDO
          ENDDO

          IF(DET(AA).LE.0.) THEN
            DO I=1,3
               AXISGR(2,I,KKK)=-AXISGR(2,I,KKK)
            ENDDO
           ENDIF

          CREATE_TWIN(KKK)=1.

         ELSE  !UPDATE THE THICKNESS OF THE ELLIPSOID AND GIVE SHAPE TO PARENT IF NEEDED

           THRES1=TWTHRES(1,IPTM,IPHBOT)
           IF(KKK.GT.2*NGR(IPHBOT)) THRES1=TWTHRES(3,IPTM,IPHBOT)
           AXISGR(0,3,KKK)=TWFRSY(IPTS-NSLSYX,KGX)
     #                  *THRES1*AXIS3(KKK)
           RATIO=AXISGR(0,2,KKK)/AXISGR(0,3,KKK)
           IF(RATIO.GT.20) AXISGR(0,3,KKK)=AXISGR(0,2,KKK)/20
           THRES2=TWTHRES(2,IPTM,IPHBOT)
           if(kkk.le.(2*ngr(1))) then !update shape of parent only if we have a 1st generation twin
             IF(TWFRSY(IPTS-NSLSYX,KGX).GE.THRES2) THEN
               if(create_parent(KGX).EQ.0) then
                  DO I=1,3
                  DO J=1,3
                     AXISGR(I,J,KGX)=AXISGR(I,J,KKK)  !GIVE TO PARENT SAME ELLIPSOID ORIENTATION AS TWIN
                  ENDDO
                  ENDDO
                  DO I=1,2
                    AXISGR(0,I,KGX)=AXISGR(0,I,KKK) !2 OF ELLIPSOID AXES HAVE SAME LENGTH FOR PARENT AND TWIN
                  ENDDO
                  create_parent(kgx)=1
               endif ! if(create_parent(KGX).EQ.0)
               AXISGR(0,3,KGX)=(1-TWFRSY(IPTS-NSLSYX,KGX))
     #                  *THRES1*AXIS3(KKK)
               RATIO=AXISGR(0,2,KGX)/AXISGR(0,3,KGX)
               IF(RATIO.GT.20) AXISGR(0,3,KGX)=AXISGR(0,2,KGX)/20
             ENDIF !  IF(TWFRSY(IPTS,KGX).GE.THRES)
           endif !  if(kkk.le.(2*ngr(1)))
         ENDIF ! IF(CREATE_TWIN(KKK).EQ.0)
       ENDIF ! IF(IOPT.EQ.0)

      RETURN
      END


C ************************************************************************
C     SUBROUTINE TWIN_BARRIERS      --->   VERSION MAY/11/2007
C
C     CALCULATES THE MEAN FREE PATH 'SYSMFP(is,it)' THAT TWIN SYSTEM 'it'
C     DEFINES FOR OTHER SLIP OR TWIN SYSTEMS 'is'.
C     -->   Dmfp = Dsep * SYSMFP      and Dmfp < Dg
C           Dsep: SEPARATION OF TWIN LAMELLAE / Dg: GRAIN SIZE.
C     CALCULATES TRANSMISSIBILITY FACTORS 'TRANSFER(is,it)' ACROSS TWINS.
C ************************************************************************

      SUBROUTINE TWIN_BARRIERS

      INCLUDE 'VPSC7.DIM'

C     DIMENSION DINTER(3),TWINBARRIER(2)
      DIMENSION DN1(3),DB1(3),DN2(3),DB2(3),DNT(3),DBT(3),DL1(3),DL2(3)
      DIMENSION AA(3,3),ATWIN(3,3),SCH1(3,3),SCH2(3,3)
      DIMENSION BARRIER(NSYSMX,NSYSMX,NTWSMX),TRANSFER(NSYSMX,NTWSMX)

        IPHEL=1
        NSLSYX=NSLSYS(IPHEL)

        WRITE(10,*)
        WRITE(10,'(''    SYSMFP(IS,IT)'')')
ccc     WRITE(10,'('' IT'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        DO IT=NSLSYX+1,NSYST(IPHEL)
          ITX=IT-NSLSYX
          DO I=1,3
            DNT(I)=DNCA(I,IT,IPHEL)
          ENDDO
          DO IS=1,NSYST(IPHEL)
            DO I=1,3
C             DB1(I)=DBCA(I,IS,IPHEL)
              DN1(I)=DNCA(I,IS,IPHEL)
            ENDDO
            COSALF=ABS(DNT(1)*DN1(1)+DNT(2)*DN1(2)+DNT(3)*DN1(3))
            IF(COSALF.GT.1.) COSALF=1.
            ALF=ACOS(COSALF)
            COSBET=COS(PI/2.-ALF)
            IF(COSBET.LE.0.01) COSBET=0.01 ! coplanar (including self) systems
C           SYSMFP(IS,ITX)=   COSBET       ! old 'wrong' definition
            SYSMFP(IS,ITX)=1./COSBET       ! inverted 4/4/2005

C           DINTER(1)=DN1(2)*DNT(3)-DN1(3)*DNT(2)
C           DINTER(2)=DN1(3)*DNT(1)-DN1(1)*DNT(3)
C           DINTER(3)=DN1(1)*DNT(2)-DN1(2)*DNT(1)
C           GAM=DB1(1)*DINTER(1)+DB1(2)*DINTER(2)+DB1(3)*DINTER(3)
C           GAM=ACOS(ABS(GAM))
C           COSDEL=COS(PI/2.-GAM)
C           IF(COSDEL.LE.0.01) COSBET=0.01
C           SYSMFP(IS,ITX)=COSBET*COSDEL     ! old 'wrong' definition
C           SYSMFP(IS,ITX)=1./(COSBET*COSDEL)

          ENDDO
ccc       WRITE(10,'(I3,2X,30F6.3)')
ccc  #          IT,(SYSMFP(IS,ITX),IS=1,NSYST(IPHEL))
        ENDDO

C *** CALCULATES CRITERION OF 'SLIP-TRANSFER' ACROSS TWIN BOUNDARIES.
C *** FOR EVERY TWINNING PLANE FINDS THE INTERSECTION OF ALL SLIP AND
C *** TWINNING PLANES. DEFINES A TRANSFER FACTOR FOR EVERY PAIR.

        ISLIPTR=0
C       WRITE(*,*) 'ENTER 0 TO SUPPRESS SLIP TRANSFER ACROSS INTERFACE'
C       WRITE(*,*) 'ENTER 1 FOR CHALMERS SLIP TRANSFER'
C       WRITE(*,*) 'ENTER 2 FOR SHEN SLIP TRANSFER  --> '
C       READ(*,*)   ISLIPTR

        TINY=1.E-4
        IPHEL=1
        NSLSYX=NSLSYS(IPHEL)

C       WRITE(10,*)
C       WRITE(10,'(''        BARRIER(IS1,IS2,IT)'')')
C       WRITE(10,'(''  IT IS1'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        DO IT=NSLSYX+1,NSYST(IPHEL)
          ITX=IT-NSLSYX
          DO I=1,3
            DNT(I)=DNCA(I,IT,IPHEL)
            DBT(I)=DBCA(I,IT,IPHEL)
          ENDDO
          CALL TWIN_ORIENTATION (DBT,ATWIN)   ! ATWIN transf from TW to CR

          DO IS1=1,NSYST(IPHEL)
            INDEX=1
            PROD1=0.
            DO I=1,3
              DN1(I)=DNCA(I,IS1,IPHEL)
              DB1(I)=DBCA(I,IS1,IPHEL)
              PROD1=PROD1+DN1(I)*DNT(I)
              AA(1,I)=DN1(I)
              AA(2,I)=DNT(I)
              IF(ABS(DN1(I))+ABS(DNT(I)) .LT. TINY) INDEX=I
              DL1(I)=(I/3)*(3/I)
              DO J=1,3
                SCH1(I,J)=DN1(I)*DB1(J)+DN1(J)*DB1(I)
              ENDDO
            ENDDO
            SNORM1=TNORM(SCH1,3,3)
            PROD1=ABS(PROD1)

            IF (ABS(PROD1-1.) .GT. TINY) THEN     ! avoid N1 // NT
              DO I=1,3
                AA(3,I)=(I/INDEX)*(INDEX/I)
              ENDDO
              CALL LU_EQSYSTEM(AA,DL1,3,IER)     ! intersection of IS1 w/IT
              DO I=1,3
                DNORM=SQRT(DL1(1)**2+DL1(2)**2+DL1(3)**2)
              ENDDO
              DO I=1,3
                DL1(I)=DL1(I)/DNORM
              ENDDO
            ENDIF

C           write(10,'('' it is1 dnt dn1 dl1'',2i3,9f8.3)')
C    #            it,is1,(dnt(i),i=1,3),(dn1(i),i=1,3),(dl1(i),i=1,3)

            DO IS2=1,NSYST(IPHEL)
              INDEX=1
              PROD2=0.
              DO I=1,3
                DN2(I)=0.
                DB2(I)=0.
                DO J=1,3
                  DN2(I)=DN2(I)+ATWIN(J,I)*DNCA(J,IS2,IPHEL)
                  DB2(I)=DB2(I)+ATWIN(J,I)*DBCA(J,IS2,IPHEL)
                ENDDO
                PROD2=PROD2+DN2(I)*DNT(I)
                AA(1,I)=DN2(I)
                AA(2,I)=DNT(I)
                IF(ABS(DN2(I))+ABS(DNT(I)) .LT. TINY) INDEX=I
                DL2(I)=(I/3)*(3/I)
                DO J=1,3
                  SCH2(I,J)=DN2(I)*DB2(J)+DN2(J)*DB2(I)
                ENDDO
              ENDDO
              SNORM2=TNORM(SCH2,3,3)
              PROD2=ABS(PROD2)

              IF (ABS(PROD2-1.) .GT. TINY) THEN     ! avoid N2 // NT
                DO I=1,3
                  AA(3,I)=(I/INDEX)*(INDEX/I)
                ENDDO
                CALL LU_EQSYSTEM(AA,DL2,3,IER)     ! intersection of IS2 w/IT
                DO I=1,3
                  DNORM=SQRT(DL2(1)**2+DL2(2)**2+DL2(3)**2)
                ENDDO
                DO I=1,3
                  DL2(I)=DL2(I)/DNORM
                ENDDO
              ENDIF

              IF (ABS(PROD1-1.).GT.TINY.AND.ABS(PROD2-1.).GT.TINY) THEN
                BARRIERB=0.
                BARRIERL=0.
                BARRIERN=0.
                BARRIERS=0.
                DO I=1,3
                  BARRIERB=BARRIERB+DB1(I)*DB2(I)
                  BARRIERL=BARRIERL+DL1(I)*DL2(I)
                  BARRIERN=BARRIERN+DN1(I)*DN2(I)
                  DO J=1,3
                    BARRIERS=BARRIERS+SCH1(I,J)*SCH2(I,J)/SNORM1/SNORM2
                  ENDDO
                ENDDO
                IF (ISLIPTR.EQ.0) BARRIER(IS1,IS2,ITX)=0.    ! perfect barrier
                IF (ISLIPTR.EQ.1)
     #            BARRIER(IS1,IS2,ITX)=ABS(BARRIERS)          ! Liv. & Chalm.
                IF (ISLIPTR.EQ.2)
     #            BARRIER(IS1,IS2,ITX)=ABS(BARRIERL*BARRIERB)  ! Shen et al.

C               write(10,'('' it is2 dnt dn2 dl2'',2i3,9f8.3,3x,2f8.3)')
C    #            it,is2,dnt,dn2,dl2,barrierl,barrierb
              ELSE
                BARRIER(IS1,IS2,ITX)=0.

C               write(10,'('' it is2 dnt dn2'',2i3,6f8.3,3x,2f8.3)')
C    #            it,is2,dnt,dn2,prod1,prod2
              ENDIF

            ENDDO      ! end of DO loop over IS2

C           WRITE(10,'(2I4,2X,31F6.3)')
C    #             IT,IS1,(BARRIER(IS1,IS2,ITX),IS2=1,NSYST(IPHEL)),BMAX

          ENDDO      ! end of DO loop over IS1
        ENDDO      ! end of DO loop over IT

C *** SEARCH FOR MAXIMUM 'TRANSFER FACTOR' OF 'IS1' ACROSS 'IT'.
C *** APPLIES TO CRYSTALLOGRAPHICALLY EQUIVALENT SLIP SYSTEMS.
C *** SETS TO ZERO TRANSFER OF TWINNING SYSTEMS.

        WRITE(10,*)
        WRITE(10,'(''      TRANSFER(IS1,IT)'')')
ccc     WRITE(10,'(''  IT'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        NSLSYX=NSLSYS(IPHEL)
        DO IT=1,NTWSYS(IPHEL)
          I=0
          DO IM1=1,NMODES(IPHEL)
          DO IS1=1,NSM(IM1,IPHEL)
            I=I+1
            J=0
            BMAX=0
            DO IM2=1,NMODES(IPHEL)
            DO IS2=1,NSM(IM2,IPHEL)
              J=J+1
C --> TRANSFER ALLOWED ONLY AMONG SYSTEMS OF THE SAME MODE
              IF(IM2.EQ.IM1) THEN
                BAUX=BARRIER(I,J,IT)
                IF (BAUX.GT.BMAX) BMAX=BAUX
              ENDIF
            ENDDO
            ENDDO
            TRANSFER(I,IT)=BMAX
          ENDDO
          ENDDO
C *** ENFORCES NO TRANSFER OF TWIN THROUGH TWIN, EXCEPT 'SELF'
          DO IT1=NSLSYX+1,NSYST(IPHEL)
            TRANSFER(IT1,IT)=0.
            IF(IT+NSLSYX.EQ.IT1) TRANSFER(IT1,IT)=1.
          ENDDO
ccc       WRITE(10,'(I4,2X,30F6.3)')
ccc  #             IT,(TRANSFER(IS1,IT),IS1=1,NSYST(IPHEL))
        ENDDO
        TRANSFER(1,1)=TRANSFER(1,1)      ! to fool the compiler

        RETURN
        END
        
c*****************************************************************************c        
c*****************************************************************************c        
        
c*****************************************************************************c
c                                                                                        c
cc Laurent add ins                                                                       c
c                                                                                        c
c*****************************************************************************c
c    UPDATE_TWINNIN_CG THIS IS THE OLD SUB ROUTINE
c    UPDATE_CRSS_CG_DISL 
c    UPDATE_SHAPE_CG THIS IS SIMPLY THE NEW SUB ROUTINE WITH AN EXTRA CONDITION TO AVOID
C                    REDUNDANCIES IN THE
c*****************************************************************************c
C **********************************************************************
C    SUB ROUTINE UPDATE_TWINNING_CG  ---> VERSION OF August 2006
C
C    SCANS TWIN SYSTEMS, CALCULATES ACCUMULATED TWIN FRACTIONS.
C    TAGS THE PREDOMINANT TWIN SYSTEM 'PTS' IN EACH GRAIN and child.
C    CREATES CHILD and grand-child AND DEFINES ITS ORIENTATION
C    AS TWIN RELATED TO PARENT IF TWFRPTS EXCEEDS THRES1.
C    TRANSFERS VOL FRACTION FROM PARENT TO CHILD IF THRE1<TWFPTS<THRE2.
C    DEACTIVATES TRANSFER IF TWFPTS>THRE2.
C **********************************************************************
C
C    TWFRSY(TWS,KKK): ACCUMMULATED TWIN VOLUME FRACTION IN SYSTEM TWS
C                     IN GRAIN KKK (RELATIVE TO CG VOLUME).
C    TWFRGR(TWM):     ACCUMULATED TWIN FRACT IN EACH MODE IN GRAIN KKK
C                     IN THE STEP (RELATIVE TO CG VOLUME).
C    TWFRPH(TWM,IPH): ACCUMULATED TWIN FRACT IN EACH MODE OF EACH 'PHASE'
C                     (PARENT OR CHILD) (ABSOLUTE VOLUME FRACTION)
C    KTWSMX(KKK)=0 --> NO 'PTS' TAGGED IN THIS GRAIN.
C    KTWSMX(KKK)>0 --> INDEX OF THE TWINNING SYSTEM WITH MAXIMUM
C                      TWIN VOLUME FRACTION IN THE GRAIN
C    KTWSMX(KKK)<0 --> GRAIN REACHED MAX TWIN FRACTION AND IS UNTAGGED
C    KTWMMX(KKK) :    INDEX OF TWINNING MODE TO WHOM KTWSMX(KKK) BELONGS.

C **********************************************************************

      SUBROUTINE UPDATE_TWINNING_CG_ZR2 (IRESET)

      INCLUDE 'VPSC7.DIM'

      DIMENSION BTWIN(3),ATWIN(3,3)
      DIMENSION TWFRGR(NTWMMX)
      REAL WGT_CHILD
      
c      DIMENSION IPTSGR(NGRPEL),IPTMGR(NGRPEL)    
      DIMENSION IPTMGR(NGRPEL)   
c      Dimension KTWSMX(NGRMX),KTWMMX(NGRMX)           
      

      write(10,'(/,'' ENTERING UPDATE_TWINNING_CG'')')
      iprint=1   ! controls diagnostic print-out

C **************************************************************************
C    THIS DO LOOP SCANS ALL GRAINS IN EACH PHASE/ELEMENT. IN EACH GRAIN
C    TWINNING STATISTICS ARE PERFORMED AND TAGGING FOR REORIENTATION IS DONE.
C **************************************************************************
      KGX=1
      DO IPH=IPHBOT,IPHTOP
        IPHEL=IPH-IPHBOT+1
        NSLSYX=NSYST(IPHEL)-NTWSYS(IPHEL)
      DO KKK=NGR(IPH-1)+1,NGR(IPH)

        ngp=1
        ngc=ngp+ngr(1)
        iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

c ** change the weight of the composite grain according to the phase
        IF(IPHEL.EQ.1) THEN
           WGTCG=WGT(KKK)+WGT(KKK+NGR(1))
           IF(IPHTOP.GT.2) WGTCG=WGTCG+WGT(KKK+2*NGR(1))
        ELSE IF(IPHEL.EQ.2) THEN
           WGTCG=WGT(KKK)
           IF(IPHTOP.GT.2) WGTCG=WGTCG+WGT(KKK+NGR(1))
        ELSE IF(IPHEL.EQ.3) THEN
              WGTCG=WGT(KKK)
        ENDIF
        IPTSGR(KGX)=0
        IPTMGR(KGX)=0

        IF(WGT(KKK).LT.1.E-6) GO TO 1000      ! SKIPS UNBORN CHILDREN
        TWFMAX=0.
        ITS=NSLSYX
        KTS=0
        DO ITM=1,NTWMOD(IPHEL)
          KK=ITM+NMODES(IPHEL)-NTWMOD(IPHEL)
          GMODE=0.
          TWFRGR(ITM)=0.
          TWSHX=TWSH(ITM,IPHEL)
          DO NSMX=1,NSM(KK,IPHEL)
            KTS=KTS+1             ! shifted counter for twin systems
            ITS=ITS+1             ! absolute counter over all systems
            IF(GAMDOT(ITS,KGX).LT.0.) PAUSE 'NEGATIVE TWIN SHEAR !!'
            GSYST=GAMDOT(ITS,KGX)*TINCR
c			PRINT*,'GAMDOT_1',GAMDOT(ITS,KGX),ITS,KGX
            GMODE=GMODE+GSYST
            TWFRINC=GSYST/TWSHX*WGT(KKK)/WGTCG
            TWFRSY(KTS,KKK)=TWFRSY(KTS,KKK)+TWFRINC
            TWFRGR(ITM)=TWFRGR(ITM)+TWFRSY(KTS,KKK)   ! relative to CG

C *** TRANSFER FRACTION BACK TO PARENT IF PTS IS ACTIVE IN CHILD (DETWINNING)
C *** DETWINNING NOT ALLOWED IN PARENT IF THERE IS A GRAND-CHILD
           M_CONDITION=0
           IF(IPHEL.EQ.3) M_CONDITION=1 !put 1 if you want to allow detwinning of primary twins, 0 otherwise
           IF((IPHEL.EQ.2).AND.(KTWSMX(KKK).EQ.0)) M_CONDITION=1
           IF(M_CONDITION.EQ.1) THEN
              IF(ITS.EQ.ABS(KTWSMX(KKK-NGR(1)))) THEN
                KKX=KGX-NGR(1)          ! child # relative to element
                TWFRSY(KTS,KKX)=TWFRSY(KTS,KKX)-GAMDOT(ITS,KKK)*TINCR/
     #                                        TWSHX*WGT(KKK)/WGTCG
                if (twfrsy(kts,kkx).lt.0) twfrsy(kts,kkx)=0
              ENDIF
           ENDIF

           IF (TWFRSY(KTS,KKK).GT.TWFMAX) THEN
              IPTSGR(KGX)=ITS
              IPTMGR(KGX)=ITM
              IPTSGR_MZ(KGX)=ITS !MZ added global array
              IPTMGR_MZ(KGX)=ITM !MZ added global array
              TWFMAX=TWFRSY(KTS,KKK)
           ENDIF
          ENDDO      ! END OF LOOP OVER TWIN SYSTEMS IN EACH MODE

          TWFRPH(ITM,IPH)=TWFRPH(ITM,IPH)+(GMODE/TWSHX)*WGT(KKK)

        ENDDO      ! END OF LOOP OVER TWIN MODES IN THE PHASE

        if(iprx.eq.1) write(10,'(''  kkk,iptm,ipts,twfmax,wgtcg'',
     #         3i5,2e12.4)') kkk,iptmgr(kgx),iptsgr(kgx),twfmax,wgtcg

 1000 KGX=KGX+1
      ENDDO
      ENDDO

C **************************************************************************************************
C *** THE FOLLOWING RUNS ONLY FOR PARENT BUT ALSO SETS PARAMETERS FOR CHILD AND GRAND-CHILD
C **************************************************************************************************

      TWINTHRES=TWINTHRES_MZ     ! HARDWIRES MIN FRACTION REQUIRED FOR DEFINING PTS-NEW !MZ read from input

C *** IF TWIN FRACT EXCEEDS MIN THRESHOLD BUT GRAIN HAS NOT BEEN TAGGED YET:
C     *  TAGS THE 'Predominant Twin System'(PTS) IN PARENT BY
C        STORING ITS INDEX IN 'KTWSMX(KKK)'. (TAG REMAINS EVEN IF ANOTHER
C        TWIN SYSTEM BECOMES PREDOMINANT LATER).
C     *  CREATES CHILD AND DEFINES ITS ORIENTATION AS TWIN RELATED TO PARENT.
C        (CO-ROTATION SCHEME WILL TAKE CARE OF THE PARENT-CHILD REORIENTATION)
C     *  DO A SIMILAR TAGGING PROCESS FOR CHILD AND GRAND-CHILD
C        THE GRAND-CHILD WILL BE TAGGED AS THE CHILD


      KGX=1
C *** MODIFY THIS SECTION TO TAKE INTO ACCOUNT 2ND GENERATION TWINS (GP)
      iseed=-1 !MZ
      IPH=IPHBOT
      IPHEL=IPH-IPHBOT+1
      NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)
      DO KKK=NGR(IPH-1)+1,NGR(IPH)

        ngp=1
        ngc=ngp+ngr(1)
        iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

        KKP=KKK+NGR(1) !CHILD
        KKP2=KKP+NGR(1) !GRAND-CHILD
        IPTS=IPTSGR(KKK)
        IPTM=IPTMGR(KKK)
        TWFRPTS=0.
        IF(IPTS.NE.0) THEN
          IF(KTWSMX(KKK).EQ.0) TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
          IF(KTWSMX(KKK).GT.0) TWFRPTS=TWFRSY(KTWSMX(KKK)-NSLSYX,KKK)

C *** CHECKS IF THE PTS IS DETWINNING AND SHRINKS BELOW THE THRESHOLD.
C *** UNTAGS THE GRAIN AND ELIMINATES THE CHILD
C *** ONLY CHILD WITHOUT GRAND-CHILD IS ALLOWED TO DETWIN
          IF((KTWSMX(KKK).GT.0).AND.(TWFRPTS.LT.TWINTHRES)) THEN
           IF(KTWSMX(KKP).EQ.0) THEN
             KTWSMX(KKK)=0
             KTWMMX(KKK)=0
             ICOMPGR(KKK)=0
             WGT(KKK)=WGT(KKK)+WGT(KKP)
             WPH(1)=WPH(1)+WGT(KKP)
             WPH(2)=WPH(2)-WGT(KKP)
             WGT(KKP)=0.
c ***      when the grain detwins, the twin volume fraction in that twin grain should be put back to 0
             do i=1,NTWSYS(2)
                 twfrsy(i,kkp)=0
             enddo
             DO I=1,NSYST(2)
                GAMDOT(I,KKP)=0
             ENDDO
             create_twin(kkp)=0
           ENDIF
          ENDIF
        ENDIF      ! end of if(ipts.ne.0)
       abba=-1 !MZ commented out
C *** IF GRAIN HASN'T BEEN TAGGED YET CREATE A CHILD
        aMZ = ran2(iseed)!MZ test
        IF(KTWSMX(KKK).EQ.0 .AND.
     #    (TWFRPTS*0.2+0.8*TWINTHRES*ran2(iseed)).GE.TWINTHRES) THEN !MZ changed

          if(iprx.eq.1) write(10,'('' TAGGING: kkk,iptm,ipts,twfmax'',
     #                  3i5,e12.4)')         kkk,iptm,ipts,twfmax

          KTWSMX(KKK)=IPTS
          KTWMMX(KKK)=IPTM

          DO I=1,3
            BTWIN(I)=DBCA(I,IPTS,IPH)
          ENDDO
          CALL TWIN_ORIENTATION (BTWIN,ATWIN)
          DO I=1,3
          DO J=1,3
            AG(I,J,KKP)=0.
            DO K=1,3
              AG(I,J,KKP)=AG(I,J,KKP)+AG(I,K,KKK)*ATWIN(K,J)
            ENDDO
          ENDDO
          ENDDO

C *** SETS THE CRSS (FOREST) IN THE NEWBORN CHILD EQUAL TO THE CRSS IN PARENT
C     TIMES A FACTOR ASSOCIATED WITH BASINSKY MECHANISM.

          HBASINSKI=1.0 !MZ basinski was 1.0
          NSLSYS=NSYST(2)-NTWSYS(2)
c------------------------------------------c          
          DO ISS=1,NSLSYX
          TAUE(ISS,KKP) =HBASINSKI*TAUE(ISS,KKK)
c          if(kkk.eq.500)write(10,*) 'bas',taue(iss,kkp),hbasinski
c          write(10,*) 'bas1',kkp,taue(iss,kkp),hbasinski
c--------Save the dislocation density---------c          
          RHO_S(ISS,KKP)=RHO_S(ISS,KKK)
          RHO_DEB(KKP)=RHO_DEB(KKK) 
c--------Save the dislocation density---------c  
          ENDDO
         
          HBASINSKI=1.0 !MZ basinski was 1.0
          DO ITS=NSLSYX+1,NSYST(2)
          TAUE(ITS,KKP) =HBASINSKI*TAUE(ITS,KKK)
          ENDDO
          
C *** INCR CRSS OF 'PTS' IN CHILD TO PREVENT 'DETWINNING'
ccc       TAUE(IPTS,KKP)=50.*TAUE(IPTS,KKK)

        ENDIF      ! END OF IF KTWSMX.EQ.0 .AND...

C *** IF KTWSMX(KKK).GT.0  --> GRAIN IS STILL TWIN REORIENTING.
C *** TWFRPTS<THRES2: THIS CONDITION IS NOT NECESSARY ANYMORE B/C WE STOP THE TWIN SYSTEM TO REACHED FRACTION ABOVE THRES2 (GP)
C *** TRANSFER FRACTION FROM PARENT TO CHILD
C *** IF IPHTOP>2, LOOK IF THERE IS A GRAND-CHILD
        IF(KTWSMX(KKK).GT.0) THEN
          IPTS=KTWSMX(KKK)
          IPTM=KTWMMX(KKK)
          WGTCG=WGT(KKK)+WGT(KKP)
          IF(IPHTOP.GT.2) WGTCG=WGTCG+WGT(KKP2)
          TWFRPTS=TWFRSY(IPTS-NSLSYX,KKK)
          WPH(1) =WPH(1)-WGT(KKK)
          WGT(KKK)=WGTCG*(1.-TWFRPTS)
          WPH(1)  =WPH(1)+WGT(KKK)
          IF((IPHTOP.LE.2)
     #     .or.(TWFRPTS.LT.(0.98*TWTHRES(2,IPTM,IPHEL)))) THEN
            wph(2)=wph(2)-wgt(kkp)
            WGT(KKP)=WGTCG-WGT(KKK)
            wph(2)=wph(2)+wgt(kkP)
          ENDIF

C *** LOOK FOR PTS IN CHILD only if child fully formed
          if(IPTM.le.2) aMZ_THRES1=aMZ_THRES(1) !MZ sec twin
          if(IPTM.gt.2) aMZ_THRES1=aMZ_THRES(2) !MZ sec twin
          IF((IPHTOP.GT.2)
     #       .AND.(TWFRPTS.GE.(aMZ_THRES1*TWTHRES(2,IPTM,IPHEL)))) THEN!MZ was 0.98*TWTHRES
            IPTS_CHILD=IPTSGR(KKP)
            IPTM_CHILD=IPTMGR(KKP)
            TWFRPTS_CHILD=0.
            IF(IPTS_CHILD.NE.0) THEN
               IF(KTWSMX(KKP).EQ.0)
     #             TWFRPTS_CHILD=TWFRSY(IPTS_CHILD-NSLSYX,KKP)
               IF(KTWSMX(KKP).GT.0)
     #             TWFRPTS_CHILD=TWFRSY(KTWSMX(KKP)-NSLSYX,KKP)

C    IF YOU WANT TO ALLOWED DETWINNING FOR PRIMARY TWINS, UNCOMMENT THE FOLLOWING LINES ****
C *** CHECKS IF THE PTS IS DETWINNING AND SHRINKS BELOW THE THRESHOLD.
C *** UNTAGS THE CHILD GRAIN AND ELIMINATES THE GRAND-CHILD
               IF(KTWSMX(KKP).GT.0.AND.TWFRPTS_CHILD.LT.TWINTHRES) THEN
                 KTWSMX(KKP)=0
                 KTWMMX(KKP)=0
                 KTWSMX(KKP2)=0
                 KTWMMX(KKP2)=0
                 WGT(KKP)=WGT(KKP)+WGT(KKP2)
                 WPH(2)=WPH(2)+WGT(KKP2)
                 WPH(3)=WPH(3)-WGT(KKP2)
                 WGT(KKP2)=0.
c ***          when the grain detwins, the twin volume fraction in that grain should be put back to 0
                 do i=1,NTWSYS(3)
                   twfrsy(i,kkp2)=0
                 enddo
                 DO I=1,NSYST(3)
                   GAMDOT(I,KKP2)=0
                 ENDDO
                 create_twin(kkp2)=0
               ENDIF

            ENDIF !IF(IPTS_CHILD.NE.0)

            IF(KTWSMX(KKP).EQ.0.AND.TWFRPTS_CHILD.GE.TWINTHRES) THEN
              KTWSMX(KKP)=IPTS_CHILD
              KTWMMX(KKP)=IPTM_CHILD
              KTWSMX(KKP2)=IPTS_CHILD
              KTWMMX(KKP2)=IPTM_CHILD

              DO I=1,3
                BTWIN(I)=DBCA(I,IPTS_CHILD,IPH)
              ENDDO
              CALL TWIN_ORIENTATION (BTWIN,ATWIN)
              DO I=1,3
              DO J=1,3
                 AG(I,J,KKP2)=0.
                 DO K=1,3
                   AG(I,J,KKP2)=AG(I,J,KKP2)+AG(I,K,KKP)*ATWIN(K,J)
                 ENDDO
              ENDDO
              ENDDO

C *** SETS THE CRSS (FOREST) IN THE NEWBORN CHILD EQUAL TO THE CRSS IN PARENT
C     TIMES A FACTOR ASSOCIATED WITH BASINSKY MECHANISM.
              HBASINSKI=3.0
              NSLSYS=NSYST(3)-NTWSYS(3)
              DO ISS=1,NSLSYX
               TAUE(ISS,KKP2) =HBASINSKI*TAU(ISS,0,2)     ! reset crss to tau0 (may 04 05)
          if(kkp2.eq.500)write(10,*) 'bas',taue(iss,kkp2),hbasinski
          if(kkp2.eq.2500)write(10,*) 'bas1',taue(iss,kkp2),hbasinski
              ENDDO
              DO ITS=NSLSYX+1,NSYST(2)
               TAUE(ITS,KKP2) =HBASINSKI*TAU(ITS,0,2)     ! reset crss to tau0 (may 04 05)
              ENDDO

            ENDIF      ! END OF IF KTWSMX.EQ.0 .AND...

            IF(KTWSMX(KKP).GT.0) THEN
                IPTS_CHILD=KTWSMX(KKP)
                IPTM_CHILD=KTWMMX(KKP)
                WGT_CHILD=WGTCG-WGT(KKK)
                TWFRPTS_CHILD=TWFRSY(IPTS_CHILD-NSLSYX,KKP)
                WPH(2) =WPH(2)-WGT(KKP)
                WGT(KKP)=WGT_CHILD*(1.-TWFRPTS_CHILD)
                WPH(2)  =WPH(2)+WGT(KKP)
                WGT(KKP2)=WGT_CHILD-WGT(KKP)
                WPH(3)  =1-(wph(1)+wph(2))
            ELSE
              WPH(2)=WPH(2)-WGT(KKP)
              WGT(KKP)=WGTCG-WGT(KKK)
              WPH(2)=WPH(2)+WGT(KKP)
            ENDIF

          ENDIF !IF((IPHTOP.GT.2).AND.....)

          if(iprx.eq.1) then
            write(10,'('' updates weight & ipts of twinned grain'')')
            write(10,'('' kkk,ipts,TWFRPTS'',2i5,e12.4)')
     #                    kkk,ipts,TWFRPTS
            write(10,'('' wgt(kkk) & wgt(kkp)'',2e12.4)')
     #                    wgt(kkk),wgt(kkp)
          endif


          if(iprx.eq.1) write(10,'('' updated weights  '',2e12.4)')
     #                             wgt(kkk),wgt(kkp)

          IF(TWFRPTS.GE.TWTHRES(2,IPTM,IPHEL)) THEN
            ICOMPGR(KKK)= ICOMPGRAIN
          ENDIF
        ENDIF      ! END OF IF(KTWSMX(kkk).GT.0)

      KGX=KGX+1

      ENDDO      ! END OF DO KKK


      RETURN
      END



C *****************************************************************************
C *****************************************************************************
c      this is the subroutine that updates the crss and that takes into account 
c      dislocation activities
*******************************************************************************


      SUBROUTINE UPDATE_CRSS_CG_disl (IOPTION)

      INCLUDE 'VPSC7.DIM'


      DIMENSION DN1(3),DB1(3),DN2(3),DB2(3),DNT(3),DBT(3),DL1(3),DL2(3)
      DIMENSION AA(3,3),ATWIN(3,3),SCH1(3,3),SCH2(3,3)
      DIMENSION BARRIER(NSYSMX,NSYSMX,NTWSMX),TRANSFER(NSYSMX,NTWSMX),
     #          DMFP(NSYSMX),DMFP_HP(NSYSMX)
     
c      DIMENSION IPTSGR(NGRPEL),IPTMGR(NGRPEL)
      DIMENSION IPTMGR(NGRPEL)
      INTEGER CONDITION,PREDOM
      
      REAL*8 RHO_W(NMODMX,NGRMX),XGMOD
      REAL*8 EDOT,BURG(4)
      REAL*8 ACTENER(4)
      REAL*8 BOLTZ,EREF
      REAL*8 KOVB3(4)
      REAL*8 ACRIT(4),TCRIT(4),YCRIT(4)
      REAL*8 APROP(4),TPROP(4),YPROP(4)
      REAL*8 TAU_PROP(4),TAU_CRIT(4)
      real*8 Q1(4),Q2(4),Q0(4),Q3(4)
      INTEGER NFLAG,NXMODE(50),MNM
      REAL*8 KGENER(4)
      REAL*8 EDOTESO(4)
      REAL*8 RHO_INITS(4),RHO_INITW(4),ATAU(4)
      REAL*8 ALPHA_INACT,DRHOS,DRHOW
      REAL*8 DELRHOS(4),DELRHOW(4)
      REAL*8 DELGAM(4),DTAU(4),TLATENT1(4,4)
      real*8 RHOSAT(4),K2(4),TLATENT(4,4),DRAG(4)
      real*8 HPK0(4),HPK1(4),HPSFE(4)
      real*8 HPK2(4),Q5(4),Q6,BURG_TW(2)
      REAL*8 YONE(4),BONE(4),SONE(4)
      character*17 identifiant
 
      iprint=1   ! controls diagnostic print-out
      

      
      IF (IOPTION.EQ.1) THEN

          EDOT=SQRT(2./3.)*TNORM(DBAR,5,1)
          
C *** INITIALIZE THE CRSS IN ALL SYSTEMS OF EACH PHASE IN ALL GRAINS
C *** THE 'CHILD' PHASE HAS INITIAL ZERO WEIGHT, AND CRSS's WILL BE
C *** REDEFINED WHEN THE LAMELLAE ARE CREATED.

        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1
          NSLSYP=NSYST(IPHEL)-NTWSYS(IPHEL)      
          NSLIPMODES=NMODES(IPHEL)-NTWMOD(IPHEL)
                    
C       START READING IN THE MATERIAL INPUT PARAMETERS
          KTWINTOT=0
          DO KCNT=1,NSLIPMODES+1
            KTWINTOT=KTWINTOT+NSM(KCNT,IPHEL)
          ENDDO

        write(10,*) 'ktwintot=',ktwintot

          OPEN (UNIT=UR1,FILE=FILECRYS,STATUS='OLD')

          identifiant='blah'
         
28        continue          
          READ(UR1,'(A)') identifiant
          if (identifiant.EQ.'DISLOCATION MODEL') then
          goto 29
          else
          goto 28
          end if
          
29        continue  
     
          READ(UR1,*) XGMOD,BOLTZ,EREF
          READ(UR1,*) ALPHA_INACT, Q6
          DO 1054 KJO=1,NSLIPMODES
          READ(UR1,'(A)') PROSA
          READ(UR1,*) BURG(KJO),KOVB3(KJO),ACTENER(KJO)
          READ(UR1,*) KGENER(KJO),DRAG(KJO)
          READ(UR1,*) EDOTESO(KJO)
          READ(UR1,*) RHO_INITS(KJO),RHO_INITW(KJO)
          READ(UR1,*) YONE(KJO),BONE(KJO),SONE(KJO)
          read(UR1,*) (TLATENT(KJO,mo),mo=1,ntwmod(iphel))
          read(UR1,*) (TLATENT1(KJO,mo),mo=1,ntwmod(iphel))
          READ(UR1,*) HPK0(KJO),HPK1(KJO),HPK2(KJO)
          READ(UR1,*) Q0(KJO),Q1(KJO),Q2(KJO)
1054      ENDDO

          DO 1056 KJO=1,NTWMOD(IPHEL)
          READ(UR1,'(A)') PROSA
          READ(UR1,*) YCRIT(KJO),ACRIT(KJO),TCRIT(KJO)
          READ(UR1,*) YPROP(KJO),APROP(KJO),TPROP(KJO)
          READ(UR1,*) BURG_TW(KJO)
1056      ENDDO
              
        CLOSE(UR1)
                
C       BELOW, IF UNKNOWN, THEN SET TO ROOM TEMPERATURE VALUES       
        write(*,*) 'tempi', tempi
        IF(TEMPERAT.LE.0.0) TEMPERAT=tempi
        IF(EDOT.LE.0.0)     EDOT=EREF
        write(10,*) 'temperature',temperat
        write(10,*) RHO_INITS(1),RHO_INITW(1)
        
c------Just in case you want it--------------------------------------c        
C       MORE EXACT RELATIONSHIP BETWEEN SHEAR MOD AND TEMP FOR ZR
C       XGMOD=(41.04298-0.02939*TEMPERAT+
C     #    9.51267E-6*TEMPERAT^2.)*1.e+03
        XGMOD=(40.06-0.022*TEMPERAT)*1.e+03     
c        !!For Zr (IN MPA*1000=GPA)
c-----------------------------------------------------------------c 
                      
          DO KKK=NGR(IPH-1)+1,NGR(IPH)
          IPTS=ABS(KTWSMX(KKK))        

C       ASSIGNING INITIAL VALUES FOR SLIP          
          ISM=0
            DO KMO=1,NSLIPMODES
            
                IF(RHO_S(KMO,KKK).LE.RHO_INITS(KMO))THEN
                RHO_S(KMO,KKK)=RHO_INITS(KMO)
                ENDIF
                IF(RHO_DEB(KKK).LE.RHO_INITW(KMO))THEN
                RHO_DEB(KKK)=RHO_INITW(KMO)
                ENDIF

            ATAU(KMO)=YONE(KMO)+BONE(KMO)*EXP(-TEMPERAT/SONE(KMO))
            Q5(KMO)=Q0(KMO)+Q1(KMO)*log(1.+temperat/Q2(KMO)) 
cmk            Q5(KMO)=Q0(KMO)*exp(-Q3(KMO)*EDOT)+Q1(KMO)*
cmk     #          log(1.+temperat/Q2(KMO)) 
            if(kkk.eq.500) write(10,*) 'q5=',q5(kmo),kmo
            
c            if(kmo.eq.3)write(10,*)'q5for pyr',kmo,q5(kmo)
                                    
                K2(KMO)=ALPHA_INACT*BURG(KMO)/ACTENER(KMO)
     #           *(1.-KOVB3(KMO)*TEMPERAT/DRAG(KMO)
     #           *LOG(EDOT/EDOTESO(KMO)))
                RHOSAT(KMO)=K2(KMO)**(-2.0)
                    
                DO IS=1,NSM(KMO,IPHEL)
                    ISM=ISM+1
                    TAUE(ISM,KKK)=XGMOD*ALPHA_INACT
     #              *BURG(KMO)*(SQRT(RHO_S(KMO,KKK)))
     #              +ATAU(KMO)

                    
                    IF(RHO_DEB(KKK).GT.RHO_INITW(KMO))THEN
                    TAUE(ISM,KKK)=TAUE(ISM,KKK)
     #              -0.086/ALPHA_INACT*SQRT(RHO_DEB(KKK))
     #              *LOG(BURG(KMO)*SQRT(RHO_DEB(KKK)))
                    ENDIF                    
     
C           HALL-PETCH EFFECT PARAMETERS FOR SLIP
         
                HPFAC(ISM,IPHEL)=SQRT(BURG(KMO))*HPK0(KMO)
     #             *XGMOD
         
            TAUHP =HPFAC(ISM,IPHEL)/SQRT(GRSZE)
            CRSS(ISM,KKK)=TAUE(ISM,KKK)+TAUHP
                
                ENDDO            !IS
                ENDDO           !KMO
                
            
C       ASSIGNING INITIAL VALUES FOR TWINNING
                
            DO 1839 MNN=1,NTWMOD(IPHEL)
            TAU_PROP(MNN)=YPROP(MNN)+
     #         APROP(MNN)*EXP(-TEMPERAT/TPROP(MNN))
            TAU_CRIT(MNN)=YCRIT(MNN)+
     #         ACRIT(MNN)*EXP(-TEMPERAT/TCRIT(MNN))
1839      ENDDO

            DO IS=NSLSYP+1,NSYST(IPHEL)

C           WARNING:THE FOLLOWING KTWINTOT IS THE TWIN SYSTEM# AT THE END OF THE FIRST TWIN MODE
             IF(IS.LE.KTWINTOT)THEN
               JTWIN=1
             ELSE
               JTWIN=2
             ENDIF    

             RHOTEMP=0.0        
             TAUE(IS,KKK)=TAUE(IS,KKK)+TAU_PROP(JTWIN)+
     #        (TAU_CRIT(JTWIN)-TAU_PROP(JTWIN))
     #        *EXP(-RHOTEMP)

             CRSS(IS,KKK)=TAUE(IS,KKK)+HPFAC(IS,IPHEL)/SQRT(GRSZE)

            ENDDO  ! FOR IS
           
            ngg=699
            iprx=iprint*(ngg/kkk)*(kkk/ngg)+
     #           iprint*((ngg+ngr(1))/kkk)*(kkk/(ngg+ngr(1)))
            if(iprx.ne.0) then
              write(10,'('' initial CRSS for grain # '', i4)') kkk
ccc           write(10,'(12f6.0)') (crss(is,kkk),is=1,nsyst(iphel))
              write(10,*) (crss(is,kkk),is=1,nsyst(iphel))
            endif

          ENDDO     !KKK
        ENDDO       !IPHEL

C *** CALCULATES INVERSE OF THE MEAN FREE PATH THAT EACH TWINNING SYSTEM
C *** DEFINES FOR THE OTHER SLIP OR TWINNING SYSTEMS IN THE GRAIN
        IPHEL=1
        NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)

        WRITE(10,*)
        WRITE(10,'(''    SYSMFP(IS,IT)'')')
ccc     WRITE(10,'('' IT'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        DO IT=NSLSYP+1,NSYST(IPHEL)
          ITX=IT-NSLSYP
          DO I=1,3
            DNT(I)=DNCA(I,IT,IPHEL)
          ENDDO
          DO IS=1,NSYST(IPHEL)
            DO I=1,3
C             DB1(I)=DBCA(I,IS,IPHEL)
              DN1(I)=DNCA(I,IS,IPHEL)
            ENDDO
            COSALF=ABS(DNT(1)*DN1(1)+DNT(2)*DN1(2)+DNT(3)*DN1(3))
            IF(COSALF.GT.1.) COSALF=1.
            ALF=ACOS(COSALF)
            COSBET=COS(PI/2.-ALF)
            IF(COSBET.LE.0.01) COSBET=0.01 ! coplanar (including self) systems
            SYSMFP(IS,ITX)=1./COSBET       ! inverted 4/4/2005


          ENDDO
ccc       WRITE(10,'(I3,2X,30F6.3)')
ccc  #          IT,(SYSMFP(IS,ITX),IS=1,NSYST(IPHEL))
        ENDDO

C *** CALCULATES CRITERION OF 'SLIP-TRANSFER' ACROSS TWIN BOUNDARIES.
C *** FOR EVERY TWINNING PLANE FINDS THE INTERSECTION OF ALL SLIP AND
C *** TWINNING PLANES. DEFINES A TRANSFER FACTOR FOR EVERY PAIR.

        ISLIPTR=0
C       WRITE(*,*) 'ENTER 0 TO SUPPRESS SLIP TRANSFER ACROSS INTERFACE'
C       WRITE(*,*) 'ENTER 1 FOR CHALMERS SLIP TRANSFER'
C       WRITE(*,*) 'ENTER 2 FOR SHEN SLIP TRANSFER  --> '
C       READ(*,*)   ISLIPTR

        TINY=1.E-4
        IPHEL=1
        NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)

C       WRITE(10,*)
C       WRITE(10,'(''        BARRIER(IS1,IS2,IT)'')')
C       WRITE(10,'(''  IT IS1'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        DO IT=NSLSYP+1,NSYST(IPHEL)
          ITX=IT-NSLSYP
          DO I=1,3
            DNT(I)=DNCA(I,IT,IPHEL)
            DBT(I)=DBCA(I,IT,IPHEL)
          ENDDO
          CALL TWIN_ORIENTATION (DBT,ATWIN)   ! ATWIN transf from TW to CR

          DO IS1=1,NSYST(IPHEL)
            INDEX=1
            PROD1=0.
            DO I=1,3
              DN1(I)=DNCA(I,IS1,IPHEL)
              DB1(I)=DBCA(I,IS1,IPHEL)
              PROD1=PROD1+DN1(I)*DNT(I)
              AA(1,I)=DN1(I)
              AA(2,I)=DNT(I)
              IF(ABS(DN1(I))+ABS(DNT(I)) .LT. TINY) INDEX=I
              DL1(I)=(I/3)*(3/I)
              DO J=1,3
                SCH1(I,J)=DN1(I)*DB1(J)+DN1(J)*DB1(I)
              ENDDO
            ENDDO
            SNORM1=TNORM(SCH1,3,3)
            PROD1=ABS(PROD1)

            IF (ABS(PROD1-1.) .GT. TINY) THEN     ! avoid N1 // NT
              DO I=1,3
                AA(3,I)=(I/INDEX)*(INDEX/I)
              ENDDO
              CALL LU_EQSYSTEM(AA,DL1,3,IER)     ! intersection of IS1 w/IT
              DO I=1,3
                DNORM=SQRT(DL1(1)**2+DL1(2)**2+DL1(3)**2)
              ENDDO
              DO I=1,3
                DL1(I)=DL1(I)/DNORM
              ENDDO
            ENDIF

C           write(10,'('' it is1 dnt dn1 dl1'',2i3,9f8.3)')
C    #            it,is1,(dnt(i),i=1,3),(dn1(i),i=1,3),(dl1(i),i=1,3)

            DO IS2=1,NSYST(IPHEL)
              INDEX=1
              PROD2=0.
              DO I=1,3
                DN2(I)=0.
                DB2(I)=0.
                DO J=1,3
                  DN2(I)=DN2(I)+ATWIN(J,I)*DNCA(J,IS2,IPHEL)
                  DB2(I)=DB2(I)+ATWIN(J,I)*DBCA(J,IS2,IPHEL)
                ENDDO
                PROD2=PROD2+DN2(I)*DNT(I)
                AA(1,I)=DN2(I)
                AA(2,I)=DNT(I)
                IF(ABS(DN2(I))+ABS(DNT(I)) .LT. TINY) INDEX=I
                DL2(I)=(I/3)*(3/I)
                DO J=1,3
                  SCH2(I,J)=DN2(I)*DB2(J)+DN2(J)*DB2(I)
                ENDDO
              ENDDO
              SNORM2=TNORM(SCH2,3,3)
              PROD2=ABS(PROD2)

              IF (ABS(PROD2-1.) .GT. TINY) THEN     ! avoid N2 // NT
                DO I=1,3
                  AA(3,I)=(I/INDEX)*(INDEX/I)
                ENDDO
                CALL LU_EQSYSTEM(AA,DL2,3,IER)     ! intersection of IS2 w/IT
                DO I=1,3
                  DNORM=SQRT(DL2(1)**2+DL2(2)**2+DL2(3)**2)
                ENDDO
                DO I=1,3
                  DL2(I)=DL2(I)/DNORM
                ENDDO
              ENDIF

              IF (ABS(PROD1-1.).GT.TINY.AND.ABS(PROD2-1.).GT.TINY) THEN
                BARRIERB=0.
                BARRIERL=0.
                BARRIERN=0.
                BARRIERS=0.
                DO I=1,3
                  BARRIERB=BARRIERB+DB1(I)*DB2(I)
                  BARRIERL=BARRIERL+DL1(I)*DL2(I)
                  BARRIERN=BARRIERN+DN1(I)*DN2(I)
                  DO J=1,3
                    BARRIERS=BARRIERS+SCH1(I,J)*SCH2(I,J)/SNORM1/SNORM2
                  ENDDO
                ENDDO
                IF (ISLIPTR.EQ.0) BARRIER(IS1,IS2,ITX)=0.    ! perfect barrier
                IF (ISLIPTR.EQ.1)
     #            BARRIER(IS1,IS2,ITX)=ABS(BARRIERS)          ! Liv. & Chalm.
                IF (ISLIPTR.EQ.2)
     #            BARRIER(IS1,IS2,ITX)=ABS(BARRIERL*BARRIERB)  ! Shen et al.

C               write(10,'('' it is2 dnt dn2 dl2'',2i3,9f8.3,3x,2f8.3)')
C    #            it,is2,dnt,dn2,dl2,barrierl,barrierb
              ELSE
                BARRIER(IS1,IS2,ITX)=0.

C               write(10,'('' it is2 dnt dn2'',2i3,6f8.3,3x,2f8.3)')
C    #            it,is2,dnt,dn2,prod1,prod2
              ENDIF

            ENDDO      ! end of DO loop over IS2

C           WRITE(10,'(2I4,2X,31F6.3)')
C    #             IT,IS1,(BARRIER(IS1,IS2,ITX),IS2=1,NSYST(IPHEL)),BMAX

          ENDDO      ! end of DO loop over IS1
        ENDDO      ! end of DO loop over IT

C *** SEARCH FOR MAXIMUM 'TRANSFER FACTOR' OF 'IS1' ACROSS 'IT'.
C *** APPLIES TO CRYSTALLOGRAPHICALLY EQUIVALENT SLIP SYSTEMS.
C *** SETS TO ZERO TRANSFER OF TWINNING SYSTEMS.

        WRITE(10,*)
        WRITE(10,'(''      TRANSFER(IS1,IT)'')')
ccc     WRITE(10,'(''  IT'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)
        DO IT=1,NTWSYS(IPHEL)
          I=0
          DO IM1=1,NMODES(IPHEL)
          DO IS1=1,NSM(IM1,IPHEL)
            I=I+1
            J=0
            BMAX=0
            DO IM2=1,NMODES(IPHEL)
            DO IS2=1,NSM(IM2,IPHEL)
              J=J+1
C --> TRANSFER ALLOWED ONLY AMONG SYSTEMS OF THE SAME MODE
              IF(IM2.EQ.IM1) THEN
                BAUX=BARRIER(I,J,IT)
                IF (BAUX.GT.BMAX) BMAX=BAUX
              ENDIF
            ENDDO
            ENDDO
            TRANSFER(I,IT)=BMAX
          ENDDO
          ENDDO
C *** ENFORCES NO TRANSFER OF TWIN THROUGH TWIN, EXCEPT 'SELF'
          DO IT1=NSLSYP+1,NSYST(IPHEL)
            TRANSFER(IT1,IT)=0.
            IF(IT+NSLSYP.EQ.IT1) TRANSFER(IT1,IT)=1.
          ENDDO
ccc       WRITE(10,'(I4,2X,30F6.3)')
ccc  #             IT,(TRANSFER(IS1,IT),IS1=1,NSYST(IPHEL))
        ENDDO
        TRANSFER(1,1)=TRANSFER(1,1)      ! to fool the compiler

      ENDIF      ! END OF IOPTION=1

C **************************************************************************
C *** UPDATES CRSS's IN PARENT AND CHILD
C **************************************************************************

      IF (IOPTION.EQ.2) THEN
      
        temperat=tempi
       
        write(10,'(/,'' ENTERING UPDATE_CRSS_CG IOPT=2'')')
        EDOT=SQRT(2./3.)*TNORM(DBAR,5,1)

        KGX=1
        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1


          NSLSYP=NSYST(IPHEL)-NTWSYS(IPHEL)

          NSLIPMODES=NMODES(IPHEL)-NTWMOD(IPHEL)
          KTWINTOT=0
          DO KCNT=1,NSLIPMODES+1
            KTWINTOT=KTWINTOT+NSM(KCNT,IPHEL)
          ENDDO

C       WITH EDOT, CALCULATING K2 AND RHOSAT FOR EACH PHASE     
C       K2 IS THE RATIO OF THE ANNIL TERM TO THE GENERATION TERM
                
                DO 2053 KMO=1,NSLIPMODES
                                    
                K2(KMO)=ALPHA_INACT*BURG(KMO)/ACTENER(KMO)
     #           *(1.-KOVB3(KMO)*TEMPERAT/DRAG(KMO)
     #           *LOG(EDOT/EDOTESO(KMO)))

                RHOSAT(KMO)=K2(KMO)**(-2.0)

2053            ENDDO

        DO KKK=NGR(IPH-1)+1,NGR(IPH)
          ngp=699
          ngc=ngp+ngr(1)
          iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

        IF(WGT(KKK).GT.1.E-6) THEN   ! AVOID 'GHOST' GRAINS IN TWINNED PHASE

C *** DEFINES A MINIMUM MEAN FREE PATH 'DMFPX' IN GRAIN/PARENT/CHILD
C *** DEFINES A BARRIER FACTOR 'BARF' TO DELAY BARRIER EFFECT.
C *** NEW APPROACH WITH INCREASING TWIN THICKNESS:
C     * TWTHRES1: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg] 1ST GENERATION
C     * TWTHRES2: MAXIMUM TWIN FRACTION IN THE GRAIN 1ST GENERATION
C     * TWTHRES3: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg] 2ND GENERATION
C     * TWTHRES4: MAXIMUM TWIN FRACTION IN THE GRAIN 2ND GENERATION

          IPTS=ABS(KTWSMX(KKK))
          IPTM=KTWMMX(KKK)
          BARF=0.
          TWFRPTS=0.
          DMFPX=GRSZE
          CONDITION=0
          TH0X=0.05

C ** DIFFERENT CONDITIONS TO CALCULATE DMFPX DEPENDING ON PHASE (GP)
C ** CODE WRITTEN AS IF THERE WERE 2 OR 3 PHASES
          IF(IPHEL.EQ.1) THEN
            IF(IPTS.NE.0) THEN
               TH1X=TWTHRES(1,IPTM,1)
               TH2X=TWTHRES(2,IPTM,1)
               CONDITION=1
             ENDIF
          ELSE IF(IPHEL.EQ.2) THEN
            IF(IPHTOP.LE.2) THEN
               IPTM_PARENT=KTWMMX(KKK-NGR(1))
               TH1X=TWTHRES(1,IPTM_PARENT,1)
               TH2X=TWTHRES(2,IPTM_PARENT,1)
               CONDITION=2
            ELSE
               IF(IPTS.EQ.0) THEN    !VERIFY THAT grand CHILD HAS NOT BEEN CREATED YET
                 IPTM_PARENT=KTWMMX(KKK-NGR(1))
                 TH1X=TWTHRES(1,IPTM_PARENT,1)
                 TH2X=TWTHRES(2,IPTM_PARENT,1)
                 CONDITION=2
               ELSE
                 IPTM_PARENT=KTWMMX(KKK-NGR(1))
                 TH1X=TWTHRES(3,IPTM,1)*TWTHRES(1,IPTM_PARENT,1) !DISTRIBUTES THE 2NDARY TWINS OVER ALL THE FIRST TWIN LAMELLAE
                 TH2X=TWTHRES(4,IPTM,1)
                 CONDITION=3
                ENDIF
             ENDIF
          ELSE IF(IPHEL.EQ.3) THEN
              IPTM_PARENT=KTWMMX(KKK-2*NGR(1))
              TH1X=TWTHRES(3,IPTM,1)*TWTHRES(1,IPTM_PARENT,1) !DISTRIBUTES THE 2NDARY TWINS OVER ALL THE FIRST TWIN LAMELLAE
              TH2X=TWTHRES(4,IPTM,1) !THIS REQUIRES THAT THE GRAND-CHILD IS TAGGED AS THE CHILD
              CONDITION=4
          ENDIF

          IF(CONDITION.NE.0) THEN !THIS DOESN'T RUN IF GRAIN WITHOUT TWIN

C *** VERSION JULY 5 2006 (GP)
C *** TWFRPTS WILL BE USE IN PARENT PHASE TO PREVENT MORE PTS DEFORMATION
C *** TWFRPTS IN PARENT WILL BE USED IN CHILD TO CALCULATED DMFPX FOR HP EFFECT
C *** SAME REASONING BETWEEN CHILD AND GRAND-CHILD

C ** NEW CONDITIONS TO CALCULATE TWIN FRACTION (GP)
            IF(IPHEL.EQ.1) TWFRPTS=TWFRSY(IPTS-NSLSYP,KKK)
            IF(IPHEL.EQ.2) THEN
               IF(IPHTOP.LE.2) THEN
                  IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                  TWFRPTS=TWFRSY(IPTS_PARENT-NSLSYP,KKK-NGR(1))
               ELSE
                  IF(IPTS.EQ.0) THEN
                    IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                    TWFRPTS=TWFRSY(IPTS_PARENT-NSLSYP,KKK-NGR(1))
                  ELSE
                    TWFRPTS=TWFRSY(IPTS-NSLSYP,KKK)
                  ENDIF
               ENDIF
            ENDIF
            IF(IPHEL.EQ.3) TWFRPTS=TWFRSY(IPTS-NSLSYP,KKK-NGR(1))

            IF(CONDITION.EQ.1) THEN
              PREDOM=IPTS
C *** POWER LAW INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
              POWER=1.0
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X)
     #                       BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER
              IF(TWFRPTS.GE.TH2X) BARF=1.

C *** SIGMOIDAL INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
C             SLOPE=2.
C             TFX=(2*TWFRPTS-TH0X-TH2X)/(TH2X-TH0X)
C             BARF=1./(1.+EXP(-SLOPE*TFX))

              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE
              DMFPX_HP=(1.-TWFRPTS)*TH1X*GRSZE

            ELSE IF(CONDITION.EQ.2) THEN
              IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
              PREDOM=IPTS_PARENT
              BARF=1.                      ! assumes full barrier in twin
              DMFPX_HP=TWFRPTS*TH1X*GRSZE     ! for increasing twin thickness FOR HP
              DMFPX=TH2X*TH1X*GRSZE        ! assumes always maximum thickness FOR GNDs

            ELSE IF(CONDITION.EQ.3) THEN   ! for conditions 3 and 4, we need to change the value of grsze
              PREDOM=IPTS
              IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
              IPTM_PARENT=KTWMMX(KKK-NGR(1))
              TWFRPTS_PARENT=TWFRSY(IPTS_PARENT-NSLSYP,KKK-NGR(1))
              GRSZE_NEW=2*TWFRPTS_PARENT*TWTHRES(1,IPTM_PARENT,1)*GRSZE
              POWER=1.0
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X)
     #                   BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER
              IF(TWFRPTS.GE.TH2X) BARF=1.
              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE_NEW
              DMFPX_HP=(1.-TWFRPTS)*TH1X*GRSZE_NEW

            ELSE IF(CONDITION.EQ.4) THEN
              PREDOM=IPTS
              IPTS_PARENT=ABS(KTWSMX(KKK-2*NGR(1)))
              IPTM_PARENT=KTWMMX(KKK-2*NGR(1))
              TWFRPTS_PARENT=TWFRSY(IPTS_PARENT-NSLSYP,KKK-2*NGR(1))
              GRSZE_NEW=2*TWFRPTS_PARENT*TWTHRES(1,IPTM_PARENT,1)*GRSZE
              BARF=1.                      ! assumes full barrier in twin
              DMFPX_HP=TWFRPTS*TH1X*GRSZE_NEW
              DMFPX=TH2X*TH1X*GRSZE_NEW
            ENDIF
          ENDIF

          IF(DMFPX.LT.0.) THEN
            WRITE(*,'('' DMFPX<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx
            STOP
          ENDIF
          IF(DMFPX_HP.LT.0.) THEN
            WRITE(*,'('' DMFPX_HP<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx_hp'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx_HP
            STOP
          ENDIF


C *** 'PTS' HARDENS ALL SYSTEMS VIA DIRECTIONAL HALL-PETCH & GND EFFECTS.
          GAMTOTX=GTOTGR(KKK)
          DELTGAM=0.0
          DO IS=1,NSYST(IPHEL)
            IF(IS.NE.IPTS) DELTGAM=DELTGAM+ABS(GAMDOT(IS,KKK))*TINCR
          ENDDO

            ISM=0
            DO 1728 KMO=1,NSLIPMODES
                DELGAM(KMO)=0.0
                DELRHOS(KMO)=0.0
                DELRHOW(KMO)=0.0
                
                 DO ISY=1,NSM(KMO,IPHEL)
                    ISM=ISM+1
                     DELGAM(KMO)=DELGAM(KMO)+
     #               ABS(GAMDOT(ISM,KKK))*TINCR
                ENDDO            
              
                TWFRGRAIN=0
                
                DO IT=1,NTWSYS(IPHEL)
                    TWFRGRAIN=TWFRGRAIN+TWFRSY(IT,KKK)
                ENDDO
                
                DELRHOS(KMO)=KGENER(KMO)*SQRT(RHO_S(KMO,KKK))
     #           *(1.0-K2(KMO)*SQRT(RHO_S(KMO,KKK)))
     
C               FOLLOWING IS ADDED 06/25/08 FOR ENHANCED TWIN STORAGE
                IF(IPHEL.EQ.2)THEN
                STOR=1.0
                DELRHOS(KMO)=KGENER(KMO)*SQRT(RHO_S(KMO,KKK))
     #           *(STOR*1.0-K2(KMO)*SQRT(RHO_S(KMO,KKK)))
                ENDIF
C               BLOCK FOR TWINS ENDS HERE
        
                if (DELRHOS(KMO).LT.0.0) DELRHOS(kmo)=0.0
         
C               THE 1E+06 IN THE ABOVE IS TO CONVERT MPA TO PA
     
                 DELRHOW(KMO)=KGENER(KMO)*RHO_S(KMO,KKK)*K2(KMO)*
     #           (BURG(KMO)*SQRT(RHO_DEB(KKK)))*Q5(KMO)
     
c            if(kkk.eq.500)write(10,*) 'pyr',kmo,q5(kmo),delrhow(kmo)
     
1728        ENDDO


                ISM=0
                DO 1720 KMO=1,NSLIPMODES
                    DRHOS=0.0
                    DRHOW=0.0
                    DTAU(KMO)=0.0
                    
                ATAU(KMO)=YONE(KMO)+BONE(KMO)*EXP(-TEMPERAT/SONE(KMO))

                RHO_S(KMO,KKK)=RHO_S(KMO,KKK)+DELRHOS(KMO)*
     #           DELGAM(KMO)
     
                RHO_DEB(KKK)=RHO_DEB(KKK)+Q6*DELRHOW(KMO)*
     #           DELGAM(KMO)
     
1720            ENDDO

                ISM=0
                DO 1770 KMO=1,NSLIPMODES
    
                DO ISY=1,NSM(KMO,IPHEL)
                    ISM=ISM+1
     
                    TAUE(ISM,KKK)=ATAU(KMO)+ALPHA_INACT*
     #                 BURG(KMO)*XGMOD*(SQRT(RHO_S(KMO,KKK))
     #                 -0.086/ALPHA_INACT*SQRT(RHO_DEB(KKK))
     #                 *LOG(BURG(KMO)*SQRT(RHO_DEB(KKK))))
     
CCCCCCCCCCCCBLOCK FOR HALL PETCHCCCCCCCC         
               IF(ipts.gt.nslsyp.and.ipts.le.nslsyp+12)THEN
C               THE ABOVE IS TO CONSIDER ONLY IF THE PTS IS A TTW

                HPFAC(ISM,IPHEL)=HPK1(KMO)*
     #          SQRT(BURG(KMO))*
     #          XGMOD
C               THE 1E+06 IN THE ABOVE IS TO CONVERT MPA TO PA
     
C           THE FOLLOWING APPLIES FOR COMPRESSION TWINNING (1122)
                ELSEIF(IPTS.GT.NSLSYP+12)THEN
                HPFAC(ISM,IPHEL)=HPK2(KMO)*XGMOD*SQRT(BURG(KMO))
                ENDIF
CCCCCCCCCCCCBLOCK FOR HALL PETCHCCCCCCCC  

                ENDDO  !ISY
    
1770            ENDDO

C *** UPDATE TAUE FOR TWINNING
          DO 1838 MNN=1,NTWMOD(IPHEL)
            TAU_PROP(MNN)=YPROP(MNN)+
     #         APROP(MNN)*EXP(-TEMPERAT/TPROP(MNN))
            TAU_CRIT(MNN)=YCRIT(MNN)+
     #         ACRIT(MNN)*EXP(-TEMPERAT/TCRIT(MNN))
1838      ENDDO

          DO 1738 IS=NSLSYP+1,NSYST(IPHEL)       
          TAUE(IS,KKK) =0.0
          IF(IS.LE.KTWINTOT)THEN
            JTWIN=1
          ELSE
            JTWIN=2
          ENDIF
            RHOTEMP=0.0
            DO KJO=1,NSLIPMODES
         
c            TAUE(IS,KKK) =TAUE(IS,KKK)+XGMOD*BURG_TW(JTWIN)*
c     #      BURG(KJO)*TLATENT(KJO,JTWIN)*RHO_S(KJO,KKK)

            TAUE(IS,KKK) =TAUE(IS,KKK)+XGMOD*BURG_TW(JTWIN)*
     #      BURG(KJO)*(TLATENT(KJO,JTWIN)+TLATENT1(KJO,JTWIN)
     #      *LOG(EDOT))*RHO_S(KJO,KKK)
   
c            if(kkk.eq.500)write(10,*) 'btwincrss',taue(is,kkk),
c     #       rho_s(kjo,kkk)
           
         if(kkk.eq.500) write (10,*) 'tlatent',tlatent(kjo,jtwin)+
     #     tlatent1(kjo,jtwin)*log(edot),kjo,jtwin
     
            RHOTEMP=RHOTEMP+RHO_S(KJO,KKK)/RHOSAT(KJO)
            ENDDO
     
            TAUE(IS,KKK)=TAUE(IS,KKK)+TAU_PROP(JTWIN)+
     #        (TAU_CRIT(JTWIN)-TAU_PROP(JTWIN))
     #        *exp(-RHOTEMP)
     
            if (TAUE(IS,KKK).le.0.0)TAUE(IS,KKK)=1.0
c            if(kkk.eq.500)write(10,*) 'twincrss',taue(is,kkk),
c     #       tau_prop(jtwin)

1738      ENDDO

C *** ADD HALL-PETCH EFFECT TO TAUE FOR SLIP AND TWINNING
          DO IS=1,NSYST(IPHEL)

C *** HALL-PETCH EFFECT

            IF(IPTS.EQ.0) DMFP(IS)=GRSZE
            IF(IPTS.GT.0) DMFP(IS)=DMFPX*SYSMFP(IS,IPTS-NSLSYP)
            IF(DMFP(IS).GT.GRSZE) DMFP(IS)=GRSZE
         
            TAUHP  =BARF*HPFAC(IS,IPHEL)/SQRT(DMFP(IS))
C            TAUHP=HPFAC(IS,IPHEL)/SQRT(GRSZE)
            TAUHP0 =HPFAC(IS,IPHEL)/SQRT(GRSZE)
            TAUHP=MAX(TAUHP,TAUHP0)

C *** NEW VERSION JULY 5 2006
C *** PTS IS BLOCKED IN PARENT WHEN THRES2 IS REACHED
C *** ALL THE TWIN SYSTEMS ARE BLOCKED WHEN THE SUM OF THEIR VOL FRACTION REACHES 1
C *** IN CHILD EVERY TWIN SYSTEMS KEEPS ITS CRSS (NO PTSDAMP FACTOR) IF NO GRAND-CHILD CONSIDERED
C *** -->NO SYSTEM BLOCKED IN CHILD IF NO GRAND-CHILD CONSIDERED
C *** SAME THING BETWEEN CHILD AND GRAND-CHILD
C *** IN THE CHILD WE BLOCK PTS PRODUCING GRAND-CHILD WHEN ITS VOLUME FRACTION REACHED THRES4
c *** AND ALL THE OTHER TWIN SYSTEMS ARE STOPPED WHEN THE SUM OF THEIR TWIN VOLUME FRACTIONS REACHED 1
            PTSDAMP=1.
            IF(IPHEL.EQ.1) THEN
             if(ktwsmx(kkk).gt.0) then
              IF(IS.GT.NSLSYP) THEN
                 TAUGND=0.
                 if(is.eq.ipts) then
                   PTSDAMP=1+(TWFRPTS/TWTHRES(2,IPTM,1))**20
                 else
                   PTSDAMP=1.+ twfrgrain**20
                 ENDIF
              ENDIF
             endif
            ENDIF
            
c            IF(IPHTOP.GT.2) THEN
c              IF(IPHEL.EQ.2) THEN
c               if(iptsgr(kkk).gt.0) then
c                if(ktwsmx(kkk).eq.0) then
c                   ipts=iptsgr(kkk)
c                   iptm=iptmgr(kkk)
c                endif
                IF(IS.GT.NSLSYP) THEN
c                  TAUGND=0.
c                  IF(IS.EQ.IPTS) THEN
c                    PTSDAMP=1+
c     #               (twfrsy(ipts-nslsyP,kkk)/TWTHRES(4,IPTM,1))**20
c                  ELSE
c                    PTSDAMP=1.+ twfrgrain**20
c                  ENDIF
c                ENDIF
c               endif
c              ENDIF
            ENDIF

            CRSS(IS,KKK)=PTSDAMP*TAUE(IS,KKK)+TAUHP
           
           
           
 
c           if (is.eq.20.and.kkk.eq.30) then 
c            write(10,*) is, 'grain 1329'
c            write(10,*) crss(is,1329),',',taue(is,1329),',',tauhp
c            write(10,*) is, 'grain 1972'
c            write(10,*) crss(is,1972),',',taue(is,1972),',',tauhp            
c           end if 

c           if (is.eq.1.and.kgx.eq.3000) then 
c           write(10,*) 'kgx'
c           write(10,*) crss(is,kgx),',',taue(is,kgx),',',tauhp
c           end if 


c            if(iprx.eq.1) then
c              write(*,'('' kkk is iptm ipts rspce'',10x,4i5,2f9.1)')
c     #                   kkk,is,iptm,ipts,SYSMFP(is,ipts-nslsys)
c              write(*,'(''    twthr1 twthr2 twfrpts barf'',5x,4f9.1)')
c     #                        th1x,th2x,twfrpts,barf
c              write(*,'(''    crss  hpfac dmfp tauhp'',5x,4f9.1)')
c     #              crss(is,kkk),hpfac(is,iphel),dmfp(is),tauhp
c              pause
c            endif
C *********************************************************************

          ENDDO      ! END OF DO LOOP OVER IS=1,NSYST
          GTOTGR(KKK)=GAMTOTX+DELTGAM
        ENDIF      ! END OF IF WGT(KKK)>0

      if(iprx.eq.1) then
c         write(10,'('' kkk dmfp(is)'',i5,6x,3f6.1,/,(12f6.1))')
c    #                  kkk,(dmfp(is),is=1,nsyst(iphel))
c         write(10,'('' TAUE(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
c    #          kkk,(taue(is,kkk),is=1,nsyst(iphel))
          write(10,'('' CRSS(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
     #          kkk,(crss(is,kkk),is=1,nsyst(iphel))
          gamdnorm=tnorm5(dbar,5,1)
          write(10,'('' GAMD(is,kkk)'',i5,6x,3f7.2,/,(12f7.2))')
     #          kkk,(gamdot(is,kkk)/gamdnorm,is=1,nsyst(iphel))
      endif

        KGX=KGX+1
        ENDDO      ! END OF LOOP OVER GRAINS IN PHASE 'IPH'
        ENDDO      ! END OF LOOP OVER PHASES
      ENDIF      ! END OF IF(IOPTION.EQ.2)

      RETURN
      END

C **************************************************************************
C     SUBROUTINE UPDATE_SHAPE      --->      VERSION 12/NOV/2007
C
C     CALCULATES THE DIRECTION AND LENGTH (EIGENVECTORS AND EIGENVALUES OF
C     DEFORMATION TENSOR FIJ) OF THE AXES OF THE ELLIPSOID ASSOCIATED WITH
C     AVERAGE, PHASE AND/OR GRAIN ACCUMULATED DEFORMATION.
C     CALCULATES THE EULER ANGLES OF ELLIPSOID AXES WRT SAMPLE AXES.
C
C     REPLACES PREVIOUS SUBR. GRAXES_AVERAGE & SUBR. GRAXES_LOCAL  (SEP/04)
C
C     GRAIN-SPLIT SCHEME ADDED ON 12/NOV/07 (CNT)
C **************************************************************************

      SUBROUTINE UPDATE_SHAPE_cg (IPH)

      INCLUDE 'VPSC7.DIM'

      DIMENSION W(3),BX(3,3),B(3,3),BT(3,3),FIJX(3,3)


C *** IF IPH=0 ELLIPSOID REPRESENTS AVERAGE DEFORMATION IN ELEMENT
C *** IF IPH>0 ELLIPSOID REPRESENTS AVERAGE DEFORMATION IN PHASE 'IPH'
C *** CALCULATES EIGENVALUES, EIGENVECTORS & EULER ANGLES OF ELEMENT GRAIN,
C     PHASE GRAIN, OR INDIVIDUAL GRAIN
C *** 'AXISPH' TRANSFORMS FROM ELLIPSOID TO SAMPLE AXES.

      DO I=1,3
      DO J=1,3
        BX(I,J)=0.
        DO K=1,3

C ***  uses UDOT forall phases to prevent numerical instability in SO procedure
        if(interaction.eq.5) then
          BX(I,J)=BX(I,J)+FIJPH(I,K,0)*FIJPH(J,K,0)
        else
          BX(I,J)=BX(I,J)+FIJPH(I,K,IPH)*FIJPH(J,K,IPH)
        endif

        ENDDO
      ENDDO
      ENDDO

      CALL JACOBI(BX,3,3,W,B,NROT,IER)
      CALL EIGSRT(W,B,3,3)
      IF (IER.EQ.1) THEN
        WRITE(*,*) 'ERROR IN UPDATE_SHAPE FOR PHASE ELLIPSOID',IPH
        STOP
      ENDIF

C *** EIGENVALUES (AND ASSOC EIGENVECTORS) ARE ORDERED FROM LARGER TO SMALLER.
C *** REDEFINE AXIS(2) TO BE THE LARGEST IN ORDER TO IMPROVE ACCURACY IN THE
C     CALCULATION OF THE ESHELBY TENSOR.
C *** IF DET(B)<0 MEANS THAT THE SYSTEM IS LEFT HANDED. IT IS MADE RIGHT
C     HANDED BY EXCHANGING 1 AND 2.

      SIGN=-1.
      IF(DET(B).LE.0.) SIGN=1.
      DO I=1,3
        EXCHANGE=B(I,1)
        B(I,1)=B(I,2)
        B(I,2)=EXCHANGE*SIGN
      ENDDO
      EXCHANGE=W(1)
      W(1)=W(2)
      W(2)=EXCHANGE

      RATMAX=SQRT(W(2)/W(3))
      RATMIN=SQRT(W(1)/W(3))

      DO I=1,3
        AXISPH(0,I,IPH)=SQRT(W(I))
        DO J=1,3
          AXISPH(I,J,IPH)=B(I,J)
          BT(I,J)        =B(J,I)
        ENDDO
      ENDDO

      CALL EULER(1,ANG1,ANG2,ANG3,BT)
      EULERPH(1,IPH)=ANG1
      EULERPH(2,IPH)=ANG2
      EULERPH(3,IPH)=ANG3

      IF (IPH.NE.0) THEN

      IF (IFRAG(IPH).EQ.0) THEN
C *************************************************************************
C *** STOPS UPDATING THE ELLIPSOID SHAPE FOR A LIMIT ASPECT RATIO

        IF(RATMAX.GT.CRIT_SHP(IPH) .AND. IFLAT(IPH).EQ.0) THEN
CFEB
          write(10,*) '*********************************************'
          write(10,*) '  STOPS UPDATING GRAIN SHAPE FOR PHASE ',iph
          write(10,*) '*********************************************'
          write(*, *) '*********************************************'
          write(*, *) '  STOPS UPDATING GRAIN SHAPE FOR PHASE ',iph
          write(*, *) '*********************************************'
CFEE
          IFLAT(IPH)=1
        ENDIF
C *************************************************************************

      ELSE IF (IFRAG(IPH).EQ.1) THEN

C *************************************************************************
C *** SPLITS AVERAGE GRAIN WHEN IT EXCEEDS A CRITICAL ASPECT RATIO.
C *** REDEFINES LENGTH OF ELLIPSOID AXES 'AXISPH(0,I,IPH)' BUT NOT THE
C     ELLIPSOID ORIENTATION 'AXIS(J,I,IPH)'
C *** RECALCULATES DEFORMATION GRADIENT OF THE PHASE 'FIJPH' EXPRESSED IN
C     SAMPLE AXES AND NEEDED LATER.

        IF(RATMAX.GE.CRIT_SHP(IPH)) THEN
          W(2)=W(2)/4.
          IF(RATMIN.GE.CRIT_SHP(IPH)/2.) W(1)=W(1)/4.

ccc       w(1)=1.    ! resetting to equiaxed
ccc       w(2)=1.
ccc       w(3)=1.

          DO I=1,3
            AXISPH(0,I,IPH)=SQRT(W(I))
            DO J=1,3
              FIJX(I,J)=XID3(I,J)*SQRT(W(I))
            ENDDO
          ENDDO
          DO I=1,3
          DO J=1,3
            FIJPH(I,J,IPH)=0.
            DO K=1,3
            DO L=1,3
              FIJPH(I,J,IPH)=FIJPH(I,J,IPH)+B(I,K)*B(J,L)*FIJX(K,L)
            ENDDO
            ENDDO
          ENDDO
          ENDDO
        ENDIF      ! END OF IF RATMAX>CRIT_RAT
C ********************************************************************
      ENDIF      ! END OF IF IFRAG

      ENDIF      ! END OF IF IPH.NE.0
CFEB
      write(10,'(''IPH='',I3,''  FIJPH  '',9F9.4)')
     #           IPH,((FIJPH(I,J,IPH),J=1,3),I=1,3)
      write(10,'(9X,''EIGNVAL'',3F9.4)')  (AXISPH(0,J,IPH),J=1,3)
      write(10,'(9X,''EIGNVEC'',9F9.4)') ((AXISPH(I,J,IPH),J=1,3),I=1,3)

C ********************************************************************
C *** UPDATES ELLIPSOIDS (EIGENVALUES & EIGENVECTORS) OF INDIVIDUAL GRAINS
C     'AXISGR(I,J)' TRANSFORMS FROM ELLIPSOID TO SAMPLE AXES.

      IF(ISHAPE(IPH).GT.0) THEN
C ********************************************************************

      DO KKK=NGR(IPH-1)+1,NGR(IPH)


C *** GP JULY 2006
C *** ADD A CONDITION TO UPDATE ONLY THE PARENTS GRAIN FOR WHICH
C *** THE PTS VOLUME FRACTION IS LOWER THAN THRES2
C *** ONLY VALID IF ISHAPE> OR = 2
      IPTM=KTWMMX(KKK)
      IPTS=KTWSMX(KKK)
      TAGG=0
      IF(ISHAPE(IPH).GE.2) THEN
        IF(KTWSMX(KKK).NE.0) TAGG=1
      ENDIF
      IF((TAGG.EQ.0).OR.
     # (TWFRSY(IPTS,KKK).LT.TWTHRES(2,IPTM,IPHBOT))) THEN

c**********************

        DO I=1,3
        DO J=1,3
          BX(I,J)=0.
          DO K=1,3
            BX(I,J)=BX(I,J)+FIJGR(I,K,KKK)*FIJGR(J,K,KKK)
          ENDDO
        ENDDO
        ENDDO

        CALL JACOBI(BX,3,3,W,B,NROT,IER)
        CALL EIGSRT(W,B,3,3)
        IF (IER.EQ.1) THEN
          WRITE(*,'(''ERROR IN UPDATE_SHAPE FOR GRAIN'',I5,
     #            ''  IN PHASE'',I3)') KKK,IPH
          STOP
        ENDIF

C *** EIGENVALUES (AND ASSOC EIGENVECTORS) ARE ORDERED FROM LARGER TO SMALLER.
C *** REDEFINE AXIS(2) TO BE THE LARGEST IN ORDER TO IMPROVE ACCURACY IN THE
C     CALCULATION OF THE ESHELBY TENSOR.
C *** IF DET(B)<0 MEANS THAT THE SYSTEM IS LEFT HANDED. IT IS MADE RIGHT
C     HANDED BY EXCHANGING 1 AND 2.

        SIGN=-1.
        IF(DET(B).LE.0.) SIGN=1.
        DO I=1,3
          EXCHANGE=B(I,1)
          B(I,1)=B(I,2)
          B(I,2)=EXCHANGE*SIGN
        ENDDO
        EXCHANGE=W(1)
        W(1)=W(2)
        W(2)=EXCHANGE

        RATMAX=SQRT(W(2)/W(3))
        RATMIN=SQRT(W(1)/W(3))

        DO I=1,3
          AXISGR(0,I,KKK)=SQRT(W(I))
          DO J=1,3
            AXISGR(I,J,KKK)=B(I,J)
          ENDDO
        ENDDO
      ENDIF
        IF(IFRAG(IPH).EQ.0) THEN
C ********************************************************************
C *** STOPS UPDATING THE ELLIPSOID SHAPE OF EVERY GRAIN IN PHASE 'iph'
C     WHEN THE 'WORST OFFENDER' GRAIN REACHES A LIMIT ASPECT RATIO

          IF(RATMAX.GT.CRIT_SHP(IPH) .AND. IFLAT(IPH).EQ.0) THEN
            write(10,*) '*********************************************'
            write(10,*) '  STOPS UPDATING GRAIN SHAPE FOR PHASE ',iph
            write(10,*) '*********************************************'
            write(*, *) '*********************************************'
            write(*, *) '  STOPS UPDATING GRAIN SHAPE FOR PHASE ',iph
            write(*, *) '*********************************************'
            IFLAT(IPH)=1
          ENDIF
C ********************************************************************

        ELSE IF (IFRAG(IPH).EQ.1) THEN

C ********************************************************************
C *** SPLITS IN HALF AXES OF THE GRAIN WHEN LIMIT ASPECT RATIOS ARE REACHED

          IF(RATMAX.GE.CRIT_SHP(IPH)) THEN
            W(2)=W(2)/4.
            IF(RATMIN.GE.CRIT_SHP(IPH)/2.) W(1)=W(1)/4.

          DO I=1,3
            AXISGR(0,I,KKK)=SQRT(W(I))
            DO J=1,3
              FIJX(I,J)=XID3(I,J)*SQRT(W(I))
            ENDDO
          ENDDO
          DO I=1,3
          DO J=1,3
            FIJGR(I,J,KKK)=0.
            DO K=1,3
            DO L=1,3
              FIJGR(I,J,KKK)=FIJGR(I,J,KKK)+B(I,K)*B(J,L)*FIJX(K,L)
            ENDDO
            ENDDO
          ENDDO
          ENDDO
        ENDIF     ! END OF IF RATMIN
C ********************************************************************
      ENDIF     ! END OF IF IFRAG

      ENDDO     ! END OF DO KKK

C ********************************************************************
      ENDIF      ! END OF IF ISHAPE.GT.0
C ********************************************************************
CFEE
      RETURN
      END

C **************************************************************************
C     SUBROUTINE WRITE_DISLOCATION_DENSITY
C
C     MAKES STATISTIC ON DEFORMATION MODE ACTIVITY.
C     CALCULATES ARRAYS:
C     - GMAX : maximum dgam for a given mode and a given grain.
C     - GTOT : sum of dgam for a given mode and a given grain.
C     - GAVGR: sum of all dgam for a given grain.
C     - GMAXG: maximum dgam for a given grain.
C     - ACT  : number of active systems in a given grain.
C     - ACTIV: weighted number of active systems in a given phase.
C     - GAVMOD: weighted sum of dgam for a given mode in a given phase.
C     - GAVPH: weighted sum of dgam for a given phase.
C
C     The above arrays are dimensioned to the maximum number of grains and
C     phases in COMMON/STATSH/
C     For FE application this COMMON is commented out and IPH varies between
C     1 and NPH=NELEM
C ***************************************************************************

      SUBROUTINE WRITE_DISLOCATION_DENSITY(ISTEP)

      INCLUDE 'VPSC7.DIM'
      CHARACTER*8 MODE(NMODMX)

C      GAVPC=0.
      DO IPH=IPHBOT,IPHTOP
        IPHEL=IPH-IPHBOT+1
        NSLIPMODES=NMODES(IPHEL)-NTWMOD(IPHEL)

        RHOSUB_AVE(IPH)=0.
C        do kmo=1,nmodes(IPHEL)
        DO KMO=1,NSLIPMODES
          RHOS_AVE(kmo,iph)=0.
        enddo

        IF(WPH(IPH).NE.0.) THEN
          do kgr=ngr(iph-1)+1,ngr(iph)
C            do kmo=1,nmodes(IPHEL)
            DO KMO=1,NSLIPMODES
            RHOS_AVE(kmo,iph)=RHOS_AVE(kmo,iph)+RHO_S(kmo,kgr)
     #       *wgt(kgr)/wph(iph)
            enddo
            RHOSUB_AVE(IPH)=RHOSUB_AVE(IPH)+RHO_DEB(kgr)
     #       *wgt(kgr)/wph(iph)
          enddo
        ENDIF
      ENDDO
c        WRITE(10,*) 'RHOSTATAVERAGE',RHOSUB_AVE(IPH),IPH,
c     #    (RHOS_AVE(KMO,1),KMO=1,NSLIPMODES),
c     #    (RHOS_AVE(KMO,2),KMO=1,NSLIPMODES)

      DO IPH=IPHBOT,IPHTOP
        IPHEL=IPH-IPHBOT+1
        IUNIT=70+IPHEL
c        write(10,*) 'iph=',iph
C        DO I=1,NMODES(IPHEL)
        DO I=1,NSLIPMODES
          MODE(I)='   MODE'//CHAR(48+I)
        ENDDO

            if(istep.eq.1)
     #      write(IUNIT,'(''  STRAIN   SUBSTRUCTURE'',30a10)')
C     #                   (MODE(I),I=1,NMODES(IPHEL))
     #                   (MODE(I),I=1,NSLIPMODES)
c     #                   ' SUBSTRUCTURE/PH   ',
c     #                   (MODE(I),I=1,NSLIPMODES)
            write(IUNIT,'(f8.3,5e14.7)') epsacu,RHOSUB_AVE(iph),
c            write(IUNIT,*) epsacu,RHOSUB_AVE(iph),
C     #            (RHOS_AVE(kmo,iph),kmo=1,nmodes(iphel))
     #             (RHOS_AVE(kmo,iph),kmo=1,NSLIPMODES)



      ENDDO

      RETURN
      END
C
C **************************************************************************





c *** ADDEDMZ - whole subroutine per system
c     Question is how will this transition effect composit grain model and how
c     will it effect rest of the code. Are there any other places where this 
c     change should be forwarded.
c     Another change is dependece of initial CRSS with strain rate according to
c     equation:
c     tau0 = A*(1 + B*ln(edot/C))*exp(-T/D)
c     [Changed to be: tau0 = A + B*ln(edot/C)*exp(-T/D) ]


C *****************************************************************************
C *****************************************************************************
c      this is the subroutine that updates the crss and that takes into account 
c      dislocation activities
*******************************************************************************


      SUBROUTINE UPDATE_CRSS_CG_disl_sys (IOPTION)

      INCLUDE 'VPSC7.DIM'


      DIMENSION DN1(3),DB1(3),DN2(3),DB2(3),DNT(3),DBT(3),DL1(3),DL2(3)
      DIMENSION AA(3,3),ATWIN(3,3),SCH1(3,3),SCH2(3,3)
      DIMENSION BARRIER(NSYSMX,NSYSMX,NTWSMX),TRANSFER(NSYSMX,NTWSMX),
     #          DMFP(NSYSMX),DMFP_HP(NSYSMX)
     
c      DIMENSION IPTSGR(NGRPEL),IPTMGR(NGRPEL)
      DIMENSION IPTMGR(NGRPEL)
      INTEGER CONDITION,PREDOM
      REAL*8 K2_MZ(50) !ADDEDMZ      
      REAL*8 RHO_W(NMODMX,NGRMX),XGMOD
      REAL*8 EDOT,BURG(4)
      REAL*8 ACTENER(4)
      REAL*8 BOLTZ,EREF,X
      REAL*8 KOVB3(4),C_T_S(3)
      REAL*8 ACRIT(4),TCRIT(4),YCRIT(4)
      REAL*8 APROP(4),TPROP(4),YPROP(4)
      REAL*8 TAU_PROP(4),TAU_CRIT(4)
      real*8 Q1(4),Q2(4),Q0(4),Q3(4)
      INTEGER NFLAG,NXMODE(50),MNM
      REAL*8 KGENER(4),KGENER_1,KGENER_2,KGENER_3 !KGENER_O IS USED TO STORE VALUE OF KGENER(1)
      REAL*8 EDOTESO(4),KR !KR is used for test the k2 in the paper
      REAL*8 RHO_INITS(4),RHO_INITW(4),ATAU(4)
      REAL*8 ALPHA_INACT,DRHOS,DRHOW
      REAL*8 DELRHOS(NSYSMX),DELRHOW(NSYSMX)
      REAL*8 DELGAM(4),DTAU(4),TLATENT1(4,4)
      real*8 RHOSAT(50),K2(4),TLATENT(4,4),DRAG(4) !***ADDEDMZ: RHOSAT(15)
      real*8 HPK0(4),HPK1(4),HPSFE(4),SUM_OF_T_SHEAR_RATE
      real*8 HPK2(4),Q5(4),Q6,BURG_TW(4)
      REAL*8 AONE(4),BONE(4),CONE(4),DONE(4) !ADDEDMZ - strain rate dependance
      real*8 C_tw_1(4),C_tw_2(4),P_tw_1(4),P_tw_2(4),T_tw(4),i_tw(4) !MZ new twin dependence
      real*8 B_tw(4),V_tw(4),A_tw(4),ami_tw(4),sigma_tw(4) !MZ new twin dependence 2
      real*8 B_tw_p(4),V_tw_p(4),A_tw_p(4),ami_tw_p(4) !MZ new twin dependence 2
     #        ,sigma_tw_p(4)
      character*17 identifiant
 
      iprint=1   ! controls diagnostic print-out
      

      
      IF (IOPTION.EQ.1) THEN

          EDOT=SQRT(2./3.)*TNORM(DBAR,5,1)
          
C *** INITIALIZE THE CRSS IN ALL SYSTEMS OF EACH PHASE IN ALL GRAINS
C *** THE 'CHILD' PHASE HAS INITIAL ZERO WEIGHT, AND CRSS's WILL BE
C *** REDEFINED WHEN THE LAMELLAE ARE CREATED.

        DO IPH=IPHBOT,IPHTOP
          IPHEL=IPH-IPHBOT+1
          NSLSYP=NSYST(IPHEL)-NTWSYS(IPHEL)      
          NSLIPMODES=NMODES(IPHEL)-NTWMOD(IPHEL)
                    
C       START READING IN THE MATERIAL INPUT PARAMETERS
          KTWINTOT=0
          DO KCNT=1,NSLIPMODES+1
            KTWINTOT=KTWINTOT+NSM(KCNT,IPHEL)
          ENDDO

        write(10,*) 'ktwintot=',ktwintot

          OPEN (UNIT=UR1,FILE=FILECRYS,STATUS='OLD')

          identifiant='blah'
         
28        continue          
          READ(UR1,'(A)') identifiant
          if (identifiant.EQ.'DISLOCATION MODEL') then
          goto 29
          else
          goto 28
          end if
          
29        continue  
     
          READ(UR1,*) XGMOD,BOLTZ,EREF
          READ(UR1,*) ALPHA_INACT, Q6 !ADDEDMZ ALPHA_INACT_SR
          DO 1054 KJO=1,NSLIPMODES
          READ(UR1,'(A)') PROSA
          READ(UR1,*) BURG(KJO),KOVB3(KJO),ACTENER(KJO)
          READ(UR1,*) KGENER(KJO),DRAG(KJO)
          READ(UR1,*) EDOTESO(KJO),C_T_S(KJO) !THE CONSTANT FOR REDUCE DISLOCATION DENSITY
          READ(UR1,*) RHO_INITS(KJO),RHO_INITW(KJO)
          READ(UR1,*) ipyr(KJO),AONE(KJO),BONE(KJO),CONE(KJO)
     #                ,DONE(KJO)              !ADDEDMZ - added two parameters for dependance on strain rate
          if (ntwmod(iphel).ne.0) then !ADDEDMZ          
          read(UR1,*) (TLATENT(KJO,mo),mo=1,ntwmod(iphel))
          read(UR1,*) (TLATENT1(KJO,mo),mo=1,ntwmod(iphel))
          endif !ADDEDMZ          
          READ(UR1,*) HPK0(KJO),HPK1(KJO),HPK2(KJO)
          READ(UR1,*) Q0(KJO),Q1(KJO),Q2(KJO)
1054      ENDDO

          DO 1056 KJO=1,NTWMOD(IPHEL)
          READ(UR1,'(A)') PROSA
          READ(UR1,*) i_tw(KJO)
          if (i_tw(KJO).eq.0) then
            READ(UR1,*) YCRIT(KJO),ACRIT(KJO),TCRIT(KJO)
            READ(UR1,*) YPROP(KJO),APROP(KJO),TPROP(KJO)
          elseif (i_tw(KJO).eq.1) then
            READ(UR1,*) C_tw_1(KJO),C_tw_2(KJO),T_tw(KJO)
            READ(UR1,*) P_tw_1(KJO),P_tw_2(KJO)
          elseif (i_tw(KJO).eq.2) then
            READ(UR1,*) B_tw(KJO),V_tw(KJO),A_tw(KJO),ami_tw(KJO)
     #                 ,sigma_tw(KJO)
            READ(UR1,*) B_tw_p(KJO),V_tw_p(KJO),A_tw_p(KJO),
     #                 ami_tw_p(KJO),sigma_tw_p(KJO)
          endif
          READ(UR1,*) BURG_TW(KJO)
1056      ENDDO
          KGENER_1=KGENER(1)
		  KGENER_2=KGENER(2)
		  KGENER_3=KGENER(3)
          !ADDEDMZ - read interaction matrix parameters
          READ(UR1,'(A)') PROSA
          i=1
          do kjo=1,NSLIPMODES
              read(UR1,*) (XINTER(kjo,kjo1),kjo1=1,NSLIPMODES+i)
              i=i-1
          enddo
          READ(UR1,'(A)') PROSA
          if (SUM(ipyr).gt.0) then !MZ_U
              READ(UR1,*) A_pyr,ami_pyr,sigma_pyr
              READ(UR1,*) A_prism,ami_prism,sigma_prism    
          endif
          ! read barf power
          READ(UR1,'(A)') PROSA
          READ(UR1,*) POWER_MZ
          READ(UR1,'(A)') PROSA          
          READ(UR1,*) aMZ_THRES(1)
          READ(UR1,*) aMZ_THRES(2)
          READ(UR1,'(A)') PROSA
          READ(UR1,*) TWINTHRES_MZ
          !END
          call INTER_MAT   
          
        CLOSE(UR1)
                
C       BELOW, IF UNKNOWN, THEN SET TO ROOM TEMPERATURE VALUES       
        write(*,*) 'tempi', tempi
        IF(TEMPERAT.LE.0.0) TEMPERAT=tempi
        IF(EDOT.LE.0.0)     EDOT=EREF
        write(10,*) 'temperature',temperat
        write(10,*) RHO_INITS(1),RHO_INITW(1)
        
c------Just in case you want it--------------------------------------c        
C       MORE EXACT RELATIONSHIP BETWEEN SHEAR MOD AND TEMP FOR ZR
C       XGMOD=(41.04298-0.02939*TEMPERAT+
C     #    9.51267E-6*TEMPERAT^2.)*1.e+03
c        XGMOD=(40.06-0.022*TEMPERAT)*1.e+03     
c        !!For Zr (IN MPA*1000=GPA)
c-----------------------------------------------------------------c 
             
                      
          DO KKK=NGR(IPH-1)+1,NGR(IPH)
          IPTS=ABS(KTWSMX(KKK))        

C       ASSIGNING INITIAL VALUES FOR SLIP  
c *** ADDEDMZ comments
c     Transiting from mode to system. Each loop where there are slip modes
c     needs to have another loop with slip systems. First there are initial
c     values. Dislocation denity is now an array over slip systems.
c     (1) inside this first loop over slip modes loop over systems is added

          ISM=0
          ISM_S = 0 !ADDEDMZ
            DO KMO=1,NSLIPMODES
                DO IS_S = 1, NSM(KMO,IPHEL) ! ADDEDMZ over systems
                    ISM_S = ISM_S+1 !counter over all slip systems
                    
                    IF(RHO_S(ISM_S,KKK).LE.RHO_INITS(KMO))THEN ! changed to be per system
                    RHO_S(ISM_S,KKK)=RHO_INITS(KMO)
                    ENDIF
                    IF(RHO_DEB(KKK).LE.RHO_INITW(KMO))THEN
                    RHO_DEB(KKK)=RHO_INITW(KMO)
                    ENDIF
                ENDDO !ADDEDMZ end do over systems
            ENDDO
               
c     ADDEDMZ (comment)                    
c     These are the terms that set up the initial value of CRSS. 
c     They can remain per mode since they are the same for all systems in one mode. 
          ISM=0
          ISM_S = 0 !ADDEDMZ
            DO KMO=1,NSLIPMODES
c     ADDEDMZ - changing initial CRSS to be dependante on temperature.
              if (ipyr(KMO).eq.1) then
                if (KMO.eq.1) then  
                ATAU(KMO)=AONE(KMO)*(1.+BONE(KMO)*LOG(EDOT/CONE(KMO)))
     #          *EXP(-TEMPERAT/DONE(KMO))
     #          *(1.0+A_prism*1./(TEMPERAT*sqrt(2.0*pi*sigma_prism))
     #         *exp(-(log(TEMPERAT)-ami_prism)**2/(2.0*sigma_prism**2)))
                endif
                if (KMO.eq.2) then  
                ATAU(KMO)=AONE(KMO)*(1.+BONE(KMO)*LOG(EDOT/CONE(KMO)))
     #          *EXP(-TEMPERAT/DONE(KMO))
     #          *(1.0+A_pyr*1./(TEMPERAT*sqrt(2.0*pi*sigma_pyr))
     #          *exp(-(log(TEMPERAT)-ami_pyr)**2/(2.0*sigma_pyr**2)))
                endif                
              else
                ATAU(KMO)=AONE(KMO)*(1.+BONE(KMO)*LOG(EDOT/CONE(KMO)))
     #          *EXP(-TEMPERAT/DONE(KMO))
              endif
c     END                
                Q5(KMO)=Q0(KMO)+Q1(KMO)*log(1.+temperat/Q2(KMO)) 
cmk               Q5(KMO)=Q0(KMO)*exp(-Q3(KMO)*EDOT)+Q1(KMO)*
cmk     #          log(1.+temperat/Q2(KMO)) 
                if(kkk.eq.500) write(10,*) 'q5=',q5(kmo),kmo
            
c                 if(kmo.eq.3)write(10,*)'q5for pyr',kmo,q5(kmo)

c     ADDEDMZ (comment)
c     Changes made in per system code should be moved at this place
c     to calculate K2 per system using the interaction between systems.
c     For now we're gona leave it as it is. 

                K2(KMO)=ALPHA_INACT*BURG(KMO)/ACTENER(KMO)
     #          *(1.-KOVB3(KMO)*TEMPERAT/DRAG(KMO)
     #          *LOG(EDOT/EDOTESO(KMO)))
                RHOSAT(KMO)=K2(KMO)**(-2.0)
                 
c     ADDEDMZ (comment) 
c     Here CRSS from dislocations is calculated. It is changed so that
c     now disl. dens. per system is used.
                 
                DO IS=1,NSM(KMO,IPHEL)
                    ISM=ISM+1
cmk                
                IF(RHO_S(ISM,KKK).LE.RHO_INITS(KMO))THEN !ADDEDMZ (RHO_S(ISM,KKK)...
                RHO_S(ISM,KKK)=RHO_INITS(KMO)
                ENDIF                
cmk                    
         
c *** ADDEDMZ
                DENOM(ISM)=0
                
                ISM_MZ=0
                  DO KMO_MZ=1,NSLIPMODES
                     DO IS_MZ=1,NSM(KMO_MZ,IPHEL)
                     ISM_MZ=ISM_MZ+1    
                     
                  TAU_SAT(ISM_MZ)=(DRAG(KMO_MZ)*
     #            ACTENER(KMO_MZ)*XGMOD)/(DRAG(KMO_MZ)/KOVB3(KMO_MZ)-
     #            TEMPERAT*log(EDOT/EDOTESO(KMO_MZ)))/KOVB3(KMO_MZ)
                  
                   DENOM(ISM)=DENOM(ISM)+XINTER_SYS_INV(ISM,ISM_MZ)
     #             *(TAU_SAT(ISM_MZ)/BURG(KMO_MZ))**2
              
                     ENDDO
                  ENDDO

                  K2_MZ(ISM)=XGMOD/sqrt(DENOM(ISM)) !MZ_inter_2nd
c                  K2_MZ(ISM)=K2(KMO) !MZ_inter_2nd
                  RHOSAT(ISM)=K2_MZ(ISM)**(-2.0) !ADDEDMZ moved it here
              ENDDO
          ENDDO

                  
                  
            ISM=0
            ISM_S = 0 !ADDEDMZ
            DO KMO=1,NSLIPMODES                  
                DO IS=1,NSM(KMO,IPHEL)
                    ISM=ISM+1                  
                  
                do ix=1,NSLSYS(iph)
                    TAUE(ISM,KKK)=TAUE(ISM,KKK)+XGMOD 
     #             *BURG(KMO)*(SQRT(RHO_S(ix,KKK)*XINTER_SYS(ISM,ix)))
                enddo                
                TAUE(ISM,KKK)=TAUE(ISM,KKK)+ATAU(KMO)
c     END
                    
                    IF(RHO_DEB(KKK).GT.RHO_INITW(KMO))THEN
                    TAUE(ISM,KKK)=TAUE(ISM,KKK)
     #              -0.086/ALPHA_INACT*SQRT(RHO_DEB(KKK))
     #              *LOG(BURG(KMO)*SQRT(RHO_DEB(KKK)))
                    ENDIF                    
     
C           HALL-PETCH EFFECT PARAMETERS FOR SLIP
         
                    HPFAC(ISM,IPHEL)=SQRT(BURG(KMO))*HPK0(KMO)
     #              *XGMOD
         
                    TAUHP =HPFAC(ISM,IPHEL)/SQRT(GRSZE)
                    CRSS(ISM,KKK)=TAUE(ISM,KKK)+TAUHP
                
                ENDDO            !IS
          ENDDO           !KMO
                
            
C       ASSIGNING INITIAL VALUES FOR TWINNING

c     ADDEDMZ (comment)
c     Here is CRSS calculated for twinning. There is a member of
c     twinning CRSS that takes into account disl. dens. (interaction term)
c     However for some reason this term is not present in the initial CRSS
c     of twin system. Therefore no changes are needed here.
            
          DO 1839 MNN=1,NTWMOD(IPHEL)
            if (i_tw(MNN).eq.0) then  
            TAU_PROP(MNN)=YPROP(MNN)+
     #         APROP(MNN)*EXP(-TEMPERAT/TPROP(MNN))
            TAU_CRIT(MNN)=YCRIT(MNN)+
     #         ACRIT(MNN)*EXP(-TEMPERAT/TCRIT(MNN))
            elseif (i_tw(MNN).eq.1) then
              if (TEMPERAT.le.T_tw(MNN)) TAU_PROP(MNN)=P_tw_1(MNN)
              if (TEMPERAT.gt.T_tw(MNN)) TAU_PROP(MNN)=P_tw_2(MNN)
              if (TEMPERAT.le.T_tw(MNN)) TAU_CRIT(MNN)=C_tw_1(MNN)
              if (TEMPERAT.gt.T_tw(MNN)) TAU_CRIT(MNN)=C_tw_2(MNN)
            elseif (i_tw(MNN).eq.2) then
              TAU_PROP(MNN)=EXP(TEMPERAT/V_tw_p(MNN))*(B_tw_p(MNN)-
     #          A_tw_p(MNN)*1/(TEMPERAT*sqrt(2.0*pi*sigma_tw_p(MNN)))
     #          *exp(-(log(TEMPERAT)-ami_tw_p(MNN))**2
     #          /(2.0*sigma_tw_p(MNN)**2)))
              TAU_CRIT(MNN)=EXP(TEMPERAT/V_tw(MNN))*(B_tw(MNN)-
     #          A_tw(MNN)*1/(TEMPERAT*sqrt(2.0*pi*sigma_tw(MNN)))
     #          *exp(-(log(TEMPERAT)-ami_tw(MNN))**2
     #          /(2.0*sigma_tw(MNN)**2)))              
            endif
1839      ENDDO

          DO IS=NSLSYP+1,NSYST(IPHEL)

            !MZ_U - identify mode 
            ns=IS          
            do imo=1,NMODES(IPHEL)
                ns=ns-NSM(imo,IPHEL)
                if (ns.le.0) then
                  JTWIN=imo
                  EXIT
                endif
            enddo 
            JTWIN=JTWIN-NSLMOD(IPHEL)
             
             RHOTEMP=0.0
			 ! WRITE(*,*),'KTWMMX(KKK)',KTWMMX(KKK)
             IF (KTWMMX(KKK).EQ.0)THEN			 
             TAUE(IS,KKK)=TAUE(IS,KKK)+TAU_PROP(JTWIN)+
     #        (TAU_CRIT(JTWIN)-TAU_PROP(JTWIN))
     #        *EXP(-RHOTEMP)
!	         WRITE(2974,*),'TAUE(IS,KKK),NUC',KKK,TAUE(IS,KKK)
	         ELSE
			 TAUE(IS,KKK)=TAU_PROP(JTWIN)
!			 WRITE(2974,*),'TAUE(IS,KKK),PROP',KKK,TAUE(IS,KKK)
			 ENDIF
             
!               WRITE(2974,*)'TAUE(15,30)',TAUE(15,30)
!               WRITE(2974,*)'HPFAC(15,30)',HPFAC(15,30)
              CRSS(IS,KKK)=TAUE(IS,KKK)+HPFAC(IS,IPHEL)/SQRT(GRSZE)

          ENDDO  ! FOR IS
           
            ngg=699
            iprx=iprint*(ngg/kkk)*(kkk/ngg)+
     #           iprint*((ngg+ngr(1))/kkk)*(kkk/(ngg+ngr(1)))
            if(iprx.ne.0) then
              write(10,'('' initial CRSS for grain # '', i4)') kkk
ccc           write(10,'(12f6.0)') (crss(is,kkk),is=1,nsyst(iphel))
              write(10,*) (crss(is,kkk),is=1,nsyst(iphel))
            endif

          ENDDO     !KKK
        ENDDO       !IPHEL

C *** CALCULATES INVERSE OF THE MEAN FREE PATH THAT EACH TWINNING SYSTEM
C *** DEFINES FOR THE OTHER SLIP OR TWINNING SYSTEMS IN THE GRAIN
        IPHEL=1
        NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)

        WRITE(10,*)
        WRITE(10,'(''    SYSMFP(IS,IT)'')')
ccc     WRITE(10,'('' IT'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        DO IT=NSLSYP+1,NSYST(IPHEL)
          ITX=IT-NSLSYP
          DO I=1,3
            DNT(I)=DNCA(I,IT,IPHEL)
          ENDDO
          DO IS=1,NSYST(IPHEL)
            DO I=1,3
C             DB1(I)=DBCA(I,IS,IPHEL)
              DN1(I)=DNCA(I,IS,IPHEL)
            ENDDO
            COSALF=ABS(DNT(1)*DN1(1)+DNT(2)*DN1(2)+DNT(3)*DN1(3))
            IF(COSALF.GT.1.) COSALF=1.
            ALF=ACOS(COSALF)
            COSBET=COS(PI/2.-ALF)
            IF(COSBET.LE.0.01) COSBET=0.01 ! coplanar (including self) systems
            SYSMFP(IS,ITX)=1./COSBET       ! inverted 4/4/2005


          ENDDO
ccc       WRITE(10,'(I3,2X,30F6.3)')
ccc  #          IT,(SYSMFP(IS,ITX),IS=1,NSYST(IPHEL))
        ENDDO

C *** CALCULATES CRITERION OF 'SLIP-TRANSFER' ACROSS TWIN BOUNDARIES.
C *** FOR EVERY TWINNING PLANE FINDS THE INTERSECTION OF ALL SLIP AND
C *** TWINNING PLANES. DEFINES A TRANSFER FACTOR FOR EVERY PAIR.

        ISLIPTR=0
C       WRITE(*,*) 'ENTER 0 TO SUPPRESS SLIP TRANSFER ACROSS INTERFACE'
C       WRITE(*,*) 'ENTER 1 FOR CHALMERS SLIP TRANSFER'
C       WRITE(*,*) 'ENTER 2 FOR SHEN SLIP TRANSFER  --> '
C       READ(*,*)   ISLIPTR

        TINY=1.E-4
        IPHEL=1
        NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)

C       WRITE(10,*)
C       WRITE(10,'(''        BARRIER(IS1,IS2,IT)'')')
C       WRITE(10,'(''  IT IS1'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        DO IT=NSLSYP+1,NSYST(IPHEL)
          ITX=IT-NSLSYP
          DO I=1,3
            DNT(I)=DNCA(I,IT,IPHEL)
            DBT(I)=DBCA(I,IT,IPHEL)
          ENDDO
          CALL TWIN_ORIENTATION (DBT,ATWIN)   ! ATWIN transf from TW to CR

          DO IS1=1,NSYST(IPHEL)
            INDEX=1
            PROD1=0.
            DO I=1,3
              DN1(I)=DNCA(I,IS1,IPHEL)
              DB1(I)=DBCA(I,IS1,IPHEL)
              PROD1=PROD1+DN1(I)*DNT(I)
              AA(1,I)=DN1(I)
              AA(2,I)=DNT(I)
              IF(ABS(DN1(I))+ABS(DNT(I)) .LT. TINY) INDEX=I
              DL1(I)=(I/3)*(3/I)
              DO J=1,3
                SCH1(I,J)=DN1(I)*DB1(J)+DN1(J)*DB1(I)
              ENDDO
            ENDDO
            SNORM1=TNORM(SCH1,3,3)
            PROD1=ABS(PROD1)

            IF (ABS(PROD1-1.) .GT. TINY) THEN     ! avoid N1 // NT
              DO I=1,3
                AA(3,I)=(I/INDEX)*(INDEX/I)
              ENDDO
              CALL LU_EQSYSTEM(AA,DL1,3,IER)     ! intersection of IS1 w/IT
              DO I=1,3
                DNORM=SQRT(DL1(1)**2+DL1(2)**2+DL1(3)**2)
              ENDDO
              DO I=1,3
                DL1(I)=DL1(I)/DNORM
              ENDDO
            ENDIF

C           write(10,'('' it is1 dnt dn1 dl1'',2i3,9f8.3)')
C    #            it,is1,(dnt(i),i=1,3),(dn1(i),i=1,3),(dl1(i),i=1,3)

            DO IS2=1,NSYST(IPHEL)
              INDEX=1
              PROD2=0.
              DO I=1,3
                DN2(I)=0.
                DB2(I)=0.
                DO J=1,3
                  DN2(I)=DN2(I)+ATWIN(J,I)*DNCA(J,IS2,IPHEL)
                  DB2(I)=DB2(I)+ATWIN(J,I)*DBCA(J,IS2,IPHEL)
                ENDDO
                PROD2=PROD2+DN2(I)*DNT(I)
                AA(1,I)=DN2(I)
                AA(2,I)=DNT(I)
                IF(ABS(DN2(I))+ABS(DNT(I)) .LT. TINY) INDEX=I
                DL2(I)=(I/3)*(3/I)
                DO J=1,3
                  SCH2(I,J)=DN2(I)*DB2(J)+DN2(J)*DB2(I)
                ENDDO
              ENDDO
              SNORM2=TNORM(SCH2,3,3)
              PROD2=ABS(PROD2)

              IF (ABS(PROD2-1.) .GT. TINY) THEN     ! avoid N2 // NT
                DO I=1,3
                  AA(3,I)=(I/INDEX)*(INDEX/I)
                ENDDO
                CALL LU_EQSYSTEM(AA,DL2,3,IER)     ! intersection of IS2 w/IT
                DO I=1,3
                  DNORM=SQRT(DL2(1)**2+DL2(2)**2+DL2(3)**2)
                ENDDO
                DO I=1,3
                  DL2(I)=DL2(I)/DNORM
                ENDDO
              ENDIF

              IF (ABS(PROD1-1.).GT.TINY.AND.ABS(PROD2-1.).GT.TINY) THEN
                BARRIERB=0.
                BARRIERL=0.
                BARRIERN=0.
                BARRIERS=0.
                DO I=1,3
                  BARRIERB=BARRIERB+DB1(I)*DB2(I)
                  BARRIERL=BARRIERL+DL1(I)*DL2(I)
                  BARRIERN=BARRIERN+DN1(I)*DN2(I)
                  DO J=1,3
                    BARRIERS=BARRIERS+SCH1(I,J)*SCH2(I,J)/SNORM1/SNORM2
                  ENDDO
                ENDDO
                IF (ISLIPTR.EQ.0) BARRIER(IS1,IS2,ITX)=0.    ! perfect barrier
                IF (ISLIPTR.EQ.1)
     #            BARRIER(IS1,IS2,ITX)=ABS(BARRIERS)          ! Liv. & Chalm.
                IF (ISLIPTR.EQ.2)
     #            BARRIER(IS1,IS2,ITX)=ABS(BARRIERL*BARRIERB)  ! Shen et al.

C               write(10,'('' it is2 dnt dn2 dl2'',2i3,9f8.3,3x,2f8.3)')
C    #            it,is2,dnt,dn2,dl2,barrierl,barrierb
              ELSE
                BARRIER(IS1,IS2,ITX)=0.

C               write(10,'('' it is2 dnt dn2'',2i3,6f8.3,3x,2f8.3)')
C    #            it,is2,dnt,dn2,prod1,prod2
              ENDIF

            ENDDO      ! end of DO loop over IS2

C           WRITE(10,'(2I4,2X,31F6.3)')
C    #             IT,IS1,(BARRIER(IS1,IS2,ITX),IS2=1,NSYST(IPHEL)),BMAX

          ENDDO      ! end of DO loop over IS1
        ENDDO      ! end of DO loop over IT

C *** SEARCH FOR MAXIMUM 'TRANSFER FACTOR' OF 'IS1' ACROSS 'IT'.
C *** APPLIES TO CRYSTALLOGRAPHICALLY EQUIVALENT SLIP SYSTEMS.
C *** SETS TO ZERO TRANSFER OF TWINNING SYSTEMS.

        WRITE(10,*)
        WRITE(10,'(''      TRANSFER(IS1,IT)'')')
ccc     WRITE(10,'(''  IT'',2X,30I6)') (I,I=1,NSYST(IPHEL))

        NSLSYS=NSYST(IPHEL)-NTWSYS(IPHEL)
        DO IT=1,NTWSYS(IPHEL)
          I=0
          DO IM1=1,NMODES(IPHEL)
          DO IS1=1,NSM(IM1,IPHEL)
            I=I+1
            J=0
            BMAX=0
            DO IM2=1,NMODES(IPHEL)
            DO IS2=1,NSM(IM2,IPHEL)
              J=J+1
C --> TRANSFER ALLOWED ONLY AMONG SYSTEMS OF THE SAME MODE
              IF(IM2.EQ.IM1) THEN
                BAUX=BARRIER(I,J,IT)
                IF (BAUX.GT.BMAX) BMAX=BAUX
              ENDIF
            ENDDO
            ENDDO
            TRANSFER(I,IT)=BMAX
          ENDDO
          ENDDO
C *** ENFORCES NO TRANSFER OF TWIN THROUGH TWIN, EXCEPT 'SELF'
          DO IT1=NSLSYP+1,NSYST(IPHEL)
            TRANSFER(IT1,IT)=0.
            IF(IT+NSLSYP.EQ.IT1) TRANSFER(IT1,IT)=1.
          ENDDO
ccc       WRITE(10,'(I4,2X,30F6.3)')
ccc  #             IT,(TRANSFER(IS1,IT),IS1=1,NSYST(IPHEL))
        ENDDO
        TRANSFER(1,1)=TRANSFER(1,1)      ! to fool the compiler

      ENDIF      ! END OF IOPTION=1
c-----------------------------------------------------------------------------------------------------------------------CHECKPOINT 1
C **************************************************************************
C *** UPDATES CRSS's IN PARENT AND CHILD
C **************************************************************************

      IF (IOPTION.EQ.2) THEN
          
c *** ADDEDMZ (comment)
c     Here is where major changes are expected. So now each system has disloc
c     dens that needs to be updated. Each CRSS uses disloc dens of corresponding
c     system.    
c     There are two ways to make change in certain equations. These equations are
c     DELRHOW and twin - slip interaction calculation. We can sum all the dislocation
c     densities from all the systems or we can take the average per mode and sum 
c     them over modes. First aproach requires change in input parameters, while
c     second does not.          
c *** ADDEDMZ (comment)          
c     Before main loop we have calculation of K2 and K1 which are responsible for
c     evolution of DD. k2/k1 is here named K2.          
        temperat=tempi
       
        write(10,'(/,'' ENTERING UPDATE_CRSS_CG IOPT=2'')')
        EDOT=SQRT(2./3.)*TNORM(DBAR,5,1)

        
        iseed2=iseed1
        KGX=1
        DO IPH=IPHBOT,IPHTOP
              IPHEL=IPH-IPHBOT+1

              NSLSYP=NSYST(IPHEL)-NTWSYS(IPHEL)

              NSLIPMODES=NMODES(IPHEL)-NTWMOD(IPHEL)
              KTWINTOT=0
              DO KCNT=1,NSLIPMODES+1
                  KTWINTOT=KTWINTOT+NSM(KCNT,IPHEL)
              ENDDO

C       WITH EDOT, CALCULATING K2 AND RHOSAT FOR EACH PHASE     
C       K2 IS THE RATIO OF THE ANNIL TERM TO THE GENERATION TERM
                
          K2(:)=0.  !ADDEDMZ
          K2_MZ(:)=0.  !ADDEDMZ
          DENOM(:)=0.  !ADDEDMZ
          TAU_SAT(:)=0.   !ADDEDMZ
          RHOSAT(:)=0.   !ADDEDMZ
          ISM=0
              DO 2053 KMO=1,NSLIPMODES
                                     
c                K2(KMO)=ALPHA_INACT*BURG(KMO)/ACTENER(KMO) ! ADDEDMZ commented out !!MZ_inter_2nd
c     #           *(1.-KOVB3(KMO)*TEMPERAT/DRAG(KMO)
c     #           *LOG(EDOT/EDOTESO(KMO)))

c                 RHOSAT(KMO)=K2(KMO)**(-2.0)
                 
c *** ADDEDMZ
               
               DO IS=1,NSM(KMO,IPHEL)
                    ISM=ISM+1                    
                    ISM_MZ=0
                  DO KMO_MZ=1,NSLIPMODES
                     DO IS_MZ=1,NSM(KMO_MZ,IPHEL)
                     ISM_MZ=ISM_MZ+1    
                     
                     TAU_SAT(ISM_MZ)=(DRAG(KMO_MZ)*
     #               ACTENER(KMO_MZ)*XGMOD)/(DRAG(KMO_MZ)-
     #               TEMPERAT*log(EDOT/EDOTESO(KMO_MZ))*KOVB3(KMO_MZ))
                  
                     DENOM(ISM)=DENOM(ISM)+XINTER_SYS_INV(ISM,ISM_MZ)
     #               *(TAU_SAT(ISM_MZ)/BURG(KMO_MZ)/XGMOD)**2
                   
                     ENDDO
                  ENDDO
                  
                  K2_MZ(ISM)=1.0/sqrt(DENOM(ISM)) !MZ_inter_2nd
c                  K2_MZ(ISM)=K2(KMO)
                  RHOSAT(ISM)=K2_MZ(ISM)**(-2.0) !MZ_inter_2nd
C                  PRINT *,'RHOSAT(ISM),ISM',RHOSAT(ISM),ISM
                ENDDO
                  
c     END                    

2053           ENDDO
        
        !debug
c        RHOSAT(2)=RHOSAT(4)
c      write(126,'(15e16.5)') RHOSAT(1:15)  !K2_MZ(1:15),RHOSAT(1:15)      
        
c *** ADDEDMZ - comment
c     The loop over all the grains starts here and with counter KKK( in phase counter)
c     and overall counter KGX.    
c     Structure in the main loop over all grains is as follows:
c     (1) Calculations connected to CG model. (Mean free path, barriers,...)  
c     (2) Update of dislocation densities and calculation of tau0.   
c     (3) Calculation of CRSS.   
c         (3.1) Calculation of tau_deb and tau_for (in one equation and summing
c         with tau0). (here hall-petch coefficient is also updated)
c         (3.2) Calculation of tau_slip_twin (interaction). Update of CRSS for
c         twinning. Calculation of hall - petch factor.
c         (3.3) Blocking of PTS when threshold is reached.        
      DO KKK=NGR(IPH-1)+1,NGR(IPH)
c     (1) ****************************************************************************         
          ngp=699
          ngc=ngp+ngr(1)
          iprx=iprint*((ngp/kkk)*(kkk/ngp)+(ngc/kkk)*(kkk/ngc))

        IF(WGT(KKK).GT.1.E-8) THEN   ! AVOID 'GHOST' GRAINS IN TWINNED PHASE !MZ changed E-6 to E-8

C *** DEFINES A MINIMUM MEAN FREE PATH 'DMFPX' IN GRAIN/PARENT/CHILD
C *** DEFINES A BARRIER FACTOR 'BARF' TO DELAY BARRIER EFFECT.
C *** NEW APPROACH WITH INCREASING TWIN THICKNESS:
C     * TWTHRES1: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg] 1ST GENERATION
C     * TWTHRES2: MAXIMUM TWIN FRACTION IN THE GRAIN 1ST GENERATION
C     * TWTHRES3: SEPARATION BETWEEN MIDPLANE OF LAMELLAE [Dg] 2ND GENERATION
C     * TWTHRES4: MAXIMUM TWIN FRACTION IN THE GRAIN 2ND GENERATION

          IPTS=ABS(KTWSMX(KKK))
          IPTM=KTWMMX(KKK)
          BARF=0.
          TWFRPTS=0.
          DMFPX=GRSZE
          CONDITION=0
          TH0X=0.05 

C ** DIFFERENT CONDITIONS TO CALCULATE DMFPX DEPENDING ON PHASE (GP)
C ** CODE WRITTEN AS IF THERE WERE 2 OR 3 PHASES
          IF(IPHEL.EQ.1) THEN
            IF(IPTS.NE.0) THEN
               TH1X=TWTHRES(1,IPTM,1)
               TH2X=TWTHRES(2,IPTM,1)
               CONDITION=1
             ENDIF
          ELSE IF(IPHEL.EQ.2) THEN
            IF(IPHTOP.LE.2) THEN
               IPTM_PARENT=KTWMMX(KKK-NGR(1))
               TH1X=TWTHRES(1,IPTM_PARENT,1)
               TH2X=TWTHRES(2,IPTM_PARENT,1)
               CONDITION=2
            ELSE
               IF(IPTS.EQ.0) THEN    !VERIFY THAT grand CHILD HAS NOT BEEN CREATED YET
                 IPTM_PARENT=KTWMMX(KKK-NGR(1))
                 TH1X=TWTHRES(1,IPTM_PARENT,1)
                 TH2X=TWTHRES(2,IPTM_PARENT,1)
                 CONDITION=2
               ELSE
                 IPTM_PARENT=KTWMMX(KKK-NGR(1))
                 TH1X=TWTHRES(3,IPTM,1)*TWTHRES(1,IPTM_PARENT,1) !DISTRIBUTES THE 2NDARY TWINS OVER ALL THE FIRST TWIN LAMELLAE
                 TH2X=TWTHRES(4,IPTM,1)
                 CONDITION=3
                ENDIF
             ENDIF
          ELSE IF(IPHEL.EQ.3) THEN
              IPTM_PARENT=KTWMMX(KKK-2*NGR(1))
              TH1X=TWTHRES(3,IPTM,1)*TWTHRES(1,IPTM_PARENT,1) !DISTRIBUTES THE 2NDARY TWINS OVER ALL THE FIRST TWIN LAMELLAE
              TH2X=TWTHRES(4,IPTM,1) !THIS REQUIRES THAT THE GRAND-CHILD IS TAGGED AS THE CHILD
              CONDITION=4
          ENDIF

          IF(CONDITION.NE.0) THEN !THIS DOESN'T RUN IF GRAIN WITHOUT TWIN

C *** VERSION JULY 5 2006 (GP)
C *** TWFRPTS WILL BE USE IN PARENT PHASE TO PREVENT MORE PTS DEFORMATION
C *** TWFRPTS IN PARENT WILL BE USED IN CHILD TO CALCULATED DMFPX FOR HP EFFECT
C *** SAME REASONING BETWEEN CHILD AND GRAND-CHILD

C ** NEW CONDITIONS TO CALCULATE TWIN FRACTION (GP)
            IF(IPHEL.EQ.1) TWFRPTS=TWFRSY(IPTS-NSLSYP,KKK)
            IF(IPHEL.EQ.2) THEN
               IF(IPHTOP.LE.2) THEN
                  IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                  TWFRPTS=TWFRSY(IPTS_PARENT-NSLSYP,KKK-NGR(1))
               ELSE
                  IF(IPTS.EQ.0) THEN
                    IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
                    TWFRPTS=TWFRSY(IPTS_PARENT-NSLSYP,KKK-NGR(1))
                  ELSE
                    TWFRPTS=TWFRSY(IPTS-NSLSYP,KKK)
                  ENDIF
               ENDIF
            ENDIF
            IF(IPHEL.EQ.3) TWFRPTS=TWFRSY(IPTS-NSLSYP,KKK-NGR(1))

            IF(CONDITION.EQ.1) THEN
              PREDOM=IPTS
C *** POWER LAW INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
              POWER=0.5
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X) 
     #                 BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER_MZ !MZ POWER_MZ instead of POWER, now it's read from input file
              IF(TWFRPTS.GE.TH2X) BARF=1.

C *** SIGMOIDAL INTERPOLATION FOR BARRIER FACTOR (0<BARF<1)
C             SLOPE=2.
C             TFX=(2*TWFRPTS-TH0X-TH2X)/(TH2X-TH0X)
C             BARF=1./(1.+EXP(-SLOPE*TFX))

              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE
              DMFPX_HP=(1.-TWFRPTS)*TH1X*GRSZE

            ELSE IF(CONDITION.EQ.2) THEN
              IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
              PREDOM=IPTS_PARENT
              BARF=1.                      ! assumes full barrier in twin
              DMFPX_HP=TWFRPTS*TH1X*GRSZE     ! for increasing twin thickness FOR HP
              DMFPX=TH2X*TH1X*GRSZE        ! assumes always maximum thickness FOR GNDs

            ELSE IF(CONDITION.EQ.3) THEN   ! for conditions 3 and 4, we need to change the value of grsze
              PREDOM=IPTS
              IPTS_PARENT=ABS(KTWSMX(KKK-NGR(1)))
              IPTM_PARENT=KTWMMX(KKK-NGR(1))
              TWFRPTS_PARENT=TWFRSY(IPTS_PARENT-NSLSYP,KKK-NGR(1))
              GRSZE_NEW=2*TWFRPTS_PARENT*TWTHRES(1,IPTM_PARENT,1)*GRSZE
              POWER=1.0
              IF(TWFRPTS.LE.TH0X) BARF=0.
              IF(TWFRPTS.GT.TH0X.AND.TWFRPTS.LT.TH2X)
     #                   BARF=((TWFRPTS-TH0X)/(TH2X-TH0X))**POWER
              IF(TWFRPTS.GE.TH2X) BARF=1.
              DMFPX=(1.-TWFRPTS)*TH1X*GRSZE_NEW
              DMFPX_HP=(1.-TWFRPTS)*TH1X*GRSZE_NEW

            ELSE IF(CONDITION.EQ.4) THEN
              PREDOM=IPTS
              IPTS_PARENT=ABS(KTWSMX(KKK-2*NGR(1)))
              IPTM_PARENT=KTWMMX(KKK-2*NGR(1))
              TWFRPTS_PARENT=TWFRSY(IPTS_PARENT-NSLSYP,KKK-2*NGR(1))
              GRSZE_NEW=2*TWFRPTS_PARENT*TWTHRES(1,IPTM_PARENT,1)*GRSZE
              BARF=1.                      ! assumes full barrier in twin
              DMFPX_HP=TWFRPTS*TH1X*GRSZE_NEW
              DMFPX=TH2X*TH1X*GRSZE_NEW
            ENDIF
          ENDIF

          IF(DMFPX.LT.0.) THEN
            WRITE(*,'('' DMFPX<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx
            STOP
          ENDIF
          IF(DMFPX_HP.LT.0.) THEN
            WRITE(*,'('' DMFPX_HP<0 INSIDE UPD_CRSS_CG    !!!!!'')')
            write(*,'('' kkk ipts twfrpts dmfpx_hp'',2i5,6x,3f7.3)')
     #                   kkk,ipts,twfrpts,dmfpx_HP
            STOP
          ENDIF


C *** 'PTS' HARDENS ALL SYSTEMS VIA DIRECTIONAL HALL-PETCH & GND EFFECTS.
c     (2)**********************************************************************
c *** ADDEDMZ (comment)
c     Here we have calculation of increment of disl. dens. (DELRHOS
c     and DELRHOW). Here change is needed to make update of DELRHOS and DELRHOW
c     over the system. So each system is evolving differently.
c     To update system disl dens formula is:
c     rho_s = rho_s + delrhos*delgama
c     In this equation everything is over the systems. We can transfere this 
c     equtaion into the:
c     rho_s = rho_s + delrhos*gamdot*tincr
c     Here gamdot is already over system. Same equation works for DELRHOW.

c *** ADDEDMZ - commented out because they are moved down in the
c     loop over the systems. DELGAM is needed for rho deb.
            
          GAMTOTX=GTOTGR(KKK)
          DELTGAM=0.0
          DO IS=1,NSYST(IPHEL)
             IF(IS.NE.IPTS) DELTGAM=DELTGAM+ABS(GAMDOT(IS,KKK))*TINCR
          ENDDO

            ISM=0
            ISM_S = 0 ! ADDEDMZ   
          SUM_OF_T_SHEAR_RATE=0.0
		  DO IT=16,21
			SUM_OF_T_SHEAR_RATE=SUM_OF_T_SHEAR_RATE+ABS(GAMDOT(IT,KKK))
		  ENDDO
c		 PRINT *,'SUM_OF_SHEAR_RATE',SUM_OF_T_SHEAR_RATE,GAMDOT(17,KKK)
			KGENER(1)= KGENER_1*(exp(-C_T_S(1)*SUM_OF_T_SHEAR_RATE/0.01289))
			KGENER(2)= KGENER_2*(exp(-C_T_S(2)*SUM_OF_T_SHEAR_RATE/0.01289))
			KGENER(3)= KGENER_3*(exp(-C_T_S(3)*SUM_OF_T_SHEAR_RATE/0.01289))
c			PRINT *,'KGENER_O',KGENER_1,KGENER_2,KGENER(1),KGENER(2)
c            KGENER(2)=KGENER_O+KGENER(2)*
c     #  (1-1/EXP(RHO_S(4,KKK)/RHOSAT(4)))
c     #		+KGENER(3)*(1-1/EXP(RHO_S(10,KKK)/RHOSAT(10)))
            
     	 
c            PRINT *,'KGENER',KGENER_O,KGENER(1),KGENER(2),KGENER(3)			
            DO KMO=1,NSLIPMODES

c *** ADDEDMZ - commented out because they are moved down in the
c     loop over the systems. DELGAM is needed for rho deb.
                    DELGAM(KMO)=0.0
c                    DELRHOS(KMO)=0.0
c                    DELRHOW(KMO)=0.0
                
                 DO ISY=1,NSM(KMO,IPHEL)
                    ISM=ISM+1
                     DELGAM(KMO)=DELGAM(KMO)+
     #               ABS(GAMDOT(ISM,KKK))*TINCR
                ENDDO           
				
                TWFRGRAIN=0
                
                DO IT=1,NTWSYS(IPHEL)
                    TWFRGRAIN=TWFRGRAIN+TWFRSY(IT,KKK)
                ENDDO
C                PRINT *,'NTWSYS(IPHEL)',NTWSYS(IPHEL)
C				PRINT *,'NSM(KMO,IPHEL)',NSM(KMO,IPHEL),KMO
                DO IS_S = 1, NSM(KMO,IPHEL)  !ADDEDMZ going over systems----SYS
                    ISM_S = ISM_S + 1  !ADDEDMZ 
                    DELRHOS(ISM_S)=0.0
                    DELRHOW(ISM_S)=0.0
C                    PRINT *,'GAMDOT',GAMDOT(16,KKK),ISM_S,KKK
                    DELRHOS(ISM_S)=KGENER(KMO)*SQRT(RHO_S(ISM_S,KKK)) !ADDEDMZ - changed to be over system
     #              *(1.0-K2_MZ(ISM_S)*SQRT(RHO_S(ISM_S,KKK)))
C                     KR=KGENER(1)*K2_MZ(1)
C					 PRINT *,'KR',KR
C               FOLLOWING IS ADDED 06/25/08 FOR ENHANCED TWIN STORAGE
                IF(IPHEL.EQ.2)THEN
                STOR=1.0
                DELRHOS(ISM_S)=KGENER(KMO)*SQRT(RHO_S(ISM_S,KKK)) !ADDEDMZ - changed to be over system
     #           *(STOR*1.0-K2_MZ(ISM_S)*SQRT(RHO_S(ISM_S,KKK)))
                ENDIF
C               BLOCK FOR TWINS ENDS HERE
        
                if (DELRHOS(ISM_S).LT.0.0) DELRHOS(ISM_S)=0.0
         
C               THE 1E+06 IN THE ABOVE IS TO CONVERT MPA TO PA

                 DELRHOW(ISM_S)=KGENER(KMO)*RHO_S(ISM_S,KKK)
     #           *K2_MZ(ISM_S)*(BURG(KMO)*SQRT(RHO_DEB(KKK)))*Q5(KMO) !/NSM(KMO,IPHEL)
         
                ENDDO !ADDEDMZ going over systems -----SYS
     
     
c            if(kkk.eq.500)write(10,*) 'pyr',kmo,q5(kmo),delrhow(kmo)
     
             ENDDO


                ISM=0
                ISM_S = 0 !ADDEDMZ                
                DO  KMO=1,NSLIPMODES
                    DO IS_S = 1, NSM(KMO,IPHEL)  !ADDEDMZ going over systems------SYS
                        ISM_S = ISM_S + 1  !ADDEDMZ                     
                        DRHOS=0.0
                        DRHOW=0.0
                        DTAU(KMO)=0.0
                    
c     ADDEDMZ - changing initial CRSS to be dependante on temperature.  
              if (ipyr(KMO).eq.1) then
                if (KMO.eq.1) then  
                ATAU(KMO)=AONE(KMO)*(1.+BONE(KMO)*LOG(EDOT/CONE(KMO)))
     #          *EXP(-TEMPERAT/DONE(KMO))
     #          *(1.0+A_prism*1/(TEMPERAT*sqrt(2.0*pi*sigma_prism))
     #         *exp(-(log(TEMPERAT)-ami_prism)**2/(2.0*sigma_prism**2)))
                endif
                if (KMO.eq.2) then  
                ATAU(KMO)=AONE(KMO)*(1.+BONE(KMO)*LOG(EDOT/CONE(KMO)))
     #          *EXP(-TEMPERAT/DONE(KMO))
     #          *(1.0+A_pyr*1/(TEMPERAT*sqrt(2.0*pi*sigma_pyr))
     #          *exp(-(log(TEMPERAT)-ami_pyr)**2/(2.0*sigma_pyr**2)))
                endif                
              else
                ATAU(KMO)=AONE(KMO)*(1.+BONE(KMO)*LOG(EDOT/CONE(KMO)))
     #          *EXP(-TEMPERAT/DONE(KMO))
              endif                    
c     END   

                       RHO_S(ISM_S,KKK)=RHO_S(ISM_S,KKK)+DELRHOS(ISM_S)*
     #                    ABS(GAMDOT(ISM_S,KKK))*TINCR
     
                        RHO_DEB(KKK)=RHO_DEB(KKK)+Q6*DELRHOW(ISM_S)*
     #                    ABS(GAMDOT(ISM_S,KKK))*TINCR
                     ENDDO !ADDEDMZ going over systems ------SYS
                ENDDO            
                
      
c      if (KKK.eq.1) write(126,'(15e16.5)') K2_MZ(1:15)
c-----------------------------------------------------------------------------------------------------------------------CHECKPOINT 2
                
                
c     (3.1)****************************************************************************
c     ADDEDMZ (comment)
c     After dislocation denisities have been updated now CRSS is
c     calculated with new dislocation densities. Here we need to
c     use dislocation densities per system instead ones per mode.
              
                TAUE(:,KKK)=0. !ADDEDMZ

                ISM=0
                DO 1770 KMO=1,NSLIPMODES
    
                DO ISY=1,NSM(KMO,IPHEL)
                    ISM=ISM+1
                    
c *** ADDEDMZ
                do ix=1,NSLSYS(iph)
                    TAUE(ISM,KKK)=TAUE(ISM,KKK)+XGMOD 
     #             *BURG(KMO)*(SQRT(RHO_S(ix,KKK)*XINTER_SYS(ISM,ix)))
                enddo 
                   
                TAUE(ISM,KKK)=TAUE(ISM,KKK)+ATAU(KMO)
     #           -0.086*BURG(KMO)*XGMOD*SQRT(RHO_DEB(KKK))
     #                 *LOG(BURG(KMO)*SQRT(RHO_DEB(KKK)))
                
                
                    
c     END                      
                    
     
c                    TAUE(ISM,KKK)=ATAU(KMO)+ALPHA_INACT* !ADDEDMZ - changed disl. dens. to be over the system & commented out
c     #                 BURG(KMO)*XGMOD*(SQRT(RHO_S(ISM,KKK))
c     #                 -0.086/ALPHA_INACT*SQRT(RHO_DEB(KKK))
c     #                 *LOG(BURG(KMO)*SQRT(RHO_DEB(KKK))))
     
CCCCCCCCCCCCBLOCK FOR HALL PETCHCCCCCCCC         
            !MZ HP for twins
            if (IPHEL.gt.1) then
              IPTS=KTWSMX(KKK-NGR(IPHBOT))
              IPTM=KTWMMX(KKK-NGR(IPHBOT))
            endif
            !END 
               IF(ipts.gt.nslsyp.and.ipts.le.nslsyp+12)THEN
C               THE ABOVE IS TO CONSIDER ONLY IF THE PTS IS A TTW

                HPFAC(ISM,IPHEL)=HPK1(KMO)* !MZ commented out
     #          SQRT(BURG(KMO))* !MZ commented out
     #          XGMOD !MZ commented out
C               THE 1E+06 IN THE ABOVE IS TO CONVERT MPA TO PA
     
C           THE FOLLOWING APPLIES FOR COMPRESSION TWINNING (1122)
                ELSEIF(IPTS.GT.NSLSYP+12)THEN
                HPFAC(ISM,IPHEL)=HPK2(KMO)*XGMOD*SQRT(BURG(KMO))
                ENDIF
CCCCCCCCCCCCBLOCK FOR HALL PETCHCCCCCCCC  

                ENDDO  !ISY
    
1770  ENDDO
   
      
c     (3.2)**************************************************************
C *** UPDATE TAUE FOR TWINNING

c     ADDEDMZ (comment)
c     Here we will have to go over the system when calculating
c     the interaction between slip and twinning. But does now
c     interaction needs to be much smaller because there are
c     many systems?


          DO  MNN=1,NTWMOD(IPHEL)
            if (i_tw(MNN).eq.0) then  
            TAU_PROP(MNN)=YPROP(MNN)+
     #         APROP(MNN)*EXP(-TEMPERAT/TPROP(MNN))
            TAU_CRIT(MNN)=YCRIT(MNN)+
     #         ACRIT(MNN)*EXP(-TEMPERAT/TCRIT(MNN))
            elseif (i_tw(MNN).eq.1) then
              if (TEMPERAT.le.T_tw(MNN)) TAU_PROP(MNN)=P_tw_1(MNN)
              if (TEMPERAT.gt.T_tw(MNN)) TAU_PROP(MNN)=P_tw_2(MNN)
              if (TEMPERAT.le.T_tw(MNN)) TAU_CRIT(MNN)=C_tw_1(MNN)
              if (TEMPERAT.gt.T_tw(MNN)) TAU_CRIT(MNN)=C_tw_2(MNN)
            elseif (i_tw(MNN).eq.2) then
              TAU_PROP(MNN)=EXP(TEMPERAT/V_tw_p(MNN))*(B_tw_p(MNN)-
     #          A_tw_p(MNN)*1/(TEMPERAT*sqrt(2.0*pi*sigma_tw_p(MNN)))
     #          *exp(-(log(TEMPERAT)-ami_tw_p(MNN))**2
     #          /(2.0*sigma_tw_p(MNN)**2)))
              TAU_CRIT(MNN)=EXP(TEMPERAT/V_tw(MNN))*(B_tw(MNN)-
     #          A_tw(MNN)*1/(TEMPERAT*sqrt(2.0*pi*sigma_tw(MNN)))
     #          *exp(-(log(TEMPERAT)-ami_tw(MNN))**2
     #          /(2.0*sigma_tw(MNN)**2)))                
            endif
          ENDDO

          DO  IS=NSLSYP+1,NSYST(IPHEL)
           ! PRINT *,'NSLSYP',NSLSYP		  
          TAUE(IS,KKK) =0.0
            !MZ_U - identify mode 
            ns=IS          
            do imo=1,NMODES(IPHEL)
                ns=ns-NSM(imo,IPHEL)
                if (ns.le.0) then
                  JTWIN=imo
                   EXIT
                endif
            enddo 
            JTWIN=JTWIN-NSLMOD(IPHEL)
			! PRINT *,'JTWIN',JTWIN
          !END
            RHOTEMP=0.0
            RHOTEMP_B=0.0 !MZ_PROP
          ISM_S = 0 ! ADDEDMZ            
            DO KJO=1,NSLIPMODES
                

                DO IS_S = 1, NSM(KJO,IPHEL)  !ADDEDMZ going over systems-----SYS changed NSM(KMO,IPHEL) -> ..(KJO,..
                   ISM_S = ISM_S + 1  !ADDEDMZ            
         
c            TAUE(IS,KKK) =TAUE(IS,KKK)+XGMOD*BURG_TW(JTWIN)*
c     #      BURG(KJO)*TLATENT(KJO,JTWIN)*RHO_S(KJO,KKK)

                 TAUE(IS,KKK) =TAUE(IS,KKK)+XGMOD*BURG_TW(JTWIN)*
     #           BURG(KJO)*(TLATENT(KJO,JTWIN)-TLATENT1(KJO,JTWIN)
     #           *LOG(EDOT))*RHO_S(ISM_S,KKK) !/NSM(KJO,IPHEL)

c            if(kkk.eq.500)write(10,*) 'btwincrss',taue(is,kkk),
c     #       rho_s(kjo,kkk)
           
         if(kkk.eq.500) write (10,*) 'tlatent',tlatent(kjo,jtwin)+
     #     tlatent1(kjo,jtwin)*log(edot),kjo,jtwin
     
c               RHOTEMP=RHOTEMP+RHO_S(ISM_S,KKK)/RHOSAT(KJO) ! ADDEDMZ  commented out
c     ADDEDMZ
          !only for prismatic dislocations
          if (KJO.eq.1) then
          RHOTEMP=RHOTEMP+RHO_S(ISM_S,KKK)/RHOSAT(ISM_S)/NSM(KJO,IPHEL)!ADDEDMZ changed NSM(KMO,IPHEL) -> ..(KJO,..
          ! following portion is for use of rho_max as a measure of disloc in mode to rho_sat    
! c          if (ISM_S.eq.1) then
! c              RHOMAX=RHO_S(ISM_S,KKK)    
! c          endif
! c          if(RHO_S(ISM_S,KKK).gt.RHOMAX.and.ISM_S.gt.1)then
! c              RHOMAX=RHO_S(ISM_S,KKK)
! c          endif
! c           RHOTEMP=RHOMAX/RHOSAT(ISM_S) 
! c           
          elseif (KJO.eq.3) then!MZ_PROP ... 
          ! following portion is for use of rho_max as a measure of disloc in mode to rho_sat       
c          if (ISM_S.eq.16) then !MZ_PROP 16th is the first system of basal mode
c              RHOMAX=RHO_S(ISM_S,KKK)    
c          endif
c          if(RHO_S(ISM_S,KKK).gt.RHOMAX.and.ISM_S.gt.16)then !MZ_PROP 16th is the first system of basal mode
c              RHOMAX=RHO_S(ISM_S,KKK)
c          endif
c           RHOTEMP_B=RHOMAX/RHOSAT(ISM_S)  !...MZ_PROP 
          RHOTEMP_B=RHOTEMP_B+RHO_S(ISM_S,KKK)
     #         /RHOSAT(ISM_S)/NSM(KJO,IPHEL)
           ! PRINT *,'RHO_S',RHO_S(ISM_S,KKK),RHOSAT(ISM_S),NSM(KJO,IPHEL)	 
          endif                      
c     E ND                
                ENDDO !ADDEDMZ going over systems-----SYS            
            ENDDO
            if (JTWIN.ne.4) then  !MZ_PROP
            TAUE(IS,KKK)=TAUE(IS,KKK)+TAU_PROP(JTWIN)+
     #        (TAU_CRIT(JTWIN)-TAU_PROP(JTWIN))
     #        *exp(-RHOTEMP)
            else !MZ_PROP...
            TAUE(IS,KKK)=TAUE(IS,KKK)+TAU_PROP(JTWIN)+
     #        (TAU_CRIT(JTWIN)-TAU_PROP(JTWIN))
     #        *exp(-RHOTEMP_B*2.0) !MZ_CHG
            endif !...MZPROP
             
            if (TAUE(IS,KKK).le.0.0)TAUE(IS,KKK)=1.0
			 IF (KTWMMX(KKK).EQ.0)THEN	
             call random_number(X)
             WRITE(2974,*),'X',X				 
             TAUE(IS,KKK)=TAU_CRIT(JTWIN)+10*X
	         WRITE(2974,*),'TAUE(IS,KKK),NUC',KKK,TAUE(IS,KKK)
	         ELSE
			 TAUE(IS,KKK)=TAU_PROP(JTWIN)
			 WRITE(2974,*),'TAUE(IS,KKK),PROP',KKK,TAUE(IS,KKK)
			 ENDIF
			 
c            if(kkk.eq.500)write(10,*) 'twincrss',taue(is,kkk),
c     #       tau_prop(jtwin)

          ENDDO

c      if (KKK.eq.53) write(126,'(15e25.5)') epsvm,RHOTEMP_B,
c     #    RHOSAT(16:18),RHO_S(16:18,53) 
          
          
          
          
C *** ADD HALL-PETCH EFFECT TO TAUE FOR SLIP AND TWINNING
          DO IS=1,NSYST(IPHEL)
          !MZ find KMO (current mode)
            ns=IS
c            imo=0
c            do while (ns.gt.0)
c              imo=imo+1
c              ns=ns-NSM(imo,IPHEL)
c              KMO=imo
c            enddo
          
            do imo=1,NMODES(IPHEL)
              ns=ns-NSM(imo,IPHEL)
              if (ns.le.0) then
                KMO=imo
                EXIT
              endif
            enddo 
            
            
C *** HALL-PETCH EFFECT  
            IF(IPTS.EQ.0) DMFP(IS)=GRSZE
            IF(IPTS.GT.0) DMFP(IS)=DMFPX_HP*SYSMFP(IS,IPTS-NSLSYP)
            IF(DMFP(IS).GT.GRSZE) DMFP(IS)=GRSZE
         
            TAUHP  =BARF*HPFAC(IS,IPHEL)/SQRT(DMFP(IS))
C            TAUHP=HPFAC(IS,IPHEL)/SQRT(GRSZE)
c            TAUHP0 =HPFAC(IS,IPHEL)/SQRT(GRSZE) !MZ commented out
            !MZ
            if (KMO.le.NSLIPMODES) then
              TAUHP0 =HPK0(KMO)*SQRT(BURG(KMO))*XGMOD/SQRT(GRSZE)     !MZ instead of current HPFAC use the one for grain boundary
            else
              TAUHP0 =HPFAC(IS,IPHEL)/SQRT(GRSZE) 
            endif
            TAUHP  =MAX(TAUHP,TAUHP0)
            !MZ
            if (DMFP(IS).eq.GRSZE) TAUHP=TAUHP0
            !DEBUG
            if (kkk.eq.1.and.is.eq.18) then
c                write(126,'(30f25.5)') TWFRSY(24,1),TAUHP
            endif
                       
            !END DEBUG
c     (3.3)***********************************************************************
C *** NEW VERSION JULY 5 2006
C *** PTS IS BLOCKED IN PARENT WHEN THRES2 IS REACHED
C *** ALL THE TWIN SYSTEMS ARE BLOCKED WHEN THE SUM OF THEIR VOL FRACTION REACHES 1
C *** IN CHILD EVERY TWIN SYSTEMS KEEPS ITS CRSS (NO PTSDAMP FACTOR) IF NO GRAND-CHILD CONSIDERED
C *** -->NO SYSTEM BLOCKED IN CHILD IF NO GRAND-CHILD CONSIDERED
C *** SAME THING BETWEEN CHILD AND GRAND-CHILD
C *** IN THE CHILD WE BLOCK PTS PRODUCING GRAND-CHILD WHEN ITS VOLUME FRACTION REACHED THRES4
c *** AND ALL THE OTHER TWIN SYSTEMS ARE STOPPED WHEN THE SUM OF THEIR TWIN VOLUME FRACTIONS REACHED 1
            PTSDAMP=1.
            IF(IPHEL.EQ.1) THEN
             if(ktwsmx(kkk).gt.0) then
              IF(IS.GT.NSLSYP) THEN
                 TAUGND=0.
                 if(is.eq.ipts) then
                   PTSDAMP=1.0+(TWFRPTS/TWTHRES(2,IPTM,1))**20
                   if(twfrgrain.gt.0.95) PTSDAMP=1.+(twfrgrain/0.9)**80!MZ changed
                 else
                   PTSDAMP=1.+ (twfrgrain/0.95)**80  !MZ changed
                 ENDIF
              ENDIF
             endif
            ENDIF
      !MZ uncommented following part            
            IF(IPHTOP.GT.2) THEN
              IF(IPHEL.EQ.2) THEN
               if(iptsgr_MZ(kkk).gt.0) then !MZ changed from iptsgr(kkk) to iptsgr_MZ(kkk)
                if(ktwsmx(kkk).eq.0) then
                   ipts_mz=iptsgr_MZ(kkk) !MZ changed from iptsgr(kkk) to iptsgr_MZ(kkk)
                   iptm=iptmgr_MZ(kkk) !MZ changed from iptmgr(kkk) to iptmgr_MZ(kkk)
                endif
                IF(IS.GT.NSLSYP) THEN
                  TAUGND=0.
                  !MZ identify twin mode        
                  !MZ_U - identify mode 
                  ns=IS          
                  do imo=1,NMODES(IPHEL)
                      ns=ns-NSM(imo,IPHEL)
                      if (ns.le.0) then
                        JTWIN=imo
                         EXIT
                      endif
                  enddo 
                  JTWIN=JTWIN-NSLMOD(IPHEL)
                  !END
                  IF(IS.EQ.IPTS_mz) THEN
                    PTSDAMP=1+(twfrsy(ipts_mz-nslsyP,kkk)/
     #               TWTHRES(4,IPTM,IPHEL))**20
                     if(twfrgrain.gt.0.9) PTSDAMP=1.
     #                +(twfrgrain/0.9)**80  !MZ changed                    
                  ELSE
                   PTSDAMP=1.+ (twfrsy(is-nslsyP,kkk)
     #              /TWTHRES(4,JTWIN,IPHEL))**80  !MZ changed
                  ENDIF
                ENDIF
               endif
              ENDIF     
            ENDIF
      !END       

            CRSS(IS,KKK)=PTSDAMP*TAUE(IS,KKK)+TAUHP
           
   
 
c           if (is.eq.20.and.kkk.eq.30) then 
c            write(10,*) is, 'grain 1329'
c            write(10,*) crss(is,1329),',',taue(is,1329),',',tauhp
c            write(10,*) is, 'grain 1972'
c            write(10,*) crss(is,1972),',',taue(is,1972),',',tauhp            
c           end if 

c           if (is.eq.1.and.kgx.eq.3000) then 
c           write(10,*) 'kgx'
c           write(10,*) crss(is,kgx),',',taue(is,kgx),',',tauhp
c           end if 


c            if(iprx.eq.1) then
c              write(*,'('' kkk is iptm ipts rspce'',10x,4i5,2f9.1)')
c     #                   kkk,is,iptm,ipts,SYSMFP(is,ipts-nslsys)
c              write(*,'(''    twthr1 twthr2 twfrpts barf'',5x,4f9.1)')
c     #                        th1x,th2x,twfrpts,barf
c              write(*,'(''    crss  hpfac dmfp tauhp'',5x,4f9.1)')
c     #              crss(is,kkk),hpfac(is,iphel),dmfp(is),tauhp
c              pause
c            endif
C *********************************************************************

          ENDDO      ! END OF DO LOOP OVER IS=1,NSYST
          GTOTGR(KKK)=GAMTOTX+DELTGAM
        ENDIF      ! END OF IF WGT(KKK)>0

      if(iprx.eq.1) then
c         write(10,'('' kkk dmfp(is)'',i5,6x,3f6.1,/,(12f6.1))')
c    #                  kkk,(dmfp(is),is=1,nsyst(iphel))
c         write(10,'('' TAUE(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
c    #          kkk,(taue(is,kkk),is=1,nsyst(iphel))
          write(10,'('' CRSS(is,kkk)'',i5,6x,3f6.0,/,(12f6.0))')
     #          kkk,(crss(is,kkk),is=1,nsyst(iphel))
          gamdnorm=tnorm5(dbar,5,1)
          write(10,'('' GAMD(is,kkk)'',i5,6x,3f7.2,/,(12f7.2))')
     #          kkk,(gamdot(is,kkk)/gamdnorm,is=1,nsyst(iphel))
      endif
c *** ADDEDMZ
c     Here certain values can be calculated.

c     (1) Calculation of disloc dens per mode per grain.
      temp01 = 0.
      temp02 = 0.
      do i=1,3
          temp01 = temp01 + RHO_S(i,KGX)
      enddo
      do i=4,15
          temp02 = temp02 + RHO_S(i,KGX)
      enddo      
      !MZ_ran_tw
      do is=1,39
        if (is.gt.15) then
c          CRSS(is,KGX)=CRSS(is,KGX)+r8_normal_01 ( iseed2 )*fluct
c          if (CRSS(is,KGX).lt.20.0) CRSS(is,KGX)=20.0
        endif
      enddo
        KGX=KGX+1
        ENDDO      ! END OF LOOP OVER GRAINS IN PHASE 'IPH'
        ENDDO      ! END OF LOOP OVER PHASES
      ENDIF      ! END OF IF(IOPTION.EQ.2)

      !MZ_FORCE_BASAL
            !for foreced basal in parents or twins
c      iforce=0
c      if(iforce.eq.1.or.iforce.eq.3) then
c          do KKK=1,NGR(1)
c              do is=16,18
c                  CRSS(is,KKK)=CRSS(is,KKK)*100.0
c              enddo
c          enddo
c      endif
c      if(iforce.eq.2.or.iforce.eq.3) then
c          do KKK=NGR(1)+1,NGR(2)
c              do is=16,18
c                  CRSS(is,KKK)=CRSS(is,KKK)*100.0
c              enddo
c          enddo
c      endif
      !MZ_FORCE_BASAL END
      
      RETURN
      END
